////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPArray.h
//    Cross platform array class.
//
//  Copyright © 2009-2013 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  09/04/13  B.D.D.  Added IndexOfObject.
//  07/30/13  B.D.D.  Added MutableArrayAtIndex, MutableDataAtIndex, MutableDictionaryAtIndex.
//  04/20/12  K.L.S.  Reworked and reworded documentation code.
//  03/15/12  G.R.W.  Added documentation to generate .docset files
//  02/24/12  G.R.W.  Extended the class with a mutable class and moved setter/change/remove methods to it.
//  02/10/12  K.L.S.  Changes for CrossPlatform API changes. Reworked header includes.
//	10/20/09  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/ExternalAPIMacros.h"
#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/CPObject.h"
#include "CrossPlatform/CPUInteger.h"

class CPMutableArray;
class CPBool;
class CPData;
class CPMutableData;
class CPValue;
class CPNumber;
class CPDate;
class CPString;
class CPDictionary;
class CPMutableDictionary;
class CPEnumerator;
class CPRetainedObject;
class CPError;

/// @brief   Cross platform array class.
/// @details CPArray and its subclass CPMutableArray manage ordered collections of objects called arrays. CPArray creates static arrays, 
///          and CPMutableArray creates dynamic arrays. You can use arrays when you need an ordered collection of objects.
///
///          After instantiation, these constructors & initializer methods return an array containing the values you pass in as arguments. 
///          Note that arrays can’t contain NULL pointers; in most cases, though, you can use an instance of CPNull instead. 
///
///          Notes: Objects stored in the array begin at index values starting at 0.
/// 
/// For more information:
/// http://developer.apple.com/library/mac/#documentation/Cocoa/Reference/Foundation/Classes/NSArray_Class
/// http://developer.apple.com/library/mac/#documentation/CoreFoundation/Reference/CFArrayRef
///
class CP_EXTERNAL_API CPArray : public CPObject
{
public:
	///	@brief     A cla3ss type that uniquely identifies this class.
	///	@return    Returns a class type that uniquely identifies this class.
	static CPClassType ClassType(void);

	/// @brief    Returns the underlying shared empty object.
	/// @details  Static method that returns the underlying object that is shared between all empty objects of this class type.
	/// @returns  the underlying shared empty object.
	static const CPId& SharedEmptyObject(void);

	///	@brief     Return an array instance containing one element which is the object passed in.
	///	@details   Creates and returns an array containing the given object as the first element.
	///	@param     Object  An object to add to the created array.
	///	@return    an array with the object as the first element.
	static CPArray ArrayWithObject(const CPObject& Object);

	static CPArray ArrayFromObject(const CPObject& Object);
	static CPArray ArrayFromId(const CPId& Object);

	/// @brief     Default Constructor.
	/// @details   Creates an empty array.
	CPArray(void);
	
	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the specified array.
	///	@param     Array  the array to copy.
	CPArray(const CPArray& Array);

	/// @brief     Creates an array object containing the objects passed in.
	/// @details   This object contains the object that were passed into the constructor.
	///	@param     Object  the object to copy.
	explicit CPArray(const CPObject& Object);
	
	///	@brief     Destructor.
	///	@details   Removes any references it has to shared resources.
	~CPArray(void);
	
	///	@brief     Initializes itself so that it contains the given object.
	///	@details   Initializes itself so that it contains the given object. Any previous contents are emptied.
	///	@param     Object  the object used to initialize the array.
	void InitWithObject(const CPObject& Object);
	
	///	@brief     Copy Operator.
	///	@details   Copies the elements from the given array.
	///	@param	   Array  the array to copy.
	const CPArray& operator=(const CPArray& Array);
	
	///	@brief     Assignment of an array object to another array object.
	///	@details   Assigns an array object to another array object.
	///	@param     Array  An underlying object to copy.
	const CPArray& operator=(const CPObject& Array);

	///	@brief     Returns whether or not elements are in the array.
	///	@return    true if elements are not contained, otherwise false.
	bool IsEmpty(void) const;
	
	///	@brief     Returns the number of elements in the array.
	///	@return    the number of elements in the array.
	CPUInteger Count(void) const;
	
	///	@brief     Returns true if the object is the class type for the specified index, otherwise false.
	///	@details   Checks to see if the existing object for the passed index in the array matches the passed class type.  If so, then 
	///            true is returned, otherwise false.
	///	@param     ClassTypeOfObject  A string of the class type of the object to be checked.
	///	@param     Index  A number used as the index into the array for its object.
	///	@return    true if the class type matches, otherwise false.
	bool IsMatchingClassTypeOfObjectAtIndex(CPClassType ClassTypeOfObject, CPUInteger Index) const;
	
	///	@brief     Returns the object type at the index.
	///	@details   Returns the object type as a string at the specified index in the array.
	///	@param     Index  A number used as the index into the array for its object.
	///	@return    the string identifying the object type for the object in the array.
	CPClassType ClassTypeOfObjectAtIndex(CPUInteger Index) const;
	
	///	@brief     Gets the object at the specified index.
	///	@details   This retrieves the object at the specified index in the array.  Note that indexes start at 0.
	///	@param     Index  An unsigned integer which is the index into the array.
	///	@return    The object that was in the array.
	CPObject ObjectAtIndex(CPUInteger Index) const;
	
	///	@brief     Returns the array at the specified index.
	///	@details   Returns the array at the specified index. If the object at the specified index is not an array, then an empty array is
	///            returned.
	///	@param     Index  An index within the bounds of the array.
	///	@return    The array located at the specified index.
	CPArray ArrayAtIndex(CPUInteger Index) const;
	CPArray ArrayAtIndex(CPUInteger Index, CPError& Error) const;

	///	@brief     Returns the array at the specified index.
	///	@details   Returns the array at the specified index. If the object at the specified index is not an array, then an empty array is
	///            returned.
	///	@param     Index  An index within the bounds of the array.
	///	@return    The array located at the specified index.
	CPMutableArray MutableArrayAtIndex(CPUInteger Index) const;
	CPMutableArray MutableArrayAtIndex(CPUInteger Index, CPError& Error) const;

	///	@brief     Returns the boolean at the specified index.
	///	@details   Returns the boolean at the specified index. If the object at the specified index is not a boolean, then an empty boolean
	///            is returned. Several objects will convert to a boolean, they are a string  (true, false, YES, NO) and a number. Note that
	///            the string value comparisons are not case sensitive.
	///	@param     Index  An index within the bounds of the array.
	///	@return    The boolean located at the specified index.
	CPBool BoolAtIndex(CPUInteger Index) const;
	CPBool BoolAtIndex(CPUInteger Index, CPError& Error) const;

	///	@brief     Returns the data at the specified index.
	///	@details   Returns the data at the specified index. If the object at the specified index is not data, then an empty data is 
	///            returned.
	///	@param     Index  An index within the bounds of the array.
	///	@return    The data located at the specified index.
	CPData DataAtIndex(CPUInteger Index) const;
	CPData DataAtIndex(CPUInteger Index, CPError& Error) const;
	
	///	@brief     Returns the data at the specified index.
	///	@details   Returns the data at the specified index. If the object at the specified index is not data, then an empty data is 
	///            returned.
	///	@param     Index  An index within the bounds of the array.
	///	@return    The data located at the specified index.
	CPMutableData MutableDataAtIndex(CPUInteger Index) const;
	CPMutableData MutableDataAtIndex(CPUInteger Index, CPError& Error) const;
	
	///	@brief     Returns the date at the specified index.
	///	@details   Returns the date at the specified index. If the object at the specified index is not a date, then an empty date is 
	///            returned.
	///	@param     Index  An index within the bounds of the array.
	///	@return    The date located at the specified index.
	CPDate DateAtIndex(CPUInteger Index) const;
	CPDate DateAtIndex(CPUInteger Index, CPError& Error) const;
	
	///	@brief     Returns the dictionary at the specified index.
	///	@details   Returns the dictionary at the specified index. If the object at the specified index is not a dictionary, then an empty 
	///            dictionary is returned.
	///	@param     Index  An index within the bounds of the array.
	///	@return    The dictionary located at the specified index.
	CPDictionary DictionaryAtIndex(CPUInteger Index) const;
	CPDictionary DictionaryAtIndex(CPUInteger Index, CPError& Error) const;
	
	///	@brief     Returns the dictionary at the specified index.
	///	@details   Returns the dictionary at the specified index. If the object at the specified index is not a dictionary, then an empty 
	///            dictionary is returned.
	///	@param     Index  An index within the bounds of the array.
	///	@return    The dictionary located at the specified index.
	CPMutableDictionary MutableDictionaryAtIndex(CPUInteger Index) const;
	CPMutableDictionary MutableDictionaryAtIndex(CPUInteger Index, CPError& Error) const;

	///	@brief     Returns the number at the specified index.
	///	@details   Returns the number at the specified index. If the object at the specified index is not a number, then an empty number is
	///            returned.
	///	@param     Index  An index within the bounds of the array.
	///	@return    The number located at the specified index.
	CPNumber NumberAtIndex(CPUInteger Index) const;
	CPNumber NumberAtIndex(CPUInteger Index, CPError& Error) const;
	
	///	@brief     Returns the retained object at the specified index.
	///	@details   Returns the retained object at the specified index. If the object at the specified index is not a retained object, then a
	///            NULL pointer is returned.
	///	@param     Index  An index within the bounds of the array.
	///	@return    The retained object located at the specified index.
	CPRetainedObject* RetainedObjectAtIndex(CPUInteger Index) const;
	CPRetainedObject* RetainedObjectAtIndex(CPUInteger Index, CPError& Error) const;

	///	@brief     Returns the string at the specified index.
	///	@details   Returns the string at the specified index. If the object at the specified index is not a string, then an empty string is
	///            returned.
	///	@param     Index  An index within the bounds of the array.
	///	@return    The string located at the specified index.
	CPString StringAtIndex(CPUInteger Index) const;
	CPString StringAtIndex(CPUInteger Index, CPError& Error) const;
	
	///	@brief     Returns the value at the specified index.
	///	@details   Returns the value at the specified index. If the object at the specified index is not a value, then an empty value is
	///            returned.
	///	@param     Index  An index within the bounds of the array.
	///	@return    The value located at the specified index.
	CPValue ValueAtIndex(CPUInteger Index) const;
	CPValue ValueAtIndex(CPUInteger Index, CPError& Error) const;

	///	@brief     Returns an enumerator object that lets you access each object in the array.
	///	@details   Returns an enumerator object that lets you access each object in the array, in order, starting with the element at index 0.
	///            When you use this method with mutable subclasses of NSArray, you must not modify the array during enumeration.
	///	@return    An enumerator object that lets you access each object in the array, in order, from the element at the lowest index upwards.
	CPEnumerator ObjectEnumerator(void) const;

	///	@brief     Returns an enumerator object that lets you access each object in the array.
	///	@details   Returns an enumerator object that lets you access each object in the array, in order, starting with the last element.
	///            When you use this method with mutable subclasses of NSArray, you must not modify the array during enumeration.
	///	@return    An enumerator object that lets you access each object in the array, in order, from the element at the highest index down
	///            to the element at index 0.
	CPEnumerator ReverseObjectEnumerator(void) const;

	///	@brief     Returns the index of the first occurance found of the object equal to the object passed in.
	///	@details   Returns the index of the first occurance found of the object equal to the object passed in. It will return CPNotFound if
	///            no occurances were found.
	///	@param     Object  An object to find in the array.
	///	@return    the index of the first occurance found of the object equal to the object passed in.
	CPUInteger IndexOfObject(const CPObject& Object) const;

	///	@brief     Returns the index of the first occurance found of the object matching the id of the object passed in.
	///	@details   Returns the index of the first occurance found of the object matching the id of the object passed in. It will return CPNotFound if
	///            no occurances were found.
	///	@param     Object  An object to find in the array.
	///	@return    the index of the first occurance found of the object matching the id of the object passed in.
	CPUInteger IndexOfObjectIdenticalTo(const CPObject& Object) const;
	
	///	@brief     Returns whether or not the element is contained in the array.
	///	@details   Returns whether or not the element is contained in the array.
	///	@param     Object An object to find in the array.
	///	@return    true if element is contained in the array, otherwise false.
	bool Contains(const CPObject& Object) const;
	
	///	@brief     Returns a string containing the elements of the array separated by the specified seperator.
	///	@details   Returns a string containing the elements of the array separated by the specified seperator. An array that contains elements
	///            that are not strings will produce unknown results. If the array is empty, an empty string is returned.
	///            Eg. array("red", "green", "blue"); array.ComponentsJoinedByString(" | "); will return a string = "red | green | blue"
	///	@param     Separator  The string to seperate the elements of the array.
	///	@return    a string that contains the elements of the array separated by the specified seperator.
	CPString ComponentsJoinedByString(const CPString& Separator) const;

#if __OBJC__
	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the specified array.
	///	@param     Array  the array to copy.
	CPArray(NSArray* Array);
	
	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the specified array.
	///	@param     Array  the array to copy.
	CPArray(NSMutableArray* Array);
	
	///	@brief     Copy Operator.
	///	@details   Copies the elements from the given array.
	///	@param	   Array  the array to copy.
	const CPArray& operator=(NSArray* Array);
	
	///	@brief     Copy Operator.
	///	@details   Copies the elements from the given array.
	///	@param	   Array  the array to copy.
	const CPArray& operator=(NSMutableArray* Array);

	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as an NSArray*.
	operator NSArray*(void) const;
#endif

#if __COREFOUNDATION__
	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the specified array.
	///	@param     Array  the array to copy.
	CPArray(CFArrayRef Array);
	
	///	@brief     Copy Operator.
	///	@details   Copies the elements from the given array.
	///	@param	   Array  the array to copy.
	const CPArray& operator=(CFArrayRef Array);

	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as a CFArrayRef*.
	operator CFArrayRef(void) const;

	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as a CFPropertyListRef*.
	operator CFPropertyListRef(void) const;
#endif

protected:
	CPId InternalObjectAtIndex(CPUInteger Index) const;
	
	CPArray(const CPId& OtherObject, CPClassType ClassType);
	CPArray(const CPObject& OtherObject, CPClassType ClassType);

	void AssignUnderlyingArray(const CPId& Array);
	void AssignUnderlyingArrayAndTakeOwnership(const CPId& Array);
};

#include "CrossPlatform/CPBool.h"
#include "CrossPlatform/CPString.h"
#include "CrossPlatform/CPData.h"
#include "CrossPlatform/CPDictionary.h"
#include "CrossPlatform/CPDate.h"
#include "CrossPlatform/CPValue.h"
#include "CrossPlatform/CPNumber.h"
