////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPAssertLog.h
//    Cross platform assert macros.
//
//  Copyright © 2009-2012 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  02/10/12  K.L.S.  Changes for CrossPlatform API changes. Reworked header includes.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Copyright © 2008-2009 MYOB Technology Pty Ltd., all rights reserved.
//
//  Change History (most recent first):
//
//  02/10/12  K.L.S.  Changes for CrossPlatform API changes. Reworked header includes.
//	07/16/08  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/// @brief   Macro definitions for asserts
/// @details Macros used to assert a condition. To be used when the assert should only be performed in a debug build. This header relies
///          on the _DEBUG macro to enable the asserting. Make sure _DEBUG is defined in debug build to allow use.
///
#pragma once

#include "CrossPlatform/CPOS_API.h"

///
///	@brief     Test that a condition is true
///
///	@details   Will test a condition and on false will output to the console log, will break program execution. This was named with "CP" prefix
///            so that it avoids the generic ASSERT macro that is defined so often.
///            Output will look like: Failed Assert! ("e")    Function:-[mainWindowController awakeFromNib] File:mainWindowController.m Line:31
///
/// @param     e the condition to test.
#ifdef _MAC
#define CPASSERT(e) ((void) ((e) ? 0 : ((void)NSLog((@"Failed Assert! (%s);    Function:%s File:%@ Line:%d"), #e, __func__, [@__FILE__ lastPathComponent], __LINE__), assert(e), 0)))
#else
#include <assert.h>
#define CPASSERT(e) ((void) ((e) ? 0 : (CPLog("Failed Assert! (%s)    Function:%s File:%s Line:%d", #e, __FUNCTION__, __FILE__, __LINE__), assert(e), 0)))
#endif

///
///	@brief     Make sure a condition is true
////
///	@details   Will test a condition and on false will output to the console log, will NOT break program execution in Release build.
///            Output will look like: Failed Assert! ("e")    Function:-[mainWindowController awakeFromNib] File:mainWindowController.m Line:31
///
/// @param     e the condition to test.
///
#ifdef _DEBUG
#define CPASSERT_LOG(e) CPASSERT(e)
#else
#ifdef _MAC
#define CPASSERT_LOG(e)  ((void) ((e) ? 0 : ((void)NSLog((@"Failed Assert! (%s);    Function:%s File:%@ Line:%d"), #e, __func__, [@__FILE__ lastPathComponent], __LINE__), 0)))
#else
#define CPASSERT_LOG(e) ((void) ((e) ? 0 : (CPLog("Failed Assert! (%s)  Function:%s File:%s Line:%d", #e, __FUNCTION__, __FILE__, __LINE__), 0)))
#endif
#endif

///
///	@brief     Make sure a condition is true
///
///	@details   Will test a condition and on false will output to the console log, will NOT break program execution. Allows you to put an 
///            addition message in the console output. Only works in debug build when DEBUG defined.  Output will look like: Failed
///            Assert! ("e") - "m"    Function:-[mainWindowController awakeFromNib] File:mainWindowController.m Line:31
///
/// @param     e the condition to test.
/// @param     m the message to add to the log output.
///
#ifdef _DEBUG
#ifdef _MAC
#define CPASSERT_LOG_WITH_MESSAGE(e,m) ((void) ((e) ? 0 : ((void)NSLog((@"Failed Assert! (%s) - %s    Function:%s File:%@ Line:%d"), #e, m, __func__, [@__FILE__ lastPathComponent], __LINE__), assert(e), 0)))
#else
#define CPASSERT_LOG_WITH_MESSAGE(e,m) ((void) ((e) ? 0 : (DebugLog("Failed Assert! (%s) - %s    Function:%s File:%s Line:%d", #e, m, __FUNCTION__, __FILE__, __LINE__), assert(e), 0)))
#endif
#else
#ifdef _MAC
#define CPASSERT_LOG_WITH_MESSAGE(e,m) ((void) ((e) ? 0 : ((void)NSLog((@"Failed Assert! (%s) - %s    Function:%s File:%@ Line:%d"), #e, m, __func__, [@__FILE__ lastPathComponent], __LINE__), 0)))
#else
#define CPASSERT_LOG_WITH_MESSAGE(e,m) ((void) ((e) ? 0 : (CPLog("Failed Assert! (%s) - %s    Function:%s File:%s Line:%d", #e, m, __FUNCTION__, __FILE__, __LINE__), 0)))
#endif
#endif
///
///	@brief     Make sure a condition is true
///
///	@details   Will test a condition and on false will output to the console log, will NOT break program execution. Allows you to put an 
///            addition message in the console output. Only works in debug build when DEBUG defined.  Output will look like: Failed
///            Assert! ("e") - "m"    Function:-[mainWindowController awakeFromNib] File:mainWindowController.m Line:31
///
/// @param     e the condition to test.
/// @param     m the message to add to the log output.
///
#ifdef _DEBUG
#ifdef _MAC
#define CPASSERT_LOG_WITH_OBJECT(e,m) ((void) ((e) ? 0 : ((void)NSLog((@"Failed Assert! (%s) - %@    Function:%s File:%@ Line:%d"), #e, m, __func__, [@__FILE__ lastPathComponent], __LINE__), assert(e), 0)))
#else
#define CPASSERT_LOG_WITH_OBJECT(e,m) ((void) ((e) ? 0 : (DebugLog("Failed Assert! (%s) - %@    Function:%s File:%s Line:%d", #e, m, __FUNCTION__, __FILE__, __LINE__), assert(e), 0)))
#endif
#else
#ifdef _MAC
#define CPASSERT_LOG_WITH_OBJECT(e,m) ((void) ((e) ? 0 : ((void)NSLog((@"Failed Assert! (%s) - %@    Function:%s File:%@ Line:%d"), #e, @#m, __func__, [@__FILE__ lastPathComponent], __LINE__), 0)))
#else
#define CPASSERT_LOG_WITH_OBJECT(e,m) ((void) ((e) ? 0 : (CPLog("Failed Assert! (%s) - %@    Function:%s File:%s Line:%d", #e, m, __FUNCTION__, __FILE__, __LINE__), 0)))
#endif
#endif
