////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPBool.h
//    Cross-platform boolean class.
//
//  Copyright © 2009-2012 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  02/10/12  K.L.S.  Changes for CrossPlatform API changes. Reworked header includes.
//	10/20/09  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/ExternalAPIMacros.h"
#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/CPObject.h"

class CPString;

/// @brief    Class encapsulation of a bool.
/// @details  CPBool is a class that offers a logical true or false in cases where a boolean is needed.  It defines a set of methods 
///           specifically for setting and accessing the boolean.  It also defines various copy and equallity methods along with 
///           "is equal to" and "is not equal to" that support a wide vaiety of parameters.
class CP_EXTERNAL_API CPBool : public CPObject
{
public:
	///	@brief     A cla3ss type that uniquely identifies this class.
	///	@return    Returns a class type that uniquely identifies this class.
	static CPClassType ClassType(void);

	/// @brief    Returns the underlying shared empty object.
	/// @details  Static method that returns the underlying object that is shared between all empty objects of this class type.
	/// @returns  the underlying shared empty object.
	static const CPId& SharedEmptyObject(void);

	///	@brief     Static bool object set to False.
	static const CPBool False(void);
	
	///	@brief     Static bool object set to True.
	static const CPBool True(void);
	
	///	@brief     Static bool object set to No.
	static const CPBool No(void);
	
	///	@brief     Static bool object set to Yes.
	static const CPBool Yes(void);

	//GRW: Ken said this goes away (5/8/12)
	static CPBool BoolFromObject(const CPObject& Object);
	static CPBool BoolFromId(const CPId& Object);

	
	/// @brief     Default Constructor.
	/// @details   Creates an empty bool object.
	CPBool(void);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the bool that was passed into the constructor.
	///	@param     Value  the bool to copy.
	CPBool(bool Value);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the bool representation of the character that was passed into the constructor.
	///	@param     Value  the character to copy.
	CPBool(char Value);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the bool representation of the unsigned character that was passed into the constructor.
	///	@param     Value  the unsigned character to copy.
	CPBool(unsigned char Value);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the bool representation of the short that was passed into the constructor.
	///	@param     Value  the short to copy.
	CPBool(short Value);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the bool representation of the unsigned short that was passed into the constructor.
	///	@param     Value  the unsigned short to copy.
	CPBool(unsigned short Value);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the bool representation of the integer that was passed into the constructor.
	///	@param     Value  the integer to copy.
	CPBool(int Value);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the bool representation of the unsigned integer that was passed into the constructor.
	///	@param     Value  the unsigned integer to copy.
	CPBool(unsigned int Value);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the bool representation of the long that was passed into the constructor.
	///	@param     Value  the long to copy.
	CPBool(long Value);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the bool representation of the unsigned long that was passed into the constructor.
	///	@param     Value  the unsigned long to copy.
	CPBool(unsigned long Value);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the bool representation of the bool object that was passed into the constructor.
	///	@param     Value  the bool object to copy.
	CPBool(const CPBool& Value);
	
	///	@brief     Destructor.
	///	@details   Removes any references it has to shared resources.
	~CPBool(void);

	
	///	@brief     Copy Operator.
	///	@details   Copies the bool representation of the given character.
	///	@param	   Value  the character to copy.
	const CPBool& operator=(char Value);
	
	///	@brief     Copy Operator.
	///	@details   Copies the bool representation of the given unsigned character.
	///	@param	   Value  the unsigned character to copy.
	const CPBool& operator=(unsigned char Value);
	
	///	@brief     Copy Operator.
	///	@details   Copies the bool representation of the given short.
	///	@param	   Value  the short to copy.
	const CPBool& operator=(short Value);
	
	///	@brief     Copy Operator.
	///	@details   Copies the bool representation of the given unsigned short.
	///	@param	   Value  the unsigned short to copy.
	const CPBool& operator=(unsigned short Value);
	
	///	@brief     Copy Operator.
	///	@details   Copies the bool representation of the given integer.
	///	@param	   Value  the integer to copy.
	const CPBool& operator=(int Value);
	
	///	@brief     Copy Operator.
	///	@details   Copies the bool representation of the given unsigned integer.
	///	@param	   Value  the unsigned integer to copy.
	const CPBool& operator=(unsigned int Value);
	
	///	@brief     Copy Operator.
	///	@details   Copies the bool representation of the given long.
	///	@param	   Value  the long to copy.
	const CPBool& operator=(long Value);
	
	///	@brief     Copy Operator.
	///	@details   Copies the bool representation of the given unsigned long.
	///	@param	   Value  the unsigned long to copy.
	const CPBool& operator=(unsigned long Value);
	
	///	@brief     Copy Operator.
	///	@details   Copies the bool of the given bool.
	///	@param	   Value  the bool to copy.
	const CPBool& operator=(bool Value);
	
	///	@brief     Copy Operator.
	///	@details   Copies the bool from the given boolean object.
	///	@param	   Value  the boolean object to copy.	
	const CPBool& operator=(const CPBool& Value);

	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the bool is equal to the given bool.
	/// @param	   Value  The bool used in the compare.
	/// @return    A bool set to true if the bool is the same, otherwise it is false.
	bool operator==(bool Value) const;

	/// @brief     Equals Operator.
	/// @details   This checks to see if the bool is equal to the given bool object.
	/// @param	   Value  The bool object used in the compare.
	/// @return    A bool set to true if the bool is the same, otherwise it is false.
	bool operator==(CPBool Value) const;
	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the bool is equal to the given character.
	/// @param	   Value  The character used in the compare.
	/// @return    A bool set to true if the bool is the same, otherwise it is false.
	bool operator==(char Value) const;
	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the bool is equal to the given unsigned character.
	/// @param	   Value  The unsigned character used in the compare.
	/// @return    A bool set to true if the bool is the same, otherwise it is false.
	bool operator==(unsigned char Value)const;
	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the bool is equal to the given short.
	/// @param	   Value  The short used in the compare.
	/// @return    A bool set to true if the bool is the same, otherwise it is false.
	bool operator==(short Value) const;
	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the bool is equal to the given unsigned short.
	/// @param	   Value  The unsigned short used in the compare.
	/// @return    A bool set to true if the bool is the same, otherwise it is false.
	bool operator==(unsigned short Value) const;
	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the bool is equal to the given integer.
	/// @param	   Value  The integer used in the compare.
	/// @return    A bool set to true if the bool is the same, otherwise it is false.
	bool operator==(int Value) const;
	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the bool is equal to the given unsigned integer.
	/// @param	   Value  The unsigned integer used in the compare.
	/// @return    A bool set to true if the bool is the same, otherwise it is false.
	bool operator==(unsigned int Value) const;
	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the bool is equal to the given long.
	/// @param	   Value  The long used in the compare.
	/// @return    A bool set to true if the bool is the same, otherwise it is false.
	bool operator==(long Value) const;
	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the bool is equal to the given unsigned long.
	/// @param	   Value  The unsigned long used in the compare.
	/// @return    A bool set to true if the bool is the same, otherwise it is false.
	bool operator==(unsigned long Value) const;

	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the bool is not equal to the given bool.
	/// @param	   Value  The bool used in the compare.
	/// @return    A bool set to true if the bool is different, otherwise it is false.
	bool operator!=(bool Value) const;
	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the bool is not equal to the given bool object.
	/// @param	   Value  The bool object used in the compare.
	/// @return    A bool set to true if the bool is different, otherwise it is false.
	bool operator!=(CPBool Value) const;
	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the bool is not equal to the given character.
	/// @param	   Value  The character used in the compare.
	/// @return    A bool set to true if the bool is different, otherwise it is false.
	bool operator!=(char Value) const;
	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the bool is not equal to the given unsigned character.
	/// @param	   Value  The unsigned character used in the compare.
	/// @return    A bool set to true if the bool is different, otherwise it is false.
	bool operator!=(unsigned char Value)const;
	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the bool is not equal to the given short.
	/// @param	   Value  The short used in the compare.
	/// @return    A bool set to true if the bool is different, otherwise it is false.
	bool operator!=(short Value) const;
	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the bool is not equal to the given unsigned short.
	/// @param	   Value  The unsigned short used in the compare.
	/// @return    A bool set to true if the bool is different, otherwise it is false.
	bool operator!=(unsigned short Value) const;
	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the bool is not equal to the given integer.
	/// @param	   Value  The integer used in the compare.
	/// @return    A bool set to true if the bool is different, otherwise it is false.
	bool operator!=(int Value) const;
	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the bool is not equal to the given unsigned integer.
	/// @param	   Value  The unsigned integer used in the compare.
	/// @return    A bool set to true if the bool is different, otherwise it is false.
	bool operator!=(unsigned int Value) const;
	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the bool is not equal to the given long.
	/// @param	   Value  The long used in the compare.
	/// @return    A bool set to true if the bool is different, otherwise it is false.
	bool operator!=(long Value) const;
	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the bool is not equal to the given unsigned long.
	/// @param	   Value  The unsigned long used in the compare.
	/// @return    A bool set to true if the bool is different, otherwise it is false.
	bool operator!=(unsigned long Value) const;

	
	///	@brief     Retrieves the bool.
	///	@return    A bool representation of the bool object.
	operator bool(void) const;

	/// @brief     Negative Operator.
	/// @details   Gives the opposite of the bool object.
	/// @return    A bool set to true if the bool object is false, otherwise it is false.
	bool operator!(void) const;

#ifdef __OBJC__
	/// @brief     Copy Constructor.
	/// @details   This object contains the bool that was passed into the constructor as BOOL.
	///	@param     Value  the BOOL to copy.
	CPBool(BOOL Value);
	
	///	@brief     Copy Operator.
	///	@details   Copies the bool of the given BOOL.
	///	@param	   Value  the BOOL to copy.
	const CPBool& operator=(BOOL Value);
	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the bool is equal to the given BOOL.
	/// @param	   Value  The BOOL used in the compare.
	/// @return    A bool set to true if the BOOL is the same, otherwise it is false.
	bool operator==(BOOL Value) const;
	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the bool is not equal to the given BOOL.
	/// @param	   Value  The BOOL used in the compare.
	/// @return    A bool set to true if the BOOL is different, otherwise it is false.
	bool operator!=(BOOL Value) const;

	///	@brief     Retrieves the BOOL.
	///	@return    A BOOL representation of the bool object.
	operator BOOL(void) const;
#endif  // #ifdef __OBJC__
#if __COREFOUNDATION__
	// Define methods that will be needed only when bridging to CoreFoundation

	/// @brief     Copy Constructor.
	/// @details   This object contains the bool that was passed into the constructor as boolean reference.
	///	@param     Value  the boolean reference to copy.
	CPBool(CFBooleanRef Value);
	
	///	@brief     Copy Operator.
	///	@details   Copies the bool of the given boolean reference.
	///	@param	   Value  the boolean reference to copy.
	const CPBool& operator=(CFBooleanRef Value);

	///	@brief     Retrieves the boolean reference.
	///	@return    A boolean reference of the bool object.
	operator CFBooleanRef(void) const;
#endif

protected:
	CPBool(const CPId&);
	CPBool(const CPObject&);
	const CPBool& operator=(const CPObject&);
};

#ifndef NO
#define NO CPBool::No()
#endif
#ifndef YES
#define YES CPBool::Yes()
#endif

#include "CrossPlatform/CPString.h"
