////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPBundle.h
//    Cross-platform bundle class. A bundle is a logical grouping for a code and its associated resources.
//
//  Copyright © 2009-2012 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//	10/20/09  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/ExternalAPIMacros.h"
#include "CrossPlatform/CPObject.h"
#include "CrossPlatform/CPString.h"

class CPString;

/// @brief    Represents a file or file folder (bundle) of groups of code and resources used in a program. 
/// @details  An NSBundle object represents a location in the file system that groups code and resources that can be used in a program. 
///           CPBundle objects locate program resources, dynamically load and unload executable code, and assist in localization. You build
///           a bundle in Xcode using one of these project types: Application, Framework, plug-ins.
///
///           Although bundle structures vary depending on the target platform and the type of bundle you are building, the CPBundle class 
///           hides this underlying structure in most (but not all) cases. Many of the methods you use to load resources from a bundle
///           automatically locate the appropriate starting directory and look for resources in known places. 
///
/// For more information:
/// http://developer.apple.com/library/mac/#documentation/Cocoa/Reference/Foundation/Classes/NSBundle_Class
/// http://developer.apple.com/library/mac/#documentation/CoreFoundation/Reference/CFBundleRef
///
class CP_EXTERNAL_API CPBundle : public CPObject
{
public:
	///	@brief     A cla3ss type that uniquely identifies this class.
	///	@return    Returns a class type that uniquely identifies this class.
	static CPClassType ClassType(void);

	///	@brief     Return a bundle instance based on the application's location.
	///	@details   This static method returns the CPBundle object that corresponds to the directory where the current application executable
	///            is located.
	///	@return    This returns the bundle object.
	static const CPBundle& MainBundle(void);
	
	// @brief      Returns the hardcoded string for a localized string
	static const CPString& DefaultStringsTableName(void);

	/// @brief     Default Constructor.
	/// @details   Creates an empty bundle.
	CPBundle(void);
	
	/// @brief     Copy Constructor.
	/// @details   This will contain the object of the given bundle.
	///	@param     Bundle  the bundle to copy.
	CPBundle(const CPBundle& Bundle);
	
	///	@brief     Destructor.
	///	@details   Removes any references it has to shared resources.
	virtual ~CPBundle(void);

	///	@brief     Copy Operator.
	///	@details   Copies the given bundle.
	///	@param	   Bundle  the bundle to copy.
	const CPBundle& operator=(const CPBundle& Bundle);
	
	/// @brief     Returns the receiver’s bundle identifier.
	///	@return    This returns the receiver’s bundle identifier, which is defined by the CFBundleIdentifier key in the bundle’s information
	///            property list.
	CPString BundleIdentifier(void) const;
	
	/// @brief     Returns the file path of the receiver’s bundle directory.
	///	@return    The file path of the receiver’s bundle directory.
	CPString BundlePath(void) const;
	
	/// @brief     Returns the file path of the receiver’s executable file.
	///	@return    The file path of the receiver’s executable file.
	CPString ExecutablePath(void) const;
	
	/// @brief     Returns the file path of the receiving bundle’s resources directory.
	///	@return    The file path of the receiving bundle’s resources directory.
	CPString ResourcePath(void) const;

	/// @brief     Returns the file path to the directory that application is contained within. On Mac this is the path to where the bundle
	///            is contained.
	///	@return    The file path to the directory that application is contained within.
	CPString AppsDirectoryPath(void) const;

	/// @brief     Returns the file path of the application. This is the BundlePath on Mac and ExecutablePath on Windows.
	///	@return    The file path of the application, path to bundle on Mac.
	CPString AppPath(void) const;

	/// @brief     Returns the path of the applications directory for the bundled helper apps. This is the BundlePath/Contents/Applications
	///            on Mac and ExecutablePath on Windows.
	///	@return    the path of the applications directory for the bundled helper apps.
	CPString BundledApplicationsPath(void) const;

	/// @brief     Returns the path of the directory for the bundled helper apps. This is the BundlePath/Contents/Helpers on Mac and
	///            ExecutablePath on Windows.
	///	@return    the path of the directory for the bundled helper apps.
	CPString HelperAppsPath(void) const;

	/// @brief     Returns a dictionary that contains information about the receiver.
	///	@return    A dictionary, constructed from the bundle's Info.plist file, that contains information about the receiver. If the bundle
	///            does not contain an Info.plist file, a valid dictionary is returned but this dictionary contains only private keys that 
	///            are used internally by the CPBundle class. The CPBundle class may add extra keys to this dictionary for its own use.
	CPDictionary InfoDictionary(void) const;

	/// @brief     Returns an array of strings indicating the actual localizations contained in the receiver’s bundle.
	/// @return    An array of CPString objects, each of which identifies the a localization in the receiver’s bundle. The languages are in
	///            the preferred order.
	CPArray PreferredLocalizations(void) const;
	
	/// @brief     Returns the localization used to create the bundle.
	/// @return    The localization used to create the bundle.
	CPString DevelopmentLocalization(void) const;

	/// @brief     Returns a localized version of the string designated by the given key and residing in the table in
	///            Localizable.strings.
	/// @param     Key  The key for a string in the table identified by the table in Localizable.strings.
	/// @return    A localized version of the string designated by key in the table in Localizable.strings.
	CPString LocalizedStringForKey(const CPString& Key) const;
	
	/// @brief     Returns a localized version of the string designated by the given key and residing in the given table.
	/// @param     Key  The key for a string in the table identified by the table in Localizable.strings.
	/// @param     TableName  The receiver’s string table to search. If tableName is nil or is an empty string, the method attempts to use 
	///            the table in Localizable.strings.
	/// @return    A localized version of the string designated by key in the given table.  If the localized string is not found in the 
	///            table, returns key. If key and value are both nil, returns the empty string.
	CPString LocalizedStringForKeyInTableNamed(const CPString& Key, const CPString& TableName) const;
	
	/// @brief     Returns a localized version of the string designated by the given key and residing in the table in
	///            Localizable.strings.
	/// @param     Key  The key for a string in the table identified by the table in Localizable.strings.
	/// @param     DefaultValue  The value to return if key is nil or if a localized string for key can’t be found in the table.
	/// @return    A localized version of the string designated by key in the table in Localizable.strings.
	CPString LocalizedStringForKeyWithDefaultValue(const CPString& Key, const CPString& DefaultValue) const;

	/// @brief     Returns a localized version of the string designated by the given key and residing in the given table.
	/// @param     Key  The key for a string in the table identified by given TableName.
	/// @param     DefaultValue  The value to return if key is nil or if a localized string for key can’t be found in the table.
	/// @param     TableName  The receiver’s string table to search. If tableName is nil or is an empty string, the method attempts to use 
	///            the table in Localizable.strings.
	/// @return    A localized version of the string designated by key in table tableName. If value is nil or an empty string, and a 
	///            localized string is not found in the table, returns key. If key and value are both nil, returns the empty string.
	CPString LocalizedStringForKeyWithDefaultValueInTableNamed(const CPString& Key, const CPString& DefaultValue, const CPString& TableName) const;
	
	/// @brief     Returns the file path for the resource identified by the specified name and file extension.
	/// @param     ResourceName  The name of the resource file.
	/// @param     Extension  If extension is an empty string or nil, the extension is assumed not to exist and the file is the first file 
	///            encountered that exactly matches name.
	CPString PathForResourceOfType(CPString ResourceName, CPString Extension) const;

#ifdef __OBJC__
	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the specified bundle.
	///	@param     Bundle  the bundle to copy.
	CPBundle(const NSBundle* Bundle);
	
	///	@brief     Copy Operator.
	///	@details   Copies the bundle.
	///	@param	   Bundle  the bundle to copy.
	const CPBundle& operator=(const NSBundle* Bundle);
#else
protected:
	/// @brief     Copy Constructor.
	/// @details   Contains the bundle object of the specified bundle at the given file path.
	///	@param     BundlePath  the path to the bundle.
	explicit CPBundle(CPString BundlePath);

	///	@brief     Return the dictionary from the table given the table filePath.
	///	@param	   TablePath  A file path to the table.
	/// @return    Returns the dictionary associated with the given table.
	CPDictionary StringTableWithPath(const CPString& TablePath) const;

private:
	CPString m_ExecutablePath;
	CPString m_BundlePath;
	CPString m_ResourcePath;

	void SetBundlePath(CPString BundlePath);
#endif  // #ifdef __OBJC__
};
