////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPCalendar.h
//      Calendar for date manipulation.
//
//  Copyright © 2015 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  04/11/15  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/ExternalAPIMacros.h"
#include "CrossPlatform/CPObject.h"
#include "CrossPlatform/CPUInteger.h"
#include "CrossPlatform/CPTimeZone.h"
#include "CrossPlatform/CPDate.h"
#include "CrossPlatform/CPOptions.h"

class CPDateComponents;

typedef CP_OPTIONS(CPUInteger, CPCalendarUnit)
{
	/// year calendar unit identifier flag
	CPCalendarUnitYear              = (1UL << 2),
	/// month calendar unit identifier flag
	CPCalendarUnitMonth             = (1UL << 3),
	/// day calendar unit identifier flag
	CPCalendarUnitDay               = (1UL << 4),
	/// hour calendar unit identifier flag
	CPCalendarUnitHour              = (1UL << 5),
	/// minute calendar unit identifier flag
	CPCalendarUnitMinute            = (1UL << 6),
	/// second calendar unit identifier flag
	CPCalendarUnitSecond            = (1UL << 7),
	/// weekday calendar unit identifier flag
	CPCalendarUnitWeekday           = (1UL << 9),
	/// nanosecond calendar unit identifier flag
	CPCalendarUnitNanosecond        = (1UL << 15),
	/// year, month and day calendar unit identifier flags
	CPCalendarUnitYearMonthDay = CPCalendarUnitYear | CPCalendarUnitMonth | CPCalendarUnitDay,
	/// hour, minute, second and nanosecond calendar unit identifier flags
	CPCalendarUnitTime = CPCalendarUnitHour | CPCalendarUnitMinute | CPCalendarUnitSecond | CPCalendarUnitNanosecond,
	/// all calendar unit identifier flags
	CPCalendarUnitAll = CPCalendarUnitYearMonthDay | CPCalendarUnitTime | CPCalendarUnitWeekday,
};

class CP_EXTERNAL_API CPCalendar : public CPObject
{
public:
	///	@brief     A class type that uniquely identifies this class.
	///
	///	@return    Returns a class type that uniquely identifies this class.
	///
	static CPClassType ClassType(void);

	/// @brief    Returns the underlying shared empty object.
	///
	/// @details  Static method that returns the underlying object that is shared between all empty objects of this class type.
	///
	/// @returns  the underlying shared empty object.
	///
	static const CPId& SharedEmptyObject(void);

	/// @brief    Returns a Gregorian calendar object.
	///
	/// @details  Static method that returns a gregorian calendar object.
	///
	/// @returns  a gregorian calendar object.
	///
	static CPCalendar GregorianCalendar(void);
	
	/// @brief    Returns a Gregorian calendar with the GMT time zone object.
	///
	/// @details  Static method that returns a gregorian calendar with the GMT time zone object.
	///
	/// @returns  a gregorian calendar with the GMT time zone object.
	///
	static CPCalendar GregorianGMTCalendar(void);
	
	/// @brief    Returns a Gregorian calendar with the specified time zone object.
	///
	/// @details  Static method that returns a gregorian calendar with the specified time zone object.
	///
	/// @returns  a gregorian calendar with the specified time zone object.
	///
	static CPCalendar GregorianCalendarWithTimeZone(const CPTimeZone& TimeZone);

	/// @brief    Returns the current calendar object.
	///
	/// @details  Static method that returns the users current calendar. Note that this does not change as the user changes calendars.
	///
	/// @returns  the calendar that the user is currently using.
	///
	static CPCalendar CurrentCalendar(void);

	/// @brief     Default Constructor.
	///
	/// @details   Creates an empty calendar.
	///
	CPCalendar(void);

	/// @brief     Copy Constructor.
	///
	/// @details   This object will contain the object of the specified calendar.
	///
	///	@param     Calendar  the calendar to copy.
	///
	CPCalendar(const CPCalendar& Calendar);

	///	@brief     Destructor.
	///
	///	@details   deallocates any unused resources.
	///
	~CPCalendar(void);

	///	@brief     Copy Operator.
	///
	///	@details   Copies the entries from the given calendar.
	///
	///	@param	   Calendar  the calendar to copy.
	///
	const CPCalendar& operator=(const CPCalendar& Calendar);

	/// @brief    Returns the objects time zone.
	///
	/// @details  Returns a CPTimeZone object that the object is using.
	///
	/// @returns  Returns a CPTimeZone object that the object is using.
	///
	CPTimeZone TimeZone(void) const;

	/// @brief    Sets the objects time zone.
	///
	/// @details  Sets the time zone that the object will use for any calculations.
	///
	///	@param    TimeZone  The time zone that the object should use for calculations.
	///
	void SetTimeZone(const CPTimeZone& TimeZone);

	/// @brief    Returns a CPDateComponents object containing a given date decomposed into specified components.
	///
	/// @details  Returns a CPDateComponents object containing a given date decomposed into specified components. The Weekday ordinality,
	///           when requested, refers to the next larger (than Week) of the requested units. Some computations can take a relatively long
	///           time.
	///           The following example shows how to use this method to determine the current year, month, and day, using a calendar
	///           (gregorian):
	///
	///           CPUInteger unitFlags = CPYearCalendarUnit | CPMonthCalendarUnit | CPDayCalendarUnit;
	///           CPDate date = CPDate::Today();
	///           CPDateComponents components = CPCalendar::GregorianCalendar().ComponentsFromDate(unitFlags, date);
	///
	///	@param    UnitFlags  The components into which to decompose date—a bitwise OR of CPCalendarUnit constants.
	///	@param    Date  The date for which to perform the calculation.
	///
	/// @returns  An CPDateComponents object containing date decomposed into the components specified by UnitFlags. Returns nil if date falls
	///           outside of the defined range of the receiver or if the computation cannot be performed.
	///
	CPDateComponents ComponentsFromDate(CPCalendarUnit UnitFlags, const CPDate& Date) const;

	/// @brief    Returns a CPDateComponents object using specified components, the difference between two supplied dates.
	///
	/// @details  Returns a CPDateComponents object containing the difference between from date and to date decomposed into specified components.
	///           The result is lossy if there is not a small enough unit requested to hold the full precision of the difference. Some operations
	///           can be ambiguous, and the behavior of the computation is calendar-specific, but generally larger components will be computed
	///           before smaller components; for example, in the Gregorian calendar a result might be 1 month and 5 days instead of, for example,
	///           0 months and 35 days. The resulting component values may be negative if resultDate is before startDate.
	///
	///	@param    UnitFlags  The components into which to decompose date—a bitwise OR of CPCalendarUnit constants.
	///	@param    FromDate  The from date for which to perform the calculation.
	///	@param    ToDate  The  to date for which to perform the calculation.
	///
	/// @returns  An CPDateComponents object containing difference between the dates decomposed into the components specified by UnitFlags.
	///           Returns nil if date falls
	///           outside of the defined range of the receiver or if the computation cannot be performed.
	///
	CPDateComponents ComponentsBetweenFromDateAndToDate(CPCalendarUnit UnitFlags, const CPDate& FromDate, const CPDate& ToDate) const;

	/// @brief    Returns a date representing the absolute time calculated by adding given components to a given date.
	///
	/// @details  Some operations can be ambiguous, and the behavior of the computation is calendar-specific, but generally components are
	///           added in the order specified. Overflow in a unit carries into the higher units (as in typical addition). Note that some
	///           computations can take a relatively long time.
	///
	///           The following example shows how to add 2 months and 3 days to the current date and time using a calendar (gregorian):
	///
	///           CPDate currentDate = CPDate::Now();
	///           CPDateComponents components;
	///           components.SetMonth(2);
	///           components.SetDay(3);
	///           CPDate date = CPCalendar::GregorianCalendar().DateByAddingComponentsToDate(components, currentDate);
	///
	///	@param    Components  The components into which to decompose date—a bitwise OR of CPCalendarUnit constants.
	///	@param    Date  The date to which Components are added.
	///
	/// @returns  A date representing the absolute time calculated by adding to date the calendrical components specified by Components.
	///           Returns an empty date if date falls outside the defined range of the receiver or if the computation cannot be performed.
	///
	CPDate DateByAddingComponentsToDate(const CPDateComponents& Components, const CPDate& Date) const;

	/// @brief    Returns a date representing the absolute time calculated by adding given components to a given date.
	///
	/// @details  When there are insufficient components provided to completely specify an absolute time, a calendar uses default values of
	///           its choice. When there is inconsistent information, a calendar may ignore some of the components parameters or the method
	///           may return an empty date. Unnecessary components are ignored (for example, Day takes precedence over Weekday and Weekday
	///           ordinals). Note that some computations can take a relatively long time to perform.
	///
	///           The following example shows how to use this method to create a date object to represent 14:10:00 on 6 January 1965, for a
	///           gregorian calendar.
	///           CPDateComponents components;
	///           components.SetYear(1965);
	///           components.SetMonth(1);
	///           components.SetDay(6);
	///           components.SetHour(14);
	///           components.SetMinute(10);
	///           components.SetSecond(0);
	///           CPDate date = CPCalendar::GregorianCalendar().DateFromComponents(components);
	///
	///	@param    Components  The components into which to decompose date—a bitwise OR of CPCalendarUnit constants.
	///
	/// @returns  A date representing the absolute time calculated from Components. Returns an empty date if the receiver cannot convert the
	///           components given in Components into an absolute time. The method also returns empty date for out-of-range values.
	///
	CPDate DateFromComponents(const CPDateComponents& Components) const;

	/// @brief    Returns a date representing a date at midnight on the day, month and Year specified.
	/// @details  A date representing a date at midnight on the day, month and Year specified. The date returned is dependent on the time
	///           zone used to create the date. A date created with a GMT calendar will not be equal to a date with the same day, month and
	///           year created with an eastern daylight time calendar. This method was added as a shorthand to make a midnight date.
	///
	///           The following example shows how to use this method to create a date object to represent 6 January 1965, for a
	///           gregorian calendar.
	///           CPDate date = CPCalendar::GregorianCalendar().DateFromComponents(components);
	///
	///	@param    Year  The year for the constructed date.
	///	@param    Month  The month for the constructed date.
	///	@param    Day  The day for the constructed date.
	///
	/// @returns  A date representing representing a date at midnight on the day, month and Year specified. The method also returns empty
	///           date for out-of-range values.
	///
	CPDate DateFromYearMonthAndDay(CPInteger Year, CPInteger Month, CPInteger Day) const;

#if __OBJC__
	///	@brief     Creates a calendar object that is the same as the calendar passed in.
	///
	///	@details   This copy is accomplished by using the same underlying calendar object as the calendar passed in.
	///
	///	@param     Calendar  A calendar to copy.
	///
	CPCalendar(NSCalendar* Calendar);

	/// @brief     Assignment of a calendar object to another calendar object.
	///
	/// @details   Assigns a calendar object to another calendar object.
	///
	/// @param	   Calendar  An underlying calendar object to copy.
	///
	const CPCalendar& operator=(NSCalendar* Calendar);

	///	@brief     Conversion Operator.
	///
	///	@details   Returns the underlying object as a NSCalendar*.
	///
	/// @returns  the underlying calendar ref.
	///
	operator NSCalendar*(void) const;
#endif
#if __COREFOUNDATION__
	/// @brief     Copy Constructor.
	///
	/// @details   This object will contain the object of the specified calendar.
	///
	///	@param     Calendar  the calendar to copy.
	///
	CPCalendar(CFCalendarRef Calendar);

	///	@brief     Copy Operator.
	///
	///	@details   Copies the entries from the given calendar.
	///
	///	@param	   Calendar  the calendar to copy.
	///
	const CPCalendar& operator=(CFCalendarRef Calendar);

	///	@brief     Conversion Operator.
	///
	///	@details   Returns the underlying object as a CFCalendarRef.
	///
	/// @returns  the underlying calendar ref.
	///
	operator CFCalendarRef(void) const;
#endif
	
	friend class CPObject;

protected:
	explicit CPCalendar(const CPId& Id);

#if _WINDOWS
private:
	CPTimeZone m_TimeZone;
#endif
};

#include "CrossPlatform/CPDateComponents.h"
