////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPCharacterSet.h
//    Cross-platform character set class.
//
//  Copyright © 2011-2012 by Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//	10/24/11  B.D.D.  Added AddCharactersInString, RemoveCharactersInString.
//	10/11/11  B.D.D.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/CPObject.h"

class CPString;

/// @brief   Represents an object holding Unicode-compliant characters.
/// @details An CPCharacterSet object represents a set of Unicode-compliant characters. CPString and CPScanner objects use CPCharacterSet
///          objects to group characters together for searching operations, so that they can find any of a particular set of characters 
///          during a search. The CPCharacterSet declares the programmatic interface for static character sets.
///
/// For more information:
/// http://developer.apple.com/library/mac/#documentation/Cocoa/Reference/Foundation/Classes/NSCharacterSet_Class
/// http://developer.apple.com/library/mac/#documentation/CoreFoundation/Reference/CFCharacterSetRef
///
class CP_EXTERNAL_API CPCharacterSet : public CPObject
{
public:
	///	@brief     A cla3ss type that uniquely identifies this class.
	///	@return    Returns a class type that uniquely identifies this class.
	static CPClassType ClassType(void);

	/// @brief    Returns the underlying shared empty object.
	/// @details  Static method that returns the underlying object that is shared between all empty objects of this class type.
	/// @returns  the underlying shared empty object.
	static const CPId& SharedEmptyObject(void);

	/// @brief     Returns the platform specific character set of whitespece characters.
	///	@details   This static method returns the platform specific character set of whitespece characters.
	///	@return    This returns the character set object.
	static const CPCharacterSet& WhitespaceCharacterSet(void);
	
	/// @brief     Returns the platform specific character set of whitespece and newline characters.
	///	@details   This static method returns the platform specific character set of whitespece and newline characters.
	///	@return    This returns the character set object.
	static const CPCharacterSet& WhitespaceAndNewlineCharacterSet(void);
	
	/// @brief     Returns the platform specific character set of decimal digit characters.
	///	@details   This static method returns the platform specific character set of decimal digit characters.
	///	@return    This returns the character set object.
	static const CPCharacterSet& DecimalDigitCharacterSet(void);
	
	/// @brief     Returns the platform specific character set of letter characters.
	///	@details   This static method returns the platform specific character set of letter characters.
	///	@return    This returns the character set object.
	static const CPCharacterSet& LetterCharacterSet(void);
	
	/// @brief     Returns the platform specific character set of alpha numeric characters.
	///	@details   This static method returns the platform specific character set of alpha numeric characters.
	///	@return    This returns the character set object.
	static const CPCharacterSet& AlphaNumericCharacterSet(void);
	
	/// @brief     Returns the platform specific character set containing only characters that don’t exist in the given string.
	///	@details   This static method returns the platform specific character set containing only characters that don’t exist in the given 
	///            string.
	/// @param     CharacterSet  The character set that holds the characters in which to return the inverted character set from.
	///	@return    This returns the character set object.
	static CPCharacterSet InvertedCharacterSet(const CPCharacterSet& CharacterSet);
	
	/// @brief     Returns the platform specific character set containing only characters that exist in the given string.
	///	@details   This static method returns the platform specific character set containing only characters that exist in the given string.
	/// @param     CharacterSet  The character set that holds the characters in which to return the character set from.
	///	@return    This returns the character set object.
	static CPCharacterSet CharacterSetWithCharactersInString(const CPString& CharacterSet);
	
	/// @brief     Default Constructor.
	/// @details   Creates an empty character set.
	CPCharacterSet(void);
	
	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the given character set.
	///	@param     CharacterSet  the character set to copy.
	CPCharacterSet(const CPCharacterSet& CharacterSet);
	
	///	@brief     Destructor.
	///	@details   Removes any references it has to shared resources.
	~CPCharacterSet(void);

	/// @brief     Adds the characters in the given string to the character set.
	void AddCharactersInString(const CPString& String);
	
	/// @brief     Removes the characters in the given string from the character set.
	void RemoveCharactersInString(const CPString& String);

#if __OBJC__
	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the given character set.
	///	@param     CharacterSet  the character set to copy.
	CPCharacterSet(NSCharacterSet* CharacterSet);

	///	@brief     Copy Operator.
	///	@details   Copies the given character set.
	///	@param	   CharacterSet  the character set to copy.
	const CPCharacterSet& operator=(NSCharacterSet* CharacterSet);
	
	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as a NSCharacterSet*.
	operator NSCharacterSet*(void) const;	
#endif  // #ifdef __OBJC__
#if __COREFOUNDATION__
	// Define methods that will be needed only when bridging to CoreFoundation

	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the given character set.
	///	@param     Value  the character set to copy.
	CPCharacterSet(CFCharacterSetRef Value);
	
	///	@brief     Copy Operator.
	///	@details   Copies the given character set.
	///	@param	   Value  the character set to copy.
	const CPCharacterSet& operator=(CFCharacterSetRef Value);
	
	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as a CFCharacterSetRef*.
	operator CFCharacterSetRef(void) const;
#endif

protected:
	CPCharacterSet(const CPObject& CharacterSet);
	CPCharacterSet(const CPId& CharacterSet);
};
