////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPData.h
//    Cross-platform string class.
//
//  Copyright © 2009-2013 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  05/29/12  G.R.W.  Removed the Range() method.
//  02/10/12  K.L.S.  Changes for CrossPlatform API changes. Reworked header includes.
//	10/20/09  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/ExternalAPIMacros.h"
#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/CPUInteger.h"
#include "CrossPlatform/CPObject.h"
#include "CrossPlatform/CPStringEncoding.h"

class CPRange;
class CPString;
class CPDictionary;
class CPURL;
class CPError;

/// @brief    Class encapsulation of a number.
/// @details  CPData and its mutable subclass CPMutableData provide data objects, object-oriented wrappers for byte buffers. Data objects
///           let simple allocated buffers (that is, data with no embedded pointers) take on the behavior of Foundation objects.
///
///           CPData creates static data objects, and CPMutableData creates dynamic data objects. CPData and CPMutableData are typically 
///           used for data storage and are also useful in Distributed Objects applications, where data contained in data objects can be 
///           copied or moved between applications.
///
///           Using 32-bit Cocoa, the size of the data is subject to a theoretical 2GB limit (in practice, because memory will be used by
///           other objects this limit will be smaller); using 64-bit Cocoa, the size of the data is subject to a theoretical limit of
///           about 8EB (in practice, the limit should not be a factor).
///
/// For more information:
/// http://developer.apple.com/library/mac/#documentation/Cocoa/Reference/Foundation/Classes/NSData_Class
/// http://developer.apple.com/library/mac/#documentation/CoreFoundation/Reference/CFDataRef
///
class CP_EXTERNAL_API CPData : public CPObject
{
public:
	///	@brief     A cla3ss type that uniquely identifies this class.
	///	@return    Returns a class type that uniquely identifies this class.
	static CPClassType ClassType(void);
	
	/// @brief    Returns the underlying shared empty object.
	/// @details  Static method that returns the underlying object that is shared between all empty objects of this class type.
	/// @returns  the underlying shared empty object.
	static const CPId& SharedEmptyObject(void);

	///	@brief     Return a data object instance based on the data object passed in.
	///	@details   This static method creates and returns a data object containing the given object.
	///	@param     Data  An underlying data object to create the data from.
	///	@return    This returns an instance of a data object.
	static CPData DataWithData(const CPData& Data);

	///	@brief     Return a data object instance based on the contents of the file.
	///	@details   This static method creates and returns a data object containing the contents of the data file specified at the filepath
	///            passed in.
	///	@param     FilePath  A path and filename to the data file to be read from.
	///	@return    This returns an instance of a data object.
	static CPData DataWithContentsOfFile(const CPString& FilePath);
	
	///	@brief     Return a data containing the contents of the URL.
	///	@details   This method creates and returns a data containing the contents of the specified URL. This method is ideal for converting
	///            data:// URLs to CPData objects, and can also be used for reading short files synchronously. If you need to read potentially
	///            large files, look to using on a seperate thread or using something else as this will pause code execution till download is
	///            complete.
	///	@param     URL  The URL from which to read data.
	///	@return    A data containing the data from the location specified by the URL. Returns and empty data if the data could not be read.
	static CPData DataWithContentsOfURL(const CPURL& URL);
	
	///	@brief     Return a data object instance based on the data buffer.
	///	@details   This static method creates and returns a data object containing the specified number of bytes using the data buffer
	///            passed in.
	///	@param     Bytes  A pointer to a byte array in memory.
	///	@param     Length  The length of the desired data object to be created.
	///	@return    This returns an instance of a data object.
	static CPData DataWithBytesOfLength(const void* Bytes, CPUInteger Length);

	///	@brief     Return a data object instance based on the data buffer. Uses the buffer as the backing storage.
	///	@details   This static method creates and returns a data object containing the specified number of bytes using the data buffer
	///            passed in. It takes ownership of memory and will delete memory when done using it.
	///	@param     Bytes  A pointer to a byte array in memory. Bytes must point to a memory block allocated with malloc.
	///	@param     Length  The number of bytes to hold from bytes. This value must not exceed the length of bytes.
	///	@return    This returns an instance of a data object.
	static CPData DataWithBytesNoCopyOfLength(void* Bytes, CPUInteger Length);

	///	@brief     Return a data object instance based on the data buffer. Uses the buffer as the backing storage.
	///	@details   This static method creates and returns a data object containing the specified number of bytes using the data buffer
	///            passed in.
	///	@param     Bytes  A pointer to a byte array in memory. If ShouldTakeOwnership is true, bytes must point to a memory block allocated
	///                   with malloc.
	///	@param     Length  The number of bytes to hold from bytes. This value must not exceed the length of bytes.
	///	@param     ShouldFreeWhenDone  If true, the returned object takes ownership of the bytes pointer and frees it on deallocation.
	///	@return    This returns an instance of a data object.
	static CPData DataWithBytesNoCopyOfLengthAndShouldFreeWhenDone(void* Bytes, CPUInteger Length, bool ShouldFreeWhenDone);

	///	@brief     Return a data object instance based on the passed string using a specified encoding.
	///	@details   This static method creates and returns a data object containing the string passed in using the specified encoding.
	///	@param     String  A string to be converted into a data object.
	///	@param     Encoding  The desired encoding to be used for the conversion.
	///	@return    This returns an instance of a data object.
	static CPData DataWithStringUsingEncoding(const CPString& String, CPStringEncoding Encoding);

	///	@brief     Return a data object instance based on the passed string using a specified encoding.
	///	@details   This static method creates and returns a data object containing the string passed in using the specified encoding.
	///	@param     String  A string to be converted into a data object.
	///	@param     Encoding  The desired encoding to be used for the conversion.
	///	@return    This returns an instance of a data object containing a C string of the String in the requested encoding.
	static CPData DataContainingCStringWithStringUsingEncoding(const CPString& String, CPStringEncoding Encoding);

	///	@brief     Return a data object instance based on the dictionary passed in.
	///	@details   This static method creates and returns a data object containing the given dictionary.
	///	@param     Dictionary  An underlying dictionary object to create the data from.
	///	@return    This returns an instance of a data object.
	static CPData DataWithDictionary(const CPDictionary& Dictionary);

	static CPData DataFromObject(const CPObject& Object);
	static CPData DataFromId(const CPId& Object);

	/// @brief     Default Constructor.
	/// @details   Creates an empty data object.
	CPData(void);
	
	/// @brief     Constructor.
	/// @details   This data object will contain the specified object.
	///	@param     Object  the object to copy.
	explicit CPData(const CPObject& Object);

	/// @brief     Constructor.
	/// @details   This data object will contain the specified object.
	///	@param     Object  the object to copy.
	explicit CPData(const CPId& Object);

	/// @brief     Copy Constructor.
	/// @details   This data object will contain the object of the specified data.
	///	@param     Data  the data to copy.	
	CPData(const CPData& Data);
	
	/// @brief     Copy Constructor.
	/// @details   This data object will contain the specified string.
	///	@param     String  the string to copy.	
	CPData(const CPString& String);
	
	/// @brief     Copy Constructor.
	/// @details   Creates a data object containing the dictionary.
	///	@param     Dictionary  A dictionary to copy.
	explicit CPData(const CPDictionary& Dictionary);
	
	///	@brief     Destructor.
	///	@details   Removes any references it has to shared resources.
	~CPData(void);
	
	/// @brief     Extract specified data from the data object.
	/// @details   Extract and return a data object based on the given range within the data object.
	///	@param     Range  the range used to extract data from the data object.
	/// @return    The new data object based on the given range of the data object.
	CPData SubdataWithRange(const CPRange& Range) const;

	///	@brief     Initializes itself so that it contains the given data.
	///	@details   Initializes itself so that it contains the given data object.  Any previous contents are emptied.
	///	@param     Data  the data used to initialize the data object.
	void InitWithData(const CPData& Data);
	
	///	@brief     Initializes itself so that it contains the given string.
	///	@details   Initializes itself so that it contains the given string.  Any previous contents are emptied.
	///	@param     String  the string used to initialize the data object.
	void InitWithString(const CPString& String);
	
	///	@brief     Initializes itself so that it contains the given string with the given encoding.
	///	@details   Initializes itself so that it contains the given string with the given encoding.  Any previous contents are emptied.
	///	@param     String  the string used to initialize the data object.
	///	@param     Encoding  The desired encoding to be used for the initialization.
	void InitWithStringUsingEncoding(const CPString& String, CPStringEncoding Encoding);

	///	@brief     Initializes itself so that it contains the given string as a C string with the given encoding.
	///	@details   Initializes itself so that it contains the given string as a C string with the given encoding.  Any previous contents are
	///            emptied.
	///	@param     String  the string used to initialize the data object.
	///	@param     Encoding  The desired encoding to be used for the initialization.
	void InitContainingCStringWithStringUsingEncoding(const CPString& String, CPStringEncoding Encoding);

	///	@brief     Initializes itself so that it contains the contents of the file.
	///	@details   Initializes itself so that it contains the contents of the file.  Any previous contents are emptied.
	///	@param     FilePath  A path and filename to the data file to be read from used to initialize the data object.
	void InitWithContentsOfFile(const CPString& FilePath);

	///	@brief     Initializes itself with the contents of the URL.
	///	@details   This method load the contents of the specified URL. This method is ideal for converting
	///            data:// URLs to CPData objects, and can also be used for reading short files synchronously. If you need to read potentially
	///            large files, look to using on a seperate thread or using something else as this will pause code execution till download is
	///            complete.
	///	@param     URL  The URL from which to read data.
	void InitWithContentsOfURL(const CPURL& URL);

	///	@brief     Initializes itself so that it contains the given dictionary.
	///	@details   Initializes itself so that it contains the given dictionary.  Any previous contents are emptied.
	///	@param     Dictionary  An underlying dictionary object used to initialize the data object.
	void InitWithDictionary(const CPDictionary& Dictionary);

	///	@brief     Initializes itself so that it contains the given data buffer.
	///	@details   Initializes itself so that it contains a data object with the specified number of bytes using the given data buffer.
	///	@param     Bytes  A pointer to a byte array in memory.
	///	@param     Length  The length of the desired data object to be created.
	void InitWithBytesOfLength(const void* Bytes, CPUInteger Length);

	///	@brief     Copy Operator.
	/// @details   Assigns a data object to another data object.
	///	@param     Data  A data object to copy from.
	const CPData& operator=(const CPData& Data);
	
	///	@brief     Copy Operator.
	/// @details   Assigns a data object to the given string.
	///	@param     String  The string to copy.
	const CPData& operator=(const CPString& String);
	
	///	@brief     Copy Operator.
	/// @details   Assigns a data object to the given dictionary.
	///	@param     Dictionary  The dictionary to copy.
	const CPData& operator=(const CPDictionary& Dictionary);

	///	@brief     Checks to see if the data object is empty.
	///	@details   This checks to see if the data object is empty and returns its status.
	///	@return    A bool set to true if the object was empty, otherwise it is false.
	bool IsEmpty(void) const;
	
	///	@brief     returns the number of bytes in the data buffer.
	///	@return    The number of bytes for the data buffer. Zero if it is null.
	CPUInteger Length(void) const;
	
	///	@brief     Retrieves data as byte pointer.
	///	@details   Retrieves the data object as a pointer to a byte array.
	///	@return    A pointer to bytes in memory.
	const void* Bytes(void) const;
	
	///	@brief     Retrieves data as a char pointer.
	///	@details   Retrieves the data object as a pointer to a char array.
	///	@return    A pointer to characters in memory.
	const char* CharPointer(void) const;
	
	///	@brief     Retrieves data as a wide char pointer.
	///	@details   Retrieves the data object as a pointer to a wide char array.
	///	@return    A pointer to wide characters in memory.
	const wchar_t* WideCharPointer(void) const;
	
	///	@brief     Retrieves bytes.
	///	@details   Retrieves data as a byte pointer set to the given variable.
	///	@param[out] ToBuffer  A pointer variable to a byte array in memory which is used for output.
	const void GetBytes(void* ToBuffer) const;
	
	///	@brief     Retrieves a number of bytes
	/// @details   Extract and return a byte pointer based on the given length within the data object.
	///	@param[out] ToBuffer  A pointer variable to a byte array in memory which is used for output.
	///	@param     Length  the length used to extract data from the data object.
	const void GetBytesWithLength(void* ToBuffer, CPUInteger Length) const;
	
	///	@brief     Retrieves a range of bytes
	/// @details   Extract and return a byte pointer based on the given range within the data object.
	///	@param[out] ToBuffer  A pointer variable to a byte array in memory which is used for output.
	///	@param     Range  the range used to extract data from the data object.
	const void GetBytesWithRange(void* ToBuffer, CPRange Range) const;

	///	@brief     Writes the data to a specified file.
	///	@details   Writes the bytes in the receiver to the file specified by a given path.
	///	@param     FilePath  the path and filename of the file to write the data to.
	///	@return    A bool set to true if the object was written successfully, otherwise it is false.
	bool WriteToFile(const CPString& FilePath) const;
	bool WriteToFile(const CPString& FilePath, CPError& Error) const;

	///	@brief     Writes the data to a temp file and then renames to specified file.
	///	@details   Writes the bytes in the receiver to the file specified by a given path by first successfully writing to a temp file the
	///            renaming to the given path.
	///	@param     FilePath  the path and filename of the file to write the data to.
	///	@return    A bool set to true if the object was written successfully, otherwise it is false.
	bool WriteToFileAtomically(const CPString& FilePath) const;
	bool WriteToFileAtomically(const CPString& FilePath, CPError& Error) const;

#if __OBJC__
	// Define methods that will be needed only when bridging to Cocoa

	/// @brief     Copy Constructor.
	/// @details   This data object will contain the object of the specified data.
	///	@param     Data  the data to copy.	
	CPData(NSData* Data);
	
	/// @brief     Copy Constructor.
	/// @details   This data object will contain the object of the specified data.
	///	@param     Data  the data to copy.	
	CPData(NSMutableData* Data);
	
	///	@brief     Copy Operator.
	/// @details   Assigns a data object to another data object.
	///	@param     Data  A data object to copy from.	
	const CPData& operator=(NSData* Data);
	
	///	@brief     Copy Operator.
	/// @details   Assigns a data object to another data object.
	///	@param     Data  A data object to copy from.	
	const CPData& operator=(NSMutableData* Data);
	
	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as a NSData*.
	operator NSData*() const;
#endif
#if __COREFOUNDATION__
	// Define methods that will be needed only when bridging to CoreFoundation

	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the specified data.
	///	@param	   Value  the data to copy.
	CPData(CFDataRef Value);

	///	@brief     Copy Operator.
	///	@details   Copies the entries from the given data.
	///	@param	   Value  the data to copy.
	const CPData& operator=(CFDataRef Value);
	
	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as a CFDataRef*.
	operator CFDataRef(void) const;
#endif

protected:
	CPData(const CPId& OtherObject, CPClassType ClassType);
	CPData(const CPObject& OtherObject, CPClassType ClassType);

	void AssignUnderlyingData(const CPId& Data);
	void AssignUnderlyingDataAndTakeOwnership(const CPId& Data);
};

#include "CrossPlatform/CPRange.h"
#include "CrossPlatform/CPString.h"
#include "CrossPlatform/CPDictionary.h"
