////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPDate.h
//    Cross-platform date class.
//
//  Copyright © 2009-2012 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  02/10/12  K.L.S.  Changes for CrossPlatform API changes. Reworked header includes.
//  01/25/12  G.R.W.  Changed the header to a new style and added documentation comments.
//  06/09/10  G.R.W.  Added method DateFromMonthDayYearHourMinuteSecond and SetFromMonthDayYearHourMinuteSecond.
//  03/25/09  K.L.S.  Created file.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/CPInteger.h"
#include "CrossPlatform/CPDouble.h"
#include "CrossPlatform/CPObject.h"
#include "CrossPlatform/CPString.h"
#include "CrossPlatform/CPComparisonResult.h"

/// @brief   Represent a single point in time.
///
/// @details CPDate is a class that provides behavior for creating dates, comparing dates, representing dates, computing intervals, and
///          similar functionality. CPDate presents a programmatic interface through which suitable date objects are requested and returned.
///          Date objects returned from CPDate are lightweight and immutable since they represent an invariant point in time. This class is
///          designed to provide the foundation for arbitrary calendrical representations.
/// 
///          The sole primitive method of CPDate, timeIntervalSinceReferenceDate, provides the basis for all the other methods in the CPDate
///          interface.  This method returns a time value relative to an absolute reference date of the first instant of 1 January 2001,GMT.
///	
///          To parse strings containing dates and to generate string representations of a date, you should use an instance of
///          CPDateFormatter using the methods DateFromString() and StringFromDate() respectively, see CPDateFormatter documentation for
///          more details.
/// 
///          Notes: CPDate uses NSDate or its Core Foundation counterpart, CFDate Reference to provide most of the functionality. NSDate and
///          CFDate are bridged Foundation objects. Therefore, in a method where you see an NSDate* parameter, you can pass a CFDateRef, and
///          in a function where you see a CFDateRef parameter, you can pass an NSDate instance. NSDate is currently not available on
///          Windows.
/// 
/// For more information:
///http://developer.apple.com/library/mac/#documentation/Cocoa/Reference/Foundation/Classes/NSDate_Class
///http://developer.apple.com/library/mac/documentation/CoreFoundation/Reference/CFDateRef
///
class CP_EXTERNAL_API CPDate : public CPObject
{
public:
	///	@brief     A class type that uniquely identifies this class.
	///
	///	@return    Returns a class type that uniquely identifies this class.
	///
	static CPClassType ClassType(void);

	/// @brief    Returns the underlying shared empty object.
	///
	/// @details  Static method that returns the underlying object that is shared between all empty objects of this class type.
	///
	/// @returns  the underlying shared empty object.
	///
	static const CPId& SharedEmptyObject(void);

	/// @brief     Returns a date object representing a date in the distant future.
	///
	/// @details   You can pass this value when an date object is required to have the date argument essentially ignored.
	///            For example, in a function that takes a date range, you can use this date to make the upper limit of the range open ended.
	///
	/// @return    A date object representing a date in the distant future (in terms of centuries).
	///
	static const CPDate& DistantFuture(void);
	
	/// @brief     Returns a date object representing a date in the distant past.
	///
	/// @details   You can use this object as a control date, a guaranteed temporal boundary.
	///
	/// @return    A date object representing a date in the distant past (in terms of centuries).
	///
	static const CPDate& DistantPast(void);
	
	/// @brief     Returns a date object representing the current date at 12 midnight using the current time zone.
	///
	/// @details   You can use this object when you want a date and do not want any time components. The time components are all zero.
	///
	/// @return    A date object representing the current date at 12 midnight using the current time zone.
	///
	static CPDate Today(void);

	/// @brief     Returns a date object representing the current date and time.
	///
	/// @details   You can use this object when you want the current date and time.
	///
	/// @return    A date object representing the current date and time using the current time zone.
	///
	static CPDate Now(void);
	
	/// @brief     Returns a date object representing the reference date of 1 January 2001, GMT.
	///
	/// @details   You can use this object when you want a date and do not want any time components. The time components are all zero.
	///
	/// @return    A date object representing the reference date.
	///
	static const CPDate& ReferenceDate(void);

	/// @brief     Returns a date object based on seconds calulation since a start date of 1 January 1970.
	///
	/// @details   You can use this object when you want a date representing the result of a number of seconds added to a start date
	///            of 1 January 1970, midnight GMT.
	///
	/// @param     Seconds  An integer representing the number of seconds used to perform the calculation.
	///
	/// @return    A date object representing the calculated date.
	///
	static CPDate DateWithTimeIntervalSince1970(CPDouble Seconds);

	/// @brief     Returns a date object based on seconds calulation since a reference date of 1 January 2001.
	///
	/// @details   You can use this object when you want a date representing the result of a number of seconds added to a reference date
	///            of 1 January 2001, midnight GMT.
	///
	/// @param     Seconds  the number of seconds used to perform the calculation.
	///
	/// @return    A date object representing the calculated date.
	///
	static CPDate DateWithTimeIntervalSinceReferenceDate(CPDouble Seconds);

	/// @brief     Returns a date object based on seconds calculation since now.
	///
	/// @details   You can use this object when you want a date representing the result of a number of seconds added to the current date and
	///            time.
	///
	/// @param     Seconds  the number of seconds used to perform the calculation.
	///
	/// @return    A date object representing the calculated date.
	///
	static CPDate DateWithTimeIntervalSinceNow(CPDouble Seconds);

	/// @brief     Returns a date object representing the date specified by the month, day and year. [DEPRECATED]
	/// @details   You can use this object when you want a specific date and do not want any time components. The time components are all zero.
	/// @param     Month  The month, identified as a number from 1 to 12.
	/// @param     Day  The day of the month.
	/// @param     Year  The year, identifed as a complete number (includes century).
	/// @return    A date object representing the date specified.
	static CPDate DateFromMonthDayYear(CPInteger Month, CPInteger Day, CPInteger Year);

	/// @brief     Returns a date object representing the date specified by the month, day and year. [DEPRECATED]
	/// @details   You can use this object when you want a specific date and do not want any time components. The time components are all zero.
	/// @param     Month  The month, identified as a number from 1 to 12.
	/// @param     Day  The day of the month.
	/// @param     Year  The year, identifed as a complete number (includes century).
	/// @return    A date object representing the date specified.
	static CPDate DateFromMonthDayYearGMT(CPInteger Month, CPInteger Day, CPInteger Year); // TODO: KLS - Look to add time zone parameter

	/// @brief     DEPRECATED! Returns a date object representing the date specified by the month, day, year, hour, minute, and second.
	/// @details   You can use this object when you want a specific date and time.
	/// @param     Month  The month, identified as a number from 1 to 12.
	/// @param     Day  The day of the month.
	/// @param     Year  The year, identifed as a complete number (includes century).
	/// @param     Hour  The hour, identifed as a number in the range from 0 to 23 (military time).
	/// @param     Minute  The minute, identifed as a number in the range from 0 to 59.
	/// @param     Second  The second, identifed as a decimal number in the range from 0.000 to 59.999.
	/// @return    A date object representing the date and time specified.
	static CPDate DateFromMonthDayYearHourMinuteSecond(CPInteger Month, CPInteger Day, CPInteger Year, CPInteger Hour, CPInteger Minute,
													CPDouble Second);
	
	/// @brief     Returns a date object with a date or date and time value specified by a given string in the string representation format
	///            (MM-DD-YYYY HH:MM:SS) or (MM-DD-YYYY). [DEPRECATED]
	/// @details   You can use this object when you want a specific date and time components.
	/// @param     DateString  A string that specifies a date or a date and time value.
	/// @return    A date object representing the date and time specified.
	static CPDate DateFromString(const CPString& DateString);
	
	/// @brief     Returns a date object with a date or date and time value specified by a given string in the format specified. [DEPRECATED]
	/// @details   You can use this object when you want a specific date and time components.
	/// @param     DateString  A string that specifies a date or a date and time value.
	/// @param     DateFormat  A string that specifies a format that the date string is in.
	/// @return    A date object representing the date and time specified.
	static CPDate DateFromStringUsingFormat(const CPString& DateString, const CPString& DateFormat);
	
	///	@brief     Return an date instance from the object passed in.
	///
	///	@details   Creates and returns an date that is the object passed in. Will be empty if the object is not an date.
	///
	///	@param     Object  An date object.
	///
	///	@return    an date with the underlying date object equal to the underlying object of the object passed in.
	///
	static CPDate DateFromObject(const CPObject& Object);

	///	@brief     Return an date instance from the object passed in.
	///
	///	@details   Creates and returns an date with the underlying object set to the object passed in.
	///
	///	@param     Object  An underlying date object.
	///
	///	@return    an date with the underlying date object equal to the object passed in.
	///
	static CPDate DateFromId(const CPId& Object);

	/// @brief     Creates a date object that is empty.
	///
	/// @details   A date object that is empty, it does not represent a date and time.  This object does not have an underlying date object
	///            and as such is limited in functionality. Think of this like having an object pointer that is null.
	///
	CPDate(void);

	///	@brief     Creates a date object that is the same as the date passed in.
	///
	///	@details   This copy is accomplished by using the same underlying date object as the date passed in.
	///
	///	@param     Date  A date to copy.
	///
	CPDate(const CPDate& Date);

	///	@brief     Creates a date object that is the same as the date passed in.
	///
	///	@details   This copy is accomplished by using the same underlying date object as the date passed in.
	///
	///	@param     Date  An object to copy if it is really a date.
	///
	explicit CPDate(const CPObject& Date);

	/// @brief     Creates a date object that is the same as the date passed in.
	///
	/// @details   This copy is accomplished by using the same underlying date object as the date passed in.
	///
	/// @param	   Date  An underlying date object to copy.
	///
	explicit CPDate(const CPId& Date);

	/// @brief     Deallocates the date object. Removes its reference to its underlying object.
	///
	/// @details   Removes its reference to its underlying object.
	///
	~CPDate(void);

	/// @brief     Checks to see if the date object is empty.
	///
	/// @details   This checks to see if the date object is empty and returns its status.
	///
	/// @return    A bool set to true if the object was empty, otherwise it is false.
	///
	bool IsEmpty(void) const;

	/// @brief     Assignment of a date object to another date object.
	///
	/// @details   Assigns a date object to another date object.
	///
	/// @param	   Date  An underlying date object to copy.
	///
	const CPDate& operator=(const CPDate& Date);

	/// @brief     Assignment of a date object to date string.
	///
	/// @details   Assigns a date object to a date string.
	///
	/// @param	   DateString  An underlying string object from which to extract the date.
	///
	const CPDate& operator=(const CPString& DateString);
	
	/// @brief     Equals Operator.
	///
	/// @details   This checks to see if the date object is equal to the same date and time contained in the compared date object.
	///
	/// @param	   Date  An underlying date object used in the compare.
	///
	/// @return    A bool set to true if the date and time are the same, otherwise it is false.
	///
	bool operator==(const CPDate& Date) const;

	/// @brief     Not Equals Operator.
	///
	/// @details   This checks to see if the date object is not equal to the same date and time contained in the compared date object.
	///
	/// @param	   Date  An underlying date object used in the compare.
	///
	/// @return    A bool set to true if the date and time are NOT the same, otherwise it is false.
	///
	bool operator!=(const CPDate& Date) const;
    
	/// @brief     Greater Than Operator.
	///
	/// @details   This checks to see if the date object is greater (later) than the date and time contained in the compared date object.
	///
	/// @param	   Date  An underlying date object used in the compare.
	///
	/// @return    A bool set to true if the date and time are later than the object's date and time being compared,
	///            otherwise it is false.
	///
	bool operator>(const CPDate& Date) const;

	/// @brief     Less Than Operator.
	///
	/// @details   This checks to see if the date object is less (earlier) than the date and time contained in the compared date object.
	/// @param	   Date  An underlying date object used in the compare.
	///
	/// @return    A bool set to true if the date and time are earlier than the object's date and time being compared,
	///            otherwise it is false.
	///
	bool operator<(const CPDate& Date) const;

	/// @brief     Greater Than or Equals Operator.
	///
	/// @details   This checks to see if the date object is greater (later) than or the same as the date and time contained in the
	///            compared date object.
	///
	/// @param	   Date  An underlying date object used in the compare.
	///
	/// @return    A bool set to true if the date and time are later than or the same as the object's date and time being compared,
	///            otherwise it is false.
	///
	bool operator>=(const CPDate& Date) const;

	/// @brief     Less Than or Equals Operator.
	///
	/// @details   This checks to see if the date object is less (earlier) than or the same as the date and time contained in the
	///            compared date object.
	///
	/// @param	   Date  An underlying date object used in the compare.
	///
	/// @return    A bool set to true if the date and time are earlier than or the same as the object's date and time being compared,
	///            otherwise it is false.
	///
	bool operator<=(const CPDate& Date) const;

	/// @brief     Returns an CPComparisonResult value that indicates the temporal ordering of the receiver and another given date.
	///
	/// @details   This method detects sub-second differences between dates. If you want to compare dates with a less fine granularity,
	///            use TimeIntervalSinceDate to compare the two dates.
	///
	/// @param    AnotherDate  The date with which to compare the receiver. This value must not be nil. If the value is nil, the behavior is
	///           undefined.
	///
	/// @return   If:
	///                 The receiver and AnotherDate are exactly equal to each other, CPOrderedSame
	///                 The receiver is later in time than AnotherDate, CPOrderedDescending
	///                 The receiver is earlier in time than AnotherDate, CPOrderedAscending.
	///
	CPComparisonResult Compare(const CPDate& AnotherDate) const;

	/// @brief     Returns a Boolean value that indicates whether a given object is exactly equal the receiver.
	///
	/// @details   This method detects sub-second differences between dates. If you want to compare dates with a less fine granularity,
	//             use TimeIntervalSinceDate to compare the two dates.
	///
	/// @param	   AnotherDate  The date to compare with the receiver.
	///
	/// @return    true if the AnotherDate is exactly equal to the receiver, otherwise false.
	///
	bool IsEqualToDate(const CPDate& AnotherDate) const;
	
	/// @brief     Returns the earlier of the receiver and another given date.
	///
	/// @details   Returns the earlier of the receiver and another given date, determined using TimeIntervalSinceDate.
	///
	/// @param	   AnotherDate  The date to compare with the receiver.
	///
	/// @return    The earlier of the receiver and AnotherDate, determined using TimeIntervalSinceDate.
	///
	CPDate EarlierDate(const CPDate& AnotherDate) const;

	/// @brief     Returns the later of the receiver and another given date.
	///
	/// @details   Returns the later of the receiver and another given date, determined using TimeIntervalSinceDate.
	///
	/// @param	   AnotherDate  The date to compare with the receiver.
	///
	/// @return    The later of the receiver and AnotherDate, determined using TimeIntervalSinceDate.
	///
	CPDate LaterDate(const CPDate& AnotherDate) const;

	/// @brief     Adds a days to the existing date. [DEPRECATED]
	/// @details   Adds a specified number of days to the existing date object.
	/// @param     Days  An integer representing the number of days to add to the existing date.
	void AddDays(CPInteger Days);
    
	/// @brief     Adds a number of seconds to the existing Date. [DEPRECATED]
	/// @details   You can use this object when you want a date representing the result of a number of seconds added the existing date object.
	/// @param     Seconds  An integer representing the number of seconds used to perform the calculation.
	void AddTimeInterval(CPDouble Seconds);
	
	/// @brief     Gets the interval of time since Now
	///
	/// @details   You can use this object when you want a time interval representing the difference between now and the date of the existing
	///            date object.
	///
	/// @return    the calculated time interval.
	///
	CPDouble TimeIntervalSinceNow(void) const;

	/// @brief     Gets the interval of time since the reference date.
	///
	/// @details   You can use this object when you want a time interval representing the difference between the reference date(of Jan 1, 1970)
	///            and the existing date object.
	///
	/// @return    the calculated time interval.
	///
	CPDouble TimeIntervalSinceReferenceDate(void) const;

	/// @brief     Gets the interval of time between the date passed and the object date.
	///
	/// @details   You can use this object when you want a time interval representing the difference the date passed and the instance date.
	///
	/// @param     SinceDate  An date representing the date used to perform the calculation.
	///
	/// @return    the calculated time interval.
	///
	CPDouble TimeIntervalSinceDate(CPDate SinceDate) const;

	/// @brief     Gets the interval of time (in Days) since Now [DEPRECATED]
	/// @details   You can use this object when you want a time interval in Days representing the difference between now and the date of the existing
	///            date object.
	/// @return    A double object representing the calculated time interval in days.
	CPDouble DaysSinceNow(void) const;
    
	/// @brief     Gets the interval of time (in Days) since the reference date. [DEPRECATED]
	/// @details   You can use this object when you want a time interval (in days) representing the difference between the reference 
	///            date (of Jan 1, 1970) and the existing date object.
	/// @return    A double object representing the calculated time interval in days.
	CPDouble DaysSinceReferenceDate(void) const;
	
	/// @brief     Sets the date instance to the specified month, day, and year. [DEPRECATED]
	/// @details   You can use this object when you want a specific date set for the existing instance of the date object.
	/// @param     Month  The month, identified as a number from 1 to 12.
	/// @param     Day  The day of the month.
	/// @param     Year  The year, identifed as a complete number (includes century).
	void SetFromMonthDayYear(CPInteger Month, CPInteger Day, CPInteger Year);

	/// @brief     Sets the date instance to the specified month, day, and year. [DEPRECATED]
	/// @details   You can use this object when you want a specific date set for the existing instance of the date object.
	/// @param     Month  The month, identified as a number from 1 to 12.
	/// @param     Day  The day of the month.
	/// @param     Year  The year, identifed as a complete number (includes century).
	/// @param     Hour  The hour, identifed as a number in the range from 0 to 23 (military time).
	/// @param     Minute  The minute, identifed as a number in the range from 0 to 59.
	/// @param     Second  The second, identifed as a decimal number in the range from 0.000 to 59.999.
	void SetFromMonthDayYearHourMinuteSecond(CPInteger Month, CPInteger Day, CPInteger Year, CPInteger Hour, CPInteger Minute, double Second);

	/// @brief     Sets the date instance to the specified month, day, and year. [DEPRECATED]
	/// @details   You can use this object when you want a specific date set for the existing instance of the date object.
	/// @param     Month  The month, identified as a number from 1 to 12.
	/// @param     Day  The day of the month.
	/// @param     Year  The year, identifed as a complete number (includes century).
	/// @param     Hour  The hour, identifed as a number in the range from 0 to 23 (military time).
	/// @param     Minute  The minute, identifed as a number in the range from 0 to 59.
	/// @param     Second  The second, identifed as a decimal number in the range from 0.000 to 59.999.
	void SetFromMonthDayYearHourMinuteSecondGMT(CPInteger Month, CPInteger Day, CPInteger Year, CPInteger Hour, CPInteger Minute, double Second);

	/// @brief     Gives the description of the current date instance.
	///
	/// @details   This method gives the description as a string of the instance of this date object. Use for Debugging purposes only.
	///
	/// @return    A string representing the description of the date.
	///
	CPString Description(void) const;

#if __OBJC__
	///	@brief     Creates a date object that is the same as the date passed in.
	///
	///	@details   This copy is accomplished by using the same underlying date object as the date passed in.
	///
	///	@param     Date  A date to copy.
	///
	CPDate(NSDate* Date);
	
	/// @brief     Assignment of a date object to another date object.
	///
	/// @details   Assigns a date object to another date object.
	///
	/// @param	   Date  An underlying date object to copy.
	///
	const CPDate& operator=(NSDate* Date);

	///	@brief     Conversion Operator.
	///
	///	@details   Returns the underlying object as a NSDate*.
	///
	operator NSDate*(void) const;

	/// @brief     Equals Operator.
	///
	/// @details   This checks to see if the date object is equal to the same date and time contained in the compared date object.
	///
	/// @param	   Date  An underlying date object used in the compare.
	///
	/// @return    A bool set to true if the date and time are the same, otherwise it is false.
	///
	bool operator==(NSDate* Date) const;
	
	/// @brief     Not Equals Operator.
	///
	/// @details   This checks to see if the date object is not equal to the same date and time contained in the compared date object.
	///
	/// @param	   Date  An underlying date object used in the compare.
	///
	/// @return    A bool set to true if the date and time are NOT the same, otherwise it is false.
	///
	bool operator!=(NSDate* Date) const;
    
	/// @brief     Greater Than Operator.
	///
	/// @details   This checks to see if the date object is greater (later) than the date and time contained in the compared date object.
	///
	/// @param	   Date  An underlying date object used in the compare.
	///
	/// @return    A bool set to true if the date and time are later than the object's date and time being compared,
	///            otherwise it is false.
	///
	bool operator>(NSDate* Date) const;
	
	/// @brief     Less Than Operator.
	///
	/// @details   This checks to see if the date object is less (earlier) than the date and time contained in the compared date object.
	///
	/// @param	   Date  An underlying date object used in the compare.
	///
	/// @return    A bool set to true if the date and time are earlier than the object's date and time being compared,
	///            otherwise it is false.
	///
	bool operator<(NSDate* Date) const;
	
	/// @brief     Greater Than or Equals Operator.
	///
	/// @details   This checks to see if the date object is greater (later) than or the same as the date and time contained in the
	///            compared date object.
	///
	/// @param	   Date  An underlying date object used in the compare.
	///
	/// @return    A bool set to true if the date and time are later than or the same as the object's date and time being compared,
	///            otherwise it is false.
	///
	bool operator>=(NSDate* Date) const;
	
	/// @brief     Less Than or Equals Operator.
	///
	/// @details   This checks to see if the date object is less (earlier) than or the same as the date and time contained in the
	///            compared date object.
	///
	/// @param	   Date  An underlying date object used in the compare.
	///
	/// @return    A bool set to true if the date and time are earlier than or the same as the object's date and time being compared,
	///            otherwise it is false.
	///
	bool operator<=(NSDate* Date) const;
#endif
#if __COREFOUNDATION__
	/// @brief     Copy Constructor.
	///
	/// @details   This object will contain the object of the specified date.
	///
	///	@param     Date  the date to copy.
	///
	CPDate(CFDateRef Date);

	///	@brief     Copy Operator.
	///
	///	@details   Copies the entries from the given date.
	///
	///	@param	   Date  the date to copy.
	///
	const CPDate& operator=(CFDateRef Date);
	
	/// @brief     Equals Operator.
	///
	/// @details   This checks to see if the date object is equal to the same date and time contained in the compared date object.
	///
	/// @param	   Date  An underlying date object used in the compare.
	///
	/// @return    A bool set to true if the date and time are the same, otherwise it is false.
	///
	bool operator==(CFDateRef Date) const;

	///	@brief     Conversion Operator.
	///
	///	@details   Returns the underlying object as a CFDateRef*.
	///
	operator CFDateRef(void) const;
#endif

};
