////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPDateComponents.h
//      Cross platform date components class.
//
//  Copyright © 2015 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  04/11/15  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/ExternalAPIMacros.h"
#include "CrossPlatform/CPInteger.h"
#include "CrossPlatform/CPEnum.h"
#include "CrossPlatform/CPObject.h"
#include "CrossPlatform/CPString.h"
#include "CrossPlatform/CPDate.h"
#include "CrossPlatform/CPCalendar.h"
#include "CrossPlatform/CPMutableDictionary.h"

typedef CP_ENUM(CPInteger, CPWeekday)
{
	/// Weekday Sunday value
	CPWeekdaySunday = 1,
	/// Weekday Monday value
	CPWeekdayMonday,
	/// Weekday Tuesday value
	CPWeekdayTuesday,
	/// Weekday Wednesday value
	CPWeekdayWednesday,
	/// Weekday Thursday value
	CPWeekdayThursday,
	/// Weekday Friday value
	CPWeekdayFriday,
	/// Weekday Saturday value
	CPWeekdaySaturday
};

/// @brief   Cross platform date components class.
///
/// @details CPDateComponents encapsulates the components of a date in an extendable, object-oriented manner. It is used to specify a date by
///          providing the temporal components that make up a date and time: hour, minutes, seconds, day, month, year, and so on. It can also
///          be used to specify a duration of time, for example, 5 hours and 16 minutes. An CPDateComponents object is not required to define
///          all the component fields. When a new instance of CPDateComponents is created the date components are set to
///          CPUndefinedDateComponent.
///          An CPDateComponents object is meaningless in itself; you need to know what calendar it is interpreted against, and you need to
///          know whether the values are absolute values of the units, or quantities of the units.
///
///          An instance of CPDateComponents is not responsible for answering questions about a date beyond the information with which it was
///          initialized. For example, if you initialize one with May 6, 2004, its weekday is NSUndefinedDateComponent, not Thursday. To get
///          the correct day of the week, you must create a suitable instance of CPCalendar, create an CPDate object using DateFromComponents()
///          and then use ComponentsFromDate() to retrieve the weekday—as illustrated in the following example.
///
///          CPDateComponents comps;
///          comps.SetDay(6);
///          comps.SetMonth(5);
///          comps.SetYear(2004);
///          CPCalendar gregorian = CPCalendar::GregorianCalendar();
///          CPDate date = gregorian.DateFromComponents(comps);
///          CPDateComponents weekdayComponents = gregorian.ComponentsFromDate(CPWeekdayCalendarUnit, date);
///          int weekday = weekdayComponents.Weekday();
///
class CP_EXTERNAL_API CPDateComponents : public CPObject
{
public:
	///	@brief    The value of an undefined component. 
	///
	///	@return   Returns the value used for undefined components.
	///
	static CPInteger CPDateComponentUndefined(void);

	///	@brief     A class type that uniquely identifies this class.
	///
	///	@return    Returns a class type that uniquely identifies this class.
	///
	static CPClassType ClassType(void);

	/// @brief    Returns the underlying shared empty object.
	///
	/// @details  Static method that returns the underlying object that is shared between all empty objects of this class type.
	///
	/// @returns  the underlying shared empty object.
	///
	static const CPId& SharedEmptyObject(void);

	/// @brief    Returns a date components object with the year, month and day set.
	///
	///	@param     Year  the value for the year component.
	///	@param     Month the value for the month component.
	///	@param     Day   the value for the day component.
	///
	/// @returns  a date components object with the year, month and day set.
	///
	static CPDateComponents ComponentsWithYearMonthAndDay(CPInteger Year, CPInteger Month, CPInteger Day);

	/// @brief    Returns a date components object with the year, month, day, hour, minute, second and nanosecond set.
	///
	///	@param     Year  the value for the year component.
	///	@param     Month  the value for the month component.
	///	@param     Day  the value for the day component.
	///	@param     Hour  the value for the hour component.
	///	@param     Minute  the value for the minute component.
	///	@param     Second  the value for the second component.
	///	@param     Nanosecond  the value for the nanosecond component.
	///
	/// @returns  a date components object with the year, month, day, hour, minute, second and nanosecond set.
	///
	static CPDateComponents ComponentsWithYearMonthDayHourMinuteSecondAndNanosecond(CPInteger Year, CPInteger Month, CPInteger Day,
		CPInteger Hour, CPInteger Minute, CPInteger Second, CPInteger Nanosecond);

	/// @brief    Returns a date components object with the day set.
	///
	///	@param     Day  the value for the day component.
	///
	/// @returns  a date components object with the day set.
	///
	static CPDateComponents ComponentsWithDay(CPInteger Day);

	/// @brief    Returns a date components object with the second set.
	///
	///	@param     Second  the value for the second component.
	///
	/// @returns  a date components object with the second set.
	///
	static CPDateComponents ComponentsWithSecond(CPInteger Second);

	/// @brief     Default Constructor.
	///
	/// @details   Creates an empty date components.
	///
	CPDateComponents(void);

	/// @brief     Copy Constructor.
	///
	/// @details   This object will contain the object of the specified date components.
	///
	///	@param     DateComponents  the date components to copy.
	///
	CPDateComponents(const CPDateComponents& DateComponents);

	///	@brief     Destructor.
	///
	///	@details   deallocates any unused resources.
	///
	~CPDateComponents(void);
	
	/// @brief     Assignment of a date components object to another date components object.
	///
	/// @details   Assigns a date components object to another date components object.
	///
	/// @param	   DateComponents  An underlying date components object to copy.
	///
	const CPDateComponents& operator=(const CPDateComponents& DateComponents);

	/// @brief     Returns the year component.
	///
	/// @details   Returns the year component. Will return CPDateComponentUndefined() if the component has not been set.
	///
	/// @return	   the year component.
	///
	CPInteger Year(void) const;

	/// @brief     Sets the year component.
	///
	/// @param	   Year  the value the set the year component.
	///
	void SetYear(CPInteger Year);

	/// @brief     Returns the month component.
	///
	/// @details   Returns the month component. Will return CPDateComponentUndefined() if the component has not been set.
	///
	/// @return	   the month component.
	///
	CPInteger Month(void) const;

	/// @brief     Sets the month component.
	///
	/// @param	   Month  the value the set the month component.
	///
	void SetMonth(CPInteger Month);

	/// @brief     Returns the day component.
	///
	/// @details   Returns the day component. Will return CPDateComponentUndefined() if the component has not been set.
	///
	/// @return	   the day component.
	///
	CPInteger Day(void) const;

	/// @brief     Sets the day component.
	///
	/// @param	   Day  the value the set the day component.
	///
	void SetDay(CPInteger Day);

	/// @brief     Returns the hour component.
	///
	/// @details   Returns the hour component. Will return CPDateComponentUndefined() if the component has not been set.
	///
	/// @return	   the hour component.
	///
	CPInteger Hour(void) const;

	/// @brief     Sets the hour component.
	///
	/// @param	   Hour  the value the set the hour component.
	///
	void SetHour(CPInteger Hour);

	/// @brief     Returns the minute component.
	///
	/// @details   Returns the minute component. Will return CPDateComponentUndefined() if the component has not been set.
	///
	/// @return	   the minute component.
	///
	CPInteger Minute(void) const;

	/// @brief     Sets the minute component.
	///
	/// @param	   Minute  the value the set the minute component.
	///
	void SetMinute(CPInteger Minute);

	/// @brief     Returns the second component.
	///
	/// @details   Returns the second component. Will return CPDateComponentUndefined() if the component has not been set.
	///
	/// @return	   the second component.
	///
	CPInteger Second(void) const;

	/// @brief     Sets the second component.
	///
	/// @param	   Second  the value the set the second component.
	///
	void SetSecond(CPInteger Second);

	/// @brief     Returns the nanosecond component.
	///
	/// @details   Returns the nanosecond component. Will return CPDateComponentUndefined() if the component has not been set.
	///
	/// @return	   the nanosecond component.
	///
	CPInteger Nanosecond(void) const;

	/// @brief     Sets the nanosecond component.
	///
	/// @param	   Nanosecond  the value the set the nanosecond component.
	///
	void SetNanosecond(CPInteger Nanosecond);

	/// @brief     Returns the weekday component.
	///
	/// @details   Returns the weekday component. Will return CPDateComponentUndefined() if the component has not been set.
	///
	/// @return	   the weekday component.
	///
	CPInteger Weekday(void) const;

	/// @brief     Sets the weekday component.
	///
	/// @param	   Weekday  the value the set the weekday component.
	///
	void SetWeekday(CPInteger Weekday);

	/// @brief     Returns the leap year component.
	///
	/// @details   Returns the leap year component. This component is currently only set via getting components from a date.
	///
	/// @return	   the leap year component, will return false if it was not set.
	///
	bool IsLeapMonth(void) const;

#if __OBJC__
	/// @brief     Copy Constructor.
	///
	/// @details   This object will contain the object of the specified gregorian date.
	///
	///	@param     DateComponents  the date components to copy.
	///
	CPDateComponents(NSDateComponents* DateComponents);

	///	@brief     Copy Operator.
	///
	///	@details   Copies the entries from the given gregorian date.
	///
	///	@param     DateComponents  the date components to copy.
	///
	const CPDateComponents& operator=(NSDateComponents* DateComponents);
	
	///	@brief     Conversion Operator.
	///
	///	@details   Returns an equivalent NSDateComponents*.
	///
	operator NSDateComponents*(void) const;
#endif
#if __COREFOUNDATION__
	/// @brief     Copy Constructor.
	///
	/// @details   This object will contain the object of the specified gregorian date.
	///
	///	@param     GregorianDate  the gregorian date to copy.
	///
	CPDateComponents(CFGregorianDate GregorianDate);

	///	@brief     Copy Operator.
	///
	///	@details   Copies the entries from the given gregorian date.
	///
	///	@param	   GregorianDate  the gregorian date to copy.
	///
	const CPDateComponents& operator=(CFGregorianDate GregorianDate);

	///	@brief     Conversion Operator.
	///
	///	@details   Returns an equivalent CFGregorianDate.
	///
	operator CFGregorianDate(void) const;
#endif

private:
	void EnsureModifiable(void);

};
