////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPDateFormatter.h
//    Cross-platform date formatter class.
//
//  Copyright © 2009-2012 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  06/09/10  G.R.W.  Added method DateFromMonthDayYearHourMinuteSecond and SetFromMonthDayYearHourMinuteSecond.
//  03/25/09  K.L.S.  Created file.
// 
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/CPObject.h"
#include "CrossPlatform/CPString.h"
#include "CrossPlatform/CPTimeZone.h"

/// @class  CPDateFormatter
/// @brief  CPDateFormatter objects are used to convert dates to and from strings.
class CP_EXTERNAL_API CPDateFormat
{
public:
	/// @brief     Retrieve format string ddMMyyyy.
	///
	///	@details   This static method retrieves the date format string for representing the date as ddMMyyyy.
	///
	///	@return    A string that can be used in the formatting of the date and time.
	///
	static CPString ddMMyyyyDateFormat(void);

	/// @brief     Retrieve format string MMddyyyy.
	///
	///	@details   This static method retrieves the date format string for representing the date as MMddyyyy.
	///
	///	@return    A string that can be used in the formatting of the date and time.
	///
	static CPString MMddyyyyDateFormat(void);
	
	/// @brief     Retrieve format string MMddyyyyHHmmssSSSS.
	///
	///	@details   This static method retrieves the date format string for representing the date as MMddyyyyHHmmssSSSS.
	///
	///	@return    A string that can be used in the formatting of the date and time.
	///
	static CPString MMddyyyyHHmmssSSSDateFormat(void);
	
	/// @brief     Retrieve format string yyyyMMdd.
	///
	///	@details   This static method retrieves the date format string for representing the date as yyyyMMdd.
	///
	///	@return    A string that can be used in the formatting of the date and time.
	///
	static CPString yyyyMMddDateFormat(void);
	
	/// @brief     Retrieve format string HHmmss.
	///
	///	@details   This static method retrieves the date format string for representing the date as HHmmss.
	///
	///	@return    A string that can be used in the formatting of the date and time.
	///
	static CPString HHmmssDateFormat(void);
	
	/// @brief     Retrieve format string HHmmssSSSS.
	///
	///	@details   This static method retrieves the date format string for representing the date as HHmmssSSSS.
	///
	///	@return    A string that can be used in the formatting of the date and time.
	///
	static CPString HHmmssSSSDateFormat(void);
	
	/// @brief     Retrieve format string HHmm.
	///
	///	@details   This static method retrieves the date format string for representing the date as HHmm.
	///
	///	@return    A string that can be used in the formatting of the date and time.
	///
	static CPString HHmmDateFormat(void);
};

/// @brief      CPDateFormatter convert dates to and from strings.
///
/// @details    To parse strings containing dates and to generate string representations of a date, you should use an instance of
///             CPDateFormatter using the methods DateFromString() and StringFromDate() respectively.
///
class CP_EXTERNAL_API CPDateFormatter : public CPObject
{
public:
	enum Enum_CPDateFormatterStyle
	{	
		CPDateFormatterNoStyle = 0,
		CPDateFormatterShortStyle = 1,
		CPDateFormatterMediumStyle = 2,
		CPDateFormatterLongStyle = 3,
		CPDateFormatterFullStyle = 4
	};

	typedef int CPDateFormatterStyle;

	///	@brief     A class type that uniquely identifies this class.
	///
	///	@return    Returns a class type that uniquely identifies this class.
	///
	static CPClassType ClassType(void);

	///	@brief     Retrieves the date formatter for representing the date in GMT.
	///
	///	@return    An intsance of the date formatter that can be used in the formatting of the date and time.
	///
	static CPDateFormatter GMTDateFormatter(void);
	
	///	@brief     Retrieves the date formatter for representing the date in the current locale.
	///
	///	@return    An intsance of the date formatter that can be used in the formatting of the date and time.
	///
	static CPDateFormatter CurrentLocaleDateFormatter(void);
	
	///	@brief     Retrieves the date formatter for representing the default formatter.
	///
	///	@return    An intsance of the date formatter that can be used in the formatting of the date and time.
	///
	static CPDateFormatter DefaultFormatter(void);
	
	///	@brief     Retrieves the date formatter for representing the date format.
	///
	///	@return    An intsance of the date formatter that can be used in the formatting of the date and time.
	///
	static CPDateFormatter DateFormatterUsingDateFormat(CPString dateFormat);
	
	///	@brief     Retrieves the date formatter for representing the date format using the date formatting style.
	///
	///	@return    An intsance of the date formatter that can be used in the formatting of the date and time.
	///
	static CPDateFormatter DateFormatterUsingDateFormatDateStyle(CPString dateFormat, CPDateFormatterStyle dateStyle);
	
	///	@brief     Retrieves the date formatter for representing the date format using the date formatting style and the time style.
	///
	///	@return    An intsance of the date formatter that can be used in the formatting of the date and time.
	///
	static CPDateFormatter DateFormatterUsingDateFormatDateStyleTimeStyle(CPString dateFormat, CPDateFormatterStyle dateStyle,
		CPDateFormatterStyle timeStyle);

	/// @brief     Default Constructor.
	///
	/// @details   Creates an empty date formatter.
	///
	CPDateFormatter(void);
	
	/// @brief     Copy Constructor.
	///
	/// @details   This will contain the object of the specified date formatter.
	///
	///	@param     DateFormatter  the date formatter to copy.
	///
	CPDateFormatter(const CPDateFormatter& DateFormatter);
	
	/// @brief     Copy Constructor.
	///
	/// @details   This will contain the id of the object of the specified date formatter.
	///
	///	@param     Id  the id of the date formatter object to copy.
	///
	explicit CPDateFormatter(const CPId& Id);
	
	///	@brief     Destructor.
	///
	///	@details   Removes any references it has to shared resources.
	///
	~CPDateFormatter(void);

	///	@brief     Copy Operator.
	///
	/// @details   This will copy the object of the specified date formatter.
	///
	///	@param     DateFormatter  the date formatter to copy.
	///
	const CPDateFormatter& operator=(const CPDateFormatter& DateFormatter);
	
	///	@brief     Set the Date Formatter.
	///
	/// @details   This will set the date formatter so that the format of a date retrieved from future calls will return a date of the
	///            given format.
	///
	///	@param     DateFormat  the string of the date format to use for retrieving future date format calls.
	///
	void SetDateFormat(const CPString& DateFormat);

	/// @brief     The time zone for this formatter.
	///
	/// @return    the time zone for this formatter.
	///
	CPTimeZone TimeZone(void) const;

	///	@brief     Set the time zone to use when formatting.
	///
	/// @details   time zone used when formatting as a date will have differnt component values depending on the time zone.
	///
	///	@param     TimeZone  time zone to use when formatting.
	///
	void SetTimeZone(const CPTimeZone& TimeZone);

	/// @brief     Returns an CPDate object with the value specified by a given string in the string representation
	///            format (MM-DD-YYYY HH:MM:SS) or (MM-DD-YYYY).
	///
	/// @details   You can use this object when you want a specific date and time components.
	///
	/// @param     DateString  A string that specifies a date or a date and time value.
	///
	/// @return    A CPDate object representing the date and time specified.
	///
	CPDate DateFromString(const CPString& DateString) const;

	//CPDate DateFromStringUsingFormat(const CPString& DateString, const CPString& DateFormat const);
	// No format methods use mm-dd-yyyy format. As this default could cause problems UI code should already convert strings to native date 
	// object; ie: NSDate.  May need to look back at this decision if it proves to be to much of a headache for others to deal with.

	///	@brief     Retrieves the string of the date given.
	///
	/// @details   This will retrieve the string of the date using the previously set format (if there was one).
	///
	///	@param     Date  the date to be converted.
	///
	CPString StringFromDate(const CPDate& Date) const;
	
#if __OBJC__
	/// @brief     Copy Constructor.
	///
	/// @details   This will contain the object of the specified date formatter.
	///
	///	@param     DateFormatter  the date formatter to copy.
	///
	CPDateFormatter(NSDateFormatter* DateFormatter);

	///	@brief     Copy Operator.
	///
	/// @details   This will copy the object of the specified date formatter.
	///
	///	@param     DateFormatter  the date formatter to copy.
	///
	const CPDateFormatter& operator=(NSDateFormatter* DateFormatter);
#endif

private:
	CPDateFormatter(CPDateFormatterStyle DateStyle, CPDateFormatterStyle TimeStyle, const CPTimeZone& TimeZone);
	CPString m_DF;
	CPTimeZone m_TZ;
};
