////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPDictionary.h
//    Cross platform dictionary class.
//
//  Copyright © 2009-2013 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  07/30/13  B.D.D.  Added MutableArrayForKey, MutableDataForKey, MutableDictionaryForKey.
//  02/10/12  K.L.S.  Changes for CrossPlatform API changes. Reworked header includes.
//	10/20/09  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/ExternalAPIMacros.h"
#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/CPUInteger.h"
#include "CrossPlatform/CPObject.h"

class CPMutableDictionary;
class CPBool;
class CPString;
class CPArray;
class CPMutableArray;
class CPData;
class CPMutableData;
class CPEnumerator;
class CPDate;
class CPNumber;
class CPValue;
class CPRetainedObject;

/// @brief   Cross platform dictionary class.
/// @details The CPDictionary class declares the programmatic interface to objects that manage associations of keys and values. 
///          Use this class or its subclass CPMutableDictionary when you need a convenient and efficient way to retrieve data associated 
///          with an arbitrary key. CPDictionary creates static dictionaries, and CPMutableDictionary creates dynamic dictionaries. (For 
///          convenience, we use the term dictionary to refer to any instance of one of these classes without specifying its exact class 
//           membership.)
///
///          A key-value pair within a dictionary is called an entry. Each entry consists of one object that represents the key and a 
//           second object that is that key’s value. Within a dictionary, the keys are unique. That is, no two keys in a single dictionary 
///          are equal (as determined by isEqual:). In general, a key can be any object, but note that when using key-value coding the key 
///          must be a string.  Neither a key nor a value can be nil; if you need to represent a null value in a dictionary, you should 
//           use CPNull.
/// 
/// For more information:
/// http://developer.apple.com/library/mac/#documentation/Cocoa/Reference/Foundation/Classes/NSDictionary_Class
/// http://developer.apple.com/library/mac/#documentation/CoreFoundation/Reference/CFDictionaryRef
///
class CP_EXTERNAL_API CPDictionary : public CPObject
{
public:
	///	@brief     A class type that uniquely identifies this class.
	///	@return    Returns a class type that uniquely identifies this class.
	static CPClassType ClassType(void);

	/// @brief    Returns the underlying shared empty object.
	/// @details  Static method that returns the underlying object that is shared between all empty objects of this class type.
	/// @returns  the underlying shared empty object.
	static const CPId& SharedEmptyObject(void);

	///	@brief     Return a dictionary instance based on the dictionary contents of the file.
	///	@details   This static method creates and returns an dictionary containing the contents of the file whose name was passed 
	///            into this method.
	///	@param     FilePath  The path and filename of the file that contains the dictionary entries.
	///	@return    This returns an instance of dictionary.
	static CPDictionary DictionaryWithContentsOfFile(const CPString& FilePath);

	///	@brief     Return a dictionary instance based on the key-object pair passed in.
	///	@details   This static method creates and returns an dictionary containing a single entry that is the key-object pair passed
	///            into this method.
	///	@param     Object  The object used in creating the dictionary entry.
	///	@param     Key  The key used in creating the dictionary entry.
	///	@return    This returns an instance of a dictionary.
	static CPDictionary DictionaryWithObjectForKey(const CPObject& Object, const CPString& Key);

	///	@brief     Return a dictionary instance based on the key-object pairs passed in.
	///	@details   This static method creates and returns an dictionary containing multiple entries based on the array of keys and 
	///            the array of objects passed into this method.
	///	@param     ObjectArray  The array of objects used in creating the dictionary entries.
	///	@param     KeyArray  The array of keys used in creating the dictionary entries.
	///	@return    This returns an instance of dictionary.
	static CPDictionary DictionaryWithObjectsForKeys(const CPArray& ObjectArray, const CPArray& KeyArray);

	static CPDictionary DictionaryFromObject(const CPObject& Object);
	static CPDictionary DictionaryFromId(const CPId& Object);

	/// @brief     Creates a dictionary object that is empty.
	/// @details   This object does not contain any objects.
	CPDictionary(void);

	/// @brief     Creates a dictionary object containing the propertyList as a dictionary object.
	/// @details   This object contains the same keys and objects that were passed as a dictionary into the constructor.
	///	@param     Dictionary  A dictionary to copy.
	CPDictionary(const CPDictionary& Dictionary);

	/// @brief     Creates a dictionary object containing the data object.
	/// @details   This creates a dictionary object containing the property list as a data object.
	///	@param     String  A string that contains a property list used to create the dictionary.
	explicit CPDictionary(const CPString& String);

	/// @brief     Creates a dictionary object containing the data object.
	/// @details   This creates a dictionary object containing the property list as a data object.
	///	@param     Data  A data object that contains a property list used to create the dictionary.
	explicit CPDictionary(const CPData& Data);

	///	@brief     Creates a dictionary object that is the same as the dictionary passed in.
	///	@details   This copy uses same underlying dictionary object as the dictionary passed in.
	///	@param     Dictionary  An object to copy if it is really a dictionary or mutable Dictionary.
	explicit CPDictionary(const CPObject& Dictionary);

	///	@brief     Creates a dictionary object that is the same as the dictionary passed in.
	///	@details   This copy uses the same underlying dictionary object as the dictionary's ID passed in.
	///	@param     Data  An id to copy if it is really a dictionary or mutable Dictionary.
	explicit CPDictionary(const CPId& Data);

	///	@brief     Deallocates the array object. Removes its reference to its underlying object.
	///	@details   Removes its reference to its underlying object.
	~CPDictionary(void);

	///	@brief     Copy Operator.
	///	@details   Assigns a dictionary object to another dictionary object.
	///	@param     Dictionary  An underlying dictionary object to copy.
	const CPDictionary& operator=(const CPDictionary& Dictionary);
	
	///	@brief     assignment Operator.
	/// @details   Assigns a data object containing the property list as a data object to the existing dictionary.
	///	@param     Data  A data object that contains a property list used to assign the dictionary.
	const CPDictionary& operator=(const CPData& Data);

	///	@brief     assignment Operator.
	/// @details   Assigns a string object containing the property list as a data object to the existing dictionary.
	///	@param     String  A string that contains a property list used to assign the dictionary.
	const CPDictionary& operator=(const CPString& String);

	///	@brief     Checks to see if the dictionary object is empty.
	///	@details   This checks to see if the dictionary object is empty and returns its status.
	///	@return    A bool set to true if the object was empty, otherwise it is false.
	bool IsEmpty(void) const;

	///	@brief     Gets the count of the objects.
	///	@details   Retrieves the count of the number of key-object pairs presently in the dictionary.
	///	@return    An integer respresenting the count of the key-object pairs in the dictionary.
	CPUInteger Count(void) const;

	///	@brief     Returns true if the object is the class type for the specified key, otherwise false.
	///	@details   Checks to see if the existing object for the passed key in the dictionary matches the passed class type.  If so, then 
	///            true is returned, otherwise false.
	///	@param     ClassTypeOfObject  A string of the class type of the object to be checked.
	///	@param     Key  The key for which to return if the corresponding object is the type specified.
	///	@return    true if the class type matches, otherwise false.
	bool IsMatchingClassTypeOfObjectForKey(CPClassType ClassTypeOfObject, CPString Key) const;

	///	@brief     Returns the object type of the object that matches the key.
	///	@details   Returns the object type as a string from the object that matches the key in the dictionary.
	///	@param     Key  The key for which to return the corresponding objects type.
	///	@return    the string identifying the object type for the key in the dictionary.
	CPClassType ClassTypeOfObjectForKey(CPString Key) const;

	///	@brief     Gets the object for the specified key.
	///	@details   Retrieves the object for the specified key in the dictionary.
	///	@param     Key  The key for which to return the corresponding object.
	///	@return    The object with the specified key.
	CPObject ObjectForKey(const CPString& Key) const;

	///	@brief     Gets the object for the specified key.
	///	@details   Retrieves the object for the specified key. A null object is returned if the key does not exist. (IsNull() == true)
	///	@param     Key  The key for which to return the corresponding object.
	///	@return    The object with the specified key.
	CPObject ObjectForExistingKey(const CPString& Key) const;

	///	@brief     Gets the array for the specified key.
	///	@details   Retrieves the array for the specified key in the dictionary.
	///	@param     Key  The key for which to return the corresponding array.
	///	@return    The array with the specified key.
	CPArray ArrayForKey(const CPString& Key) const;
	CPArray ArrayForKey(const CPString& Key, CPError& Error) const;

	///	@brief     Gets the array for the specified key.
	///	@details   Retrieves the array for the specified key. A null object is returned if the key does not exist. (IsNull() == true)
	///	@param     Key  The key for which to return the corresponding array.
	///	@return    The array with the specified key.
	CPArray ArrayForExistingKey(const CPString& Key) const;
	CPArray ArrayForExistingKey(const CPString& Key, CPError& Error) const;

	///	@brief     Gets the array for the specified key.
	///	@details   Retrieves the array for the specified key in the dictionary.
	///	@param     Key  The key for which to return the corresponding array.
	///	@return    The array with the specified key.
	CPMutableArray MutableArrayForKey(const CPString& Key) const;
	CPMutableArray MutableArrayForKey(const CPString& Key, CPError& Error) const;

	///	@brief     Gets the array for the specified key.
	///	@details   Retrieves the array for the specified key. A null object is returned if the key does not exist. (IsNull() == true)
	///	@param     Key  The key for which to return the corresponding array.
	///	@return    The array with the specified key.
	CPMutableArray MutableArrayForExistingKey(const CPString& Key) const;
	CPMutableArray MutableArrayForExistingKey(const CPString& Key, CPError& Error) const;

	///	@brief     Gets the boolean for the specified key.
	///	@details   Retrieves the boolean for the specified key in the dictionary.
	///	@param     Key  The key for which to return the corresponding boolean.
	///	@return    The boolean with the specified key.
	CPBool BoolForKey(const CPString& Key) const;
	CPBool BoolForKey(const CPString& Key, CPError& Error) const;

	///	@brief     Gets the boolean for the specified key.
	///	@details   Retrieves the boolean for the specified key in the dictionary.
	///	@param     Key  The key for which to return the corresponding boolean.
	///	@return    The boolean with the specified key.
	CPBool BoolForExistingKey(const CPString& Key) const;
	CPBool BoolForExistingKey(const CPString& Key, CPError& Error) const;

	///	@brief     Gets the data for the specified key.
	///	@details   Retrieves the data object for the specified key in the dictionary.
	///	@param     Key  The key for which to return the corresponding data.
	///	@return    The data with the specified key.
	CPData DataForKey(const CPString& Key) const;
	CPData DataForKey(const CPString& Key, CPError& Error) const;

	///	@brief     Gets the data for the specified key.
	///	@details   Retrieves the data for the specified key. A null object is returned if the key does not exist. (IsNull() == true)
	///	@param     Key  The key for which to return the corresponding data.
	///	@return    The data with the specified key.
	CPData DataForExistingKey(const CPString& Key) const;
	CPData DataForExistingKey(const CPString& Key, CPError& Error) const;

	///	@brief     Gets the data for the specified key.
	///	@details   Retrieves the data for the specified key.
	///	@param     Key  The key for which to return the corresponding data.
	///	@return    The data with the specified key.
	CPMutableData MutableDataForKey(const CPString& Key) const;
	CPMutableData MutableDataForKey(const CPString& Key, CPError& Error) const;

	///	@brief     Gets the data for the specified key.
	///	@details   Retrieves the data for the specified key. A null object is returned if the key does not exist. (IsNull() == true)
	///	@param     Key  The key for which to return the corresponding data.
	///	@return    The data with the specified key.
	CPMutableData MutableDataForExistingKey(const CPString& Key) const;
	CPMutableData MutableDataForExistingKey(const CPString& Key, CPError& Error) const;

	///	@brief     Gets the date for the specified key.
	///	@details   Retrieves the date for the specified key.
	///	@param     Key  The key for which to return the corresponding date.
	///	@return    The date with the specified key.
	CPDate DateForKey(const CPString& Key) const;
	CPDate DateForKey(const CPString& Key, CPError& Error) const;

	///	@brief     Gets the date for the specified key.
	///	@details   Retrieves the date for the specified key. A null object is returned if the key does not exist. (IsNull() == true)
	///	@param     Key  The key for which to return the corresponding date.
	///	@return    The date with the specified key.
	CPDate DateForExistingKey(const CPString& Key) const;
	CPDate DateForExistingKey(const CPString& Key, CPError& Error) const;

	///	@brief     Gets the dictionary for the specified key.
	///	@details   Retrieves the dictionary for the specified key in the dictionary.
	///	@param     Key  The key for which to return the corresponding dictionary.
	///	@return    The dictionary with the specified key.
	CPDictionary DictionaryForKey(const CPString& Key) const;
	CPDictionary DictionaryForKey(const CPString& Key, CPError& Error) const;

	///	@brief     Gets the dictionary for the specified key.
	///	@details   Retrieves the dictionary for the specified key. A null object is returned if the key does not exist. (IsNull() == true)
	///	@param     Key  The key for which to return the corresponding dictionary.
	///	@return    The dictionary with the specified key.
	CPDictionary DictionaryForExistingKey(const CPString& Key) const;
	CPDictionary DictionaryForExistingKey(const CPString& Key, CPError& Error) const;

	///	@brief     Gets the dictionary for the specified key.
	///	@details   Retrieves the dictionary for the specified key in the dictionary.
	///	@param     Key  The key for which to return the corresponding dictionary.
	///	@return    The dictionary with the specified key.
	CPMutableDictionary MutableDictionaryForKey(const CPString& Key) const;
	CPMutableDictionary MutableDictionaryForKey(const CPString& Key, CPError& Error) const;

	///	@brief     Gets the dictionary for the specified key.
	///	@details   Retrieves the dictionary for the specified key. A null object is returned if the key does not exist. (IsNull() == true)
	///	@param     Key  The key for which to return the corresponding dictionary.
	///	@return    The dictionary with the specified key.
	CPMutableDictionary MutableDictionaryForExistingKey(const CPString& Key) const;
	CPMutableDictionary MutableDictionaryForExistingKey(const CPString& Key, CPError& Error) const;

	///	@brief     Gets the number for the specified key.
	///	@details   Retrieves the number for the specified.
	///	@param     Key  The key for which to return the corresponding number.
	///	@return    The number with the specified key.
	CPNumber NumberForKey(const CPString& Key) const;
	CPNumber NumberForKey(const CPString& Key, CPError& Error) const;

	///	@brief     Gets the number for the specified key.
	///	@details   Retrieves the number for the specified key. A null object is returned if the key does not exist. (IsNull() == true)
	///	@param     Key  The key for which to return the corresponding number.
	///	@return    The number with the specified key.
	CPNumber NumberForExistingKey(const CPString& Key) const;
	CPNumber NumberForExistingKey(const CPString& Key, CPError& Error) const;

	///	@brief     Gets the retained object for the specified key.
	///	@details   Retrieves the retained object for the specified key in the dictionary.
	///	@param     Key  The key for which to return the corresponding object.
	///	@return    The retained object that was in the dictionary at the specified key.
	CPRetainedObject* RetainedObjectForKey(const CPString& Key) const;
	CPRetainedObject* RetainedObjectForKey(const CPString& Key, CPError& Error) const;

	///	@brief     Gets the string for the specified key.
	///	@details   Retrieves the string for the specified key.
	///	@param     Key  The key for which to return the corresponding string.
	///	@return    The string with the specified key.
	CPString StringForKey(const CPString& Key) const;
	CPString StringForKey(const CPString& Key, CPError& Error) const;

	///	@brief     Gets the string for the specified key.
	///	@details   Retrieves the string for the specified key. A null object is returned if the key does not exist. (IsNull() == true)
	///	@param     Key  The key for which to return the corresponding string.
	///	@return    The string with the specified key.
	CPString StringForExistingKey(const CPString& Key) const;
	CPString StringForExistingKey(const CPString& Key, CPError& Error) const;

	///	@brief     Gets the value for the specified key.
	///	@details   Retrieves the value for the specified key.
	///	@param     Key  The key for which to return the corresponding value.
	///	@return    The value withat the specified key.
	CPValue ValueForKey(const CPString& Key) const;
	CPValue ValueForKey(const CPString& Key, CPError& Error) const;

	///	@brief     Gets the value for the specified key.
	///	@details   Retrieves the value for the specified key. A null object is returned if the key does not exist. (IsNull() == true)
	///	@param     Key  The key for which to return the corresponding value.
	///	@return    The value withat the specified key.
	CPValue ValueForExistingKey(const CPString& Key) const;
	CPValue ValueForExistingKey(const CPString& Key, CPError& Error) const;

	///	@brief     Gets the enumerator for the keys of the dictionary.
	///	@details   Retrieves an instance of the enumerator which can be used in a loop to get the keys in the dictionary.
	///	@return    The enumerator representation for the current dictionary.
	CPEnumerator ObjectEnumerator(void) const;

	///	@brief     Gets the enumerator for the entries of the dictionary.
	///	@details   Retrieves an instance of the enumerator which can be used in a loop to get the objects in the dictionary.
	///	@return    The enumerator representation for the current dictionary.
	CPEnumerator KeyEnumerator(void) const;

	///	@brief     Gives an array that represents all keys that match the object.
	///	@details   The returns an array of keys that had an associated object that matched the passed object.
	///	@param     Object  The object in which to search for matching keys.
	///	@return    An array object that consists of keys as strings.
	CPArray AllKeysForObject(const CPObject& Object) const;

	///	@brief     Write the dictionary directly to a file.
	///	@details   This will write the current dictionary out to the file based on the filepath.
	///	@param     FilePath  The path and filename of the file to write to.
	///	@return    A boolean representing a successful write (true) to the file or not (false).
	bool WriteToFile(const CPString& FilePath) const;

	///	@brief     Write the dictionary to a file atomically.
	///	@details   This will write the current dictionary out to a temporary file and when complete, rename it to the filepath specified.
	///	@param     FilePath  The path and filename of the file to write to.
	///	@return    A boolean representing a successful write (true) to the file or not (false).
	bool WriteToFileAtomically(const CPString& FilePath) const;

#if __OBJC__
	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the specified dictionary.
	///	@param     Dictionary  the dictionary to copy.
	CPDictionary(NSDictionary* Dictionary);

	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the specified dictionary.
	///	@param     Dictionary  the dictionary to copy.
	CPDictionary(NSMutableDictionary* Dictionary);
	
	///	@brief     Copy Operator.
	///	@details   Copies the entries from the given dictionary.
	///	@param	   Dictionary  the dictionary to copy.
	const CPDictionary& operator=(NSDictionary* Dictionary);
	
	///	@brief     Copy Operator.
	///	@details   Copies the entries from the given dictionary.
	///	@param	   Dictionary  the dictionary to copy.
	const CPDictionary& operator=(NSMutableDictionary* Dictionary);
	
	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as a NSDictionary*.
	operator NSDictionary*(void) const;
#endif

#if __COREFOUNDATION__
	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the specified dictionary.
	///	@param     Dictionary  the dictionary to copy.
	CPDictionary(CFDictionaryRef Dictionary);
	
	///	@brief     Copy Operator.
	///	@details   Copies the entries from the given dictionary.
	///	@param	   Dictionary  the dictionary to copy.
	const CPDictionary& operator=(CFDictionaryRef Dictionary);
	
	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as a CFDictionaryRef*.
	operator CFDictionaryRef(void) const;
	
	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as a CFPropertyListRef*.
	operator CFPropertyListRef(void) const;
#endif

	friend class CPObject;

protected:
	CPId CPIdForKeyIfPresent(const CPString& Key) const;
	CPDictionary(const CPId& OtherObject, CPClassType ClassType);
	CPDictionary(const CPObject& OtherObject, CPClassType ClassType);

	void AssignUnderlyingDictionary(const CPId& Dictionary);
	void AssignUnderlyingDictionaryAndTakeOwnership(const CPId& Dictionary);
};

#include "CrossPlatform/CPData.h"
#include "CrossPlatform/CPBool.h"
#include "CrossPlatform/CPString.h"
#include "CrossPlatform/CPArray.h"
#include "CrossPlatform/CPEnumerator.h"
#include "CrossPlatform/CPDate.h"
#include "CrossPlatform/CPNumber.h"
#include "CrossPlatform/CPValue.h"
#include "CrossPlatform/CPDictionary.h"
