////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPEnumerator.h
//    Cross-platform enumerator class.
//
//  Copyright © 2009-2012 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  02/10/12  K.L.S.  Changes for CrossPlatform API changes. Reworked header includes.
//  03/30/09  K.L.S.  Created file.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/ExternalAPIMacros.h"
#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/CPObject.h"

class CPArray;

class CP_EXTERNAL_API CPEnumerator : public CPObject
{
public:
	///	@brief     A class type that uniquely identifies this class.
	///	@return    Returns a class type that uniquely identifies this class.
	static CPClassType ClassType(void);

	/// @brief     Default Constructor.
	/// @details   Creates an empty enumerator object.
	CPEnumerator(void);
	
	/// @brief     Creates an enumerator object containing the given enumerator object.
	/// @details   This object contains the object that was passed into the constructor.
	///	@param     OtherObject  the enumerator object to copy.
	CPEnumerator(const CPEnumerator& OtherObject);
	
	///	@brief     Destructor.
	///	@details   Removes any references it has to shared resources.
	~CPEnumerator(void);
	
	///	@brief     Copy Operator.
	///	@details   Copies the given enumerator.
	///	@param	   OtherObject  the enumerator to copy.
	const CPEnumerator& operator=(const CPEnumerator& OtherObject);
	
	///	@brief     Retrieve all objects.
	///	@details   Retrieves all objects from the enumerator.
	///	@return	   An array of all objects of the enumerator.
	CPArray AllObjects(void) const;

	///	@brief     Retrieve next object.
	///	@details   Retrieves the next object from the enumerator.
	///	@return	   An object from the enumerator.
	CPObject NextObject(void);
	
	///	@brief     Retrieve next string.
	///	@details   Retrieves the next string object from the enumerator.
	///	@return	   A string object from the enumerator.
	CPString NextStringObject(void);

	/// @brief     Checks is the enumerator is empty.
	///	@details   Returns whether or not elements are in the enumerator object.
	///	@return    Returns true if enumerator has no elements contained, otherwise false.
	virtual bool IsEmpty(void) const;
	
	/// @brief     Empties the enumerator.
	///	@details   Empties all the elements in the enumerator object.
	virtual void Empty(void);

#ifdef __OBJC__
	// Define methods that will be needed only when bridging to Cocoa

	/// @brief     Copy Constructor.
	/// @details   This object will contain the specified enumerator.
	///	@param     Enumerator  the enumerator to copy.
	CPEnumerator(NSEnumerator* Enumerator);
	
	///	@brief     Copy Operator.
	///	@details   Copies the given enumerator.
	///	@param	   Enumerator  the enumerator to copy.
	const CPEnumerator& operator=(NSEnumerator* Enumerator);

	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as an NSEnumerator*.
	operator NSEnumerator*() const;
#else
	/// @brief     Add array to Enumerator.
	/// @details   Add the given array to the enumerator object.
	///	@param     Values  the array to add.
	explicit CPEnumerator(const CPArray& Values);

private:
	CPUInteger m_Pos;
#endif
};

#include "CrossPlatform/CPArray.h"
