////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPError.h
//    Cross-platform error reporting class.
//
//  Copyright © 2009-2012 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  02/10/12  K.L.S.  Changes for CrossPlatform API changes. Reworked header includes.
//  03/04/09  K.L.S.  Created file.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/CPObject.h"
#include "CrossPlatform/CPInteger.h"
#include "CrossPlatform/CPCrossPlatformErrors.h"
#include "CrossPlatform/CPExceptionName.h"

class CPString;
class CPDictionary;
class CPArray;
class CPException;

///
/// @brief   Cross platform error class.
/// @details An CPError object encapsulates richer and more extensible error information than is possible using only an error code or error
///          string. The core attributes of an CPError object are an error domain (represented by a string), a domain-specific error code 
///          and a user info dictionary containing application specific information.
///
///          CPError allows you to attach an arbitrary user info dictionary to an error object, and provides the means to return a 
///          human-readable description for the error.
///
///          In general, a method should signal an error condition by—for example—returning NO or nil rather than by the simple presence of 
///          an error object. The method can then optionally return an CPError object by reference, in order to further describe the error.
///
/// Notes:   To output debug information of the CPError, use the Description() method instead of using the "%@" on the underlying error 
///          object since this will cause an error.
///
/// For more information:
///http://developer.apple.com/library/mac/#documentation/Cocoa/Reference/Foundation/Classes/NSError_Class
///http://developer.apple.com/library/mac/#documentation/CoreFoundation/Reference/CFErrorRef
///
class CP_EXTERNAL_API CPError : public CPObject
{
public:
	static CPString ErrorDomain(void);

	///	@brief    Creates an dictionary object with the specified info.
	///
	/// @details  Creates and initializes an dictionary object with a given error description, recovery description and recovery options array.
	///           This is a helper routine to create a user info dictionary for putting into a CPError.
	///
	///	@param    ErrorDescription  The error description to put into the dictionary with LocalizedDescriptionKey.
	///	@param    RecoverySuggestion  The recovery suggestion to put into the dictionary with LocalizedRecoverySuggestionErrorKey.
	///	@param    RecoveryOptions  The recovery options array to put into the dictionary with LocalizedRecoveryOptionsErrorKey.
	///	@param    FailureReason  A string containing the localized explanation of the reason for the error.
	///	@param    RecoverySuggestion  A string containing the localized recovery suggestion for the error.
	///	@param    AdditionalInfo  Dictionary that can be use to add any additional info you want to attach to the error.
	///
	/// @return   An dictionary with the specified information added.
	///
	static CPDictionary DictionaryWithErrorDescriptionFailureReasonRecoverySuggestionRecoveryOptionsAndAdditionalInfo(const CPString& ErrorDescription,
		const CPString& FailureReason, const CPString& RecoverySuggestion, const CPArray& RecoveryOptions, const CPDictionary& AdditionalInfo);
	static CPDictionary DictionaryWithErrorDescriptionFailureReasonRecoverySuggestionAndRecoveryOptions(const CPString& ErrorDescription,
		const CPString& FailureReason, const CPString& RecoverySuggestion, const CPArray& RecoveryOptions);
	static CPDictionary DictionaryWithErrorDescriptionFailureReasonRecoverySuggestionAndAdditionalInfo(const CPString& ErrorDescription,
		const CPString& FailureReason, const CPString& RecoverySuggestion, const CPDictionary& AdditionalInfo);
	static CPDictionary DictionaryWithErrorDescriptionFailureReasonAndRecoverySuggestion(const CPString& ErrorDescription, const CPString& FailureReason,
																						 const CPString& RecoverySuggestion);
	static CPDictionary DictionaryWithErrorDescriptionAndFailureReason(const CPString& ErrorDescription, const CPString& FailureReason);
	static CPDictionary DictionaryWithErrorDescription(const CPString& ErrorDescription);

	///	@brief    Creates an error object with the specified info.
	/// @details  Creates and initializes an error object with a given code, domain and user information dictionary. A code of zero will result
	///           in an empty error object and the other supplied information will not be retained. An error is unique by error code within a
	///           domain.
	///	@param    Domain  The error domain which identifies to grouping for the error.
	///	@param    Code  The error code. An error code of zero will result in an empty error object.
	///	@param    UserInfo  The user info contains other information about the error, such as a readable description of the error.
	/// @return   An error object with the code, domain and user info.
	static CPError ErrorWithDomainCodeAndUserInfo(const CPString& Domain, CPInteger Code, const CPDictionary& UserInfo); // Base

	///	@brief    Creates an error object with the specified exception.
	/// @details  Creates and initializes an error object with a given exception. This method is used to make to make the behavior on both
	///           platforms the same by converting some exceptions to errors so dealing with problems will be platform independent. This method
	///           may return an empty error which in most cases would be an indication to re-throw the exception.
	///	@param    Exception  The exception to attempt to make into an error.
	/// @return   An error object based on the exception.
	static CPError ErrorWithException(const CPException& Exception);

	///	@brief     A cla3ss type that uniquely identifies this class.
	///	@return    Returns a class type that uniquely identifies this class.
	static CPClassType ClassType(void);

	/// @brief    Returns the underlying shared empty object.
	/// @details  Static method that returns the underlying object that is shared between all empty objects of this class type.
	/// @returns  the underlying shared empty object.
	static const CPId& SharedEmptyObject(void);

	/// @brief  The corresponding value is a localized string representation of the error that, if present, will be returned by localizedDescription.
	static CPString LocalizedDescriptionKey(void);

	/// @brief  The corresponding value is a localized string representation containing the reason for the failure that, if present, will be
	///         returned by LocalizedFailureReason. This string provides a more detailed explanation of the error than the description.
	static CPString LocalizedFailureReasonKey(void);

	/// @brief  The corresponding value is a string containing the localized recovery suggestion for the error.
	static CPString LocalizedRecoverySuggestionKey(void);

	/// @brief  The corresponding value is an array containing the localized titles of buttons appropriate for displaying in an alert panel.
	///         The first string is the title of the right-most and default button, the second the one to the left, and so on. The recovery
	///         options should be appropriate for the recovery suggestion returned by LocalizedRecoverySuggestion.
	static CPString LocalizedRecoveryOptionsKey(void);
	
	///	@brief  The corresponding value is an error that was encountered in an underlying implementation and caused the error that the
	///         receiver represents to occur.
	static CPString UnderlyingErrorKey(void);
	
	///	@brief     creates an error object.
	/// @details   Creates and initializes a error object for a given domain and code with a given error description.
	/// @return    An error object for domain with the specified error code and the error description.
	static CPError ErrorWithCodeDescriptionAndDomain(CPInteger Code, const CPString& ErrorDescription, const CPString& Domain);

	/// @brief     Default Constructor.
	/// @details   Creates an empty data object.
	CPError(void);

	/// @brief     Copy Constructor.
	/// @details   This error object will contain the object of the specified error.
	///	@param     OtherObject  the error to copy.	
	CPError(const CPError& OtherObject);
	
	///	@brief     Destructor.
	///	@details   Deallocates the object and removes any references it has to shared resources.
	~CPError(void);

	///	@brief    initializes an error object with the specified info.
	///
	/// @details  Initializes an error object with a given code, domain and user information dictionary. A code of zero will result
	///           in an empty error object and the other supplied information will not be retained. An error is unique by error code within a
	///           domain.
	///
	///	@param    ErrorDomain  The error domain which identifies to grouping for the error.
	///	@param    ErrorCode  The error code. An error code of zero will result in an empty error object.
	///	@param    UserInfoIn  The user info contains other information about the error, such as a readable description of the error.
	///
	void InitWithDomainCodeAndUserInfo(const CPString& ErrorDomain, CPInteger ErrorCode, const CPDictionary& UserInfoIn);

	///	@brief    initializes an error object with the specified info.
	///
	/// @details  Initializes an error object with a given code, domain and user information dictionary. A code of zero will result
	///           in an empty error object and the other supplied information will not be retained. An error is unique by error code within a
	///           domain.
	///
	///	@param    ErrorDomain  The error domain which identifies to grouping for the error.
	///	@param    ErrorCode  The error code. An error code of zero will result in an empty error object.
	///	@param    ErrorDescription  A readable description of the error.
	///
	void InitWithDomainCodeAndDescription(const CPString& ErrorDomain, CPInteger ErrorCode, const CPString& ErrorDescription);

	///	@brief    initializes an error object with the specified info.
	///
	/// @details  Initializes an error object with a given code, domain and user information dictionary. A code of zero will result
	///           in an empty error object and the other supplied information will not be retained. An error is unique by error code within a
	///           domain.
	///
	///	@param    ErrorDomain  The error domain which identifies to grouping for the error.
	///	@param    ErrorCode  The error code. An error code of zero will result in an empty error object.
	///	@param    ErrorDescription  A readable description of the error.
	///	@param    FailureReason  A description of the reason for the failure.
	///
	void InitWithDomainCodeDescriptionAndFailureReason(const CPString& ErrorDomain, CPInteger ErrorCode, const CPString& ErrorDescription,
		const CPString& FailureReason);

	/// @brief    Checks to see if there is an error.
	///
	/// @details  Checks to see if there is an error. There is an error if the code is not zero.
	///
	/// @return   true if there is an error, otherwise false.
	///
	bool IsError(void) const;

	/// @brief    Checks to see if the error matches the code and domain specified.
	/// @details  Checks to see if the error matches the code and domain specified. This it the best way to test for error equality as the
	///           same error code can be used in differnt domains.
	/// @return   true if the error code and domain match, otherwise set to false.
	bool IsErrorWithCodeAndDomain(int CodeIn, const CPString& DomainIn) const;
	
	///	@brief     Copy Operator.
	/// @details   Assigns an error object to another error object.
	///	@param     OtherObject  the error to copy.	
	const CPError& operator=(const CPError& OtherObject);
	
	/// @brief     Returns error code.
	/// @details   Returns error code. Note that the error code is specific to a domain.
	/// @return    the code of the error.
	CPInteger Code(void) const;
	
	/// @brief     Returns the error domain.
	/// @return    the domain of the error.
	CPString Domain(void) const;
	
	/// @brief    Returns the user info dictionary.
	/// @return   the user info dictionary.
	CPDictionary UserInfo(void) const;


	/// @brief     Returns a localized description of the error.
	/// @details   The main description of the error. It often includes the failure reason. If no error description is present in the user
	///            info dictionary, CPError either constructs one from the error domain and code or attempts to fetch a suitable string from
	///            a function or method.
	/// @return    A string containing the localized description of the error.
	CPString LocalizedDescription(void) const;

	/// @brief     A brief sentence that explains the reason why the error occurred.
	/// @details   A brief sentence that explains the reason why the error occurred. It is typically part of the error description.
	///            The failure reason is for clients that only want to display the reason for the failure.
	/// @return    A string containing the localized reason why the error occurred.
	CPString LocalizedFailureReason(void) const;

	/// @brief     Gets the localized suggested recovery for the error.
	/// @details   Returns a string containing the localized recovery suggestion for the error.
	/// @return    Returns the object in the user info dictionary for the key LocalizedRecoverySuggestionErrorKey. If the user info
	///            dictionary doesn’t contain a value for LocalizedRecoverySuggestionErrorKey, nil is returned.
	CPString LocalizedRecoverySuggestion(void) const;

	/// @brief     Gets an array of localized button titles.
	/// @details   Returns an array containing the localized titles of buttons appropriate for displaying in an alert panel.
	/// @return    Returns the object in the user info dictionary for the key LocalizedRecoveryOptionsErrorKey. If the user info dictionary
	///            doesn’t contain a value for LocalizedRecoveryOptionsErrorKey, an empty array is returned.
	CPArray LocalizedRecoveryOptions(void) const;

	///	@brief     Gets the object at the specified key.
	///	@details   This retrieves the object for the specified key in the user info dictionary inside the error object.
	///	@param     Key  A string which is the key used for searching in the user info dictionary.
	///	@return    The object located at the specified index.
	CPObject ObjectForKey(const CPString& Key) const;
	
	///	@brief     Adds/replaces the key-object pair in the existing user info dictionary.
	///	@details   Adds or replaces the object in the existing user info dictionary associated with the passed key.
	///	@param     Object  The object to add.	
	///	@param     Key  The string used to add the passed object based parameter.
	void SetObjectForKey(const CPObject& Object, const CPString& Key);
	
	/// @brief    A convienient way to retrieve the error code, domain, & user info dictionary (as a string) for the error.
	///	@details  To output debug information of the CPError, use the Ref() of this method instead of using using Ref() itself
	///           since this will cause an error.
	/// @return   A description of the error.
	virtual CPString Description(void) const;

#ifdef __OBJC__
	// Define methods that will be needed only when bridging to Cocoa
	
	/// @brief     Copy Constructor.
	/// @details   This error object will contain the object of the specified error.
	///	@param     Error  the error to copy.	
	CPError(NSError* Error);
	
	///	@brief     Copy Operator.
	/// @details   Assigns a error object to another error object.
	///	@param     Error  An error object to copy from.	
	const CPError& operator=(NSError* Error);
	
	///	@brief     Conversion Operator.
	///	@details   Returns the underlying error object as a NSError*.
	operator NSError*() const;	

#endif
#if __COREFOUNDATION__
	// Define methods that will be needed only when bridging to CoreFoundation
	
	/// @brief     Copy Constructor.
	/// @details   This error object will contain the object of the specified error.
	///	@param     Value  the error to copy.	
	CPError(CFErrorRef Value);
	
	///	@brief     Copy Operator.
	/// @details   Assigns a error object to another error object.
	///	@param     Value  An error object to copy from.	
	const CPError& operator=(CFErrorRef Value);
	
	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as a CFErrorRef*.
	operator CFErrorRef(void) const;
#endif
};

#define FileNameAsCPErrorDomain() FilePathToDomain(__FILE__)

CPString CP_EXTERNAL_API FilePathToDomain(const CPString& FilePath);
CPError CP_EXTERNAL_API MakeCPError(CPInteger Code, const CPString& ErrorDescription, const CPString& RecoveryDescription,
								 const CPArray& RecoveryOptions, const CPString& Domain);
CPError CP_EXTERNAL_API MakeCPError(CPInteger Code, const CPString& ErrorDescription, const CPString& RecoveryDescription,
								 const CPDictionary& AdditionalInfo, const CPString& Domain);
CPError CP_EXTERNAL_API MakeCPError(CPInteger Code, const CPString& ErrorDescription, const CPString& RecoveryDescription,
								 const CPString& Domain);

CPError CP_EXTERNAL_API MakeCPError(CPInteger Code, const CPString& Domain, const CPDictionary& UserInfo); // Base
