////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPFileHandle.h
//    CPFileHandle objects provide an object-oriented wrapper for accessing open files or communications channels.
//
//  Copyright © 2011-2013 Acclivity Software, all rights reserved.
//
//  Change History (most recent first):
//
//  03/15/13  G.R.W.  Setup exceptions for Windows into some functions of CPFileHandle and added try-catch in calling code.
//	03/23/11  K.L.S.  Created File.
//
//  Notes:
//    The deallocation of an CFFileHandle object closes the represented file or channel unless the CFFileHandle object CloseOnDealloc
//    property was set to false.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/CPObject.h"
#include "CrossPlatform/CPString.h"
#include "CrossPlatform/CPData.h"
#include "CrossPlatform/CPULongLong.h"

/// @brief    The CPFileHandle class is an object-oriented wrapper for a file descriptor. You use file handle objects to access data 
///           associated with files, sockets, pipes, and devices. For files, you can read, write, and seek within the file. For sockets, 
///           pipes, and devices, you can use a file handle object to monitor the device and process data asynchronously.
///
///           Most creation methods for CPFileHandle cause the file handle object to take ownership of the associated file descriptor. This 
///           means that the file handle object both creates the file descriptor and is responsible for closing it later, usually when the 
///           file handle object itself is deallocated.
///
/// For more information:
/// http://developer.apple.com/library/mac/#documentation/Cocoa/Reference/Foundation/Classes/NSFileHandle_Class
///
class CP_EXTERNAL_API CPFileHandle : public CPObject
{
public:
	enum CPFileShareOptions
	{
		CPFileShareWinOSDenyAllAccess = (int) 0x00010,
		CPFileShareWinOSDenyWriteAccess = (int) 0x00020,
		CPFileShareWinOSDenyReadAccess = (int) 0x00030,
		CPFileShareAllowAllAccess = (int) 0x00040
	};

	///	@brief     A class type that uniquely identifies this class.
	///	@return    Returns a class type that uniquely identifies this class.
	static CPClassType ClassType(void);

	/// @brief     Returns a file handle initialized for reading the file, device, or named socket at the given path.
	/// @details   Returns a file handle initialized for reading the file, device, or named socket at the given path.	The file pointer is 
	///            set to the beginning of the file.  You cannot write data to the returned file handle object. Use the ReadDataToEndOfFile
	///            or ReadDataOfLength methods to read data from it.
	///            When using this method to create a file handle object, the file handle owns its associated file descriptor and is
	///            responsible for closing it.
	///	@param	   Path  A string of the path to the file, device, or named socket to access.  
	/// @return    Returns the initialized file handle object.
	static CPFileHandle FileHandleForReadingAtPath(const CPString& Path);

	/// @brief     Returns a file handle initialized for reading the file, device, or named socket at the given path.
	/// @details   Returns a file handle initialized for reading the file, device, or named socket at the given path.	The file pointer is 
	///            set to the beginning of the file.  You cannot write data to the returned file handle object. Use the ReadDataToEndOfFile
	///            or ReadDataOfLength methods to read data from it.
	///            When using this method to create a file handle object, the file handle owns its associated file descriptor and is
	///            responsible for closing it.
	///	@param	   Path  A string of the path to the file, device, or named socket to access.  
	///	@param	   ShareOptions  Flag to specify sharing options for access to the file. Flags currently only affect Windows behavior.  
	/// @return    Returns the initialized file handle object.
	static CPFileHandle FileHandleForReadingAtPathWithShareAccess(const CPString& Path, CPFileShareOptions ShareOptions);

	/// @brief     Returns a file handle initialized for writing to the file, device, or named socket at the given path.
	/// @details   Returns a file handle initialized for writing to the file, device, or named socket at the given path.  The file pointer
	///            is set to the beginning of the file. You cannot read data from the returned file handle object. Use the WriteData method 
	///            to write data to the file handle.
	///            When using this method to create a file handle object, the file handle owns its associated file descriptor and is
	///            responsible for closing it.
	///	@param	   Path  A string of the path to the file, device, or named socket to access. 
	/// @return    The initialized file handle object.
	static CPFileHandle FileHandleForWritingAtPath(const CPString& Path);

	/// @brief     Returns a file handle initialized for writing to the file, device, or named socket at the given path.
	/// @details   Returns a file handle initialized for writing to the file, device, or named socket at the given path.  The file pointer
	///            is set to the beginning of the file. You cannot read data from the returned file handle object. Use the WriteData method 
	///            to write data to the file handle.
	///            When using this method to create a file handle object, the file handle owns its associated file descriptor and is
	///            responsible for closing it.
	///	@param	   Path  A string of the path to the file, device, or named socket to access. 
	///	@param	   ShareOptions  Flag to specify sharing options for access to the file. Flags currently only affect Windows behavior.  
	/// @return    The initialized file handle object.
	static CPFileHandle FileHandleForWritingAtPathWithShareAccess(const CPString& Path, CPFileShareOptions ShareOptions);
	
	/// @brief     Returns a file handle initialized for reading and writing to the file, device, or named socket at the given path.
	/// @details   Returns a file handle initialized for reading and writing to the file, device, or named socket at the given path. The file
	///            pointer is set to the beginning of the file. Both read and write methods can be used on this object. When using this method
	///            to create a file handle object, the file handle owns its associated file descriptor and is responsible for closing it.
	///	@param	   Path  A string of the path to the file, device, or named socket to access. 
	/// @return    The initialized file handle object.
	static CPFileHandle FileHandleForUpdatingAtPath(const CPString& Path);
	
	/// @brief     Returns a file handle initialized for reading and writing to the file, device, or named socket at the given path.
	/// @details   Returns a file handle initialized for reading and writing to the file, device, or named socket at the given path. The file
	///            pointer is set to the beginning of the file. Both read and write methods can be used on this object. When using this method
	///            to create a file handle object, the file handle owns its associated file descriptor and is responsible for closing it.
	///	@param	   Path  A string of the path to the file, device, or named socket to access. 
	///	@param	   ShareOptions  Flag to specify sharing options for access to the file. Flags currently only affect Windows behavior.  
	/// @return    The initialized file handle object.
	static CPFileHandle FileHandleForUpdatingAtPathWithShareAccess(const CPString& Path, CPFileShareOptions ShareOptions);
	
	/// @brief     Default Constructor.
	/// @details   Creates an empty file handle.
	CPFileHandle(void);
	
	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the given file handle.
	///	@param     Object  the file handle to copy.
	CPFileHandle(const CPFileHandle& Object);
	
	///	@brief     Destructor.
	///	@details   Removes any references it has to shared resources.
	~CPFileHandle(void);

	
	///	@brief     Copy Operator.
	///	@details   Copies the given file handle.
	///	@param	   Object  the file handle to copy.
	const CPFileHandle& operator=(const CPFileHandle& Object);
	
	/// @brief     Sets a file handle initialized for reading the file, device, or named socket at the given path.
	/// @details   Sets a file handle initialized for reading the file, device, or named socket at the given path.	The file pointer is 
	///            set to the beginning of the file.  You cannot write data to the returned file handle object. Use the ReadDataToEndOfFile
	///            or ReadDataOfLength methods to read data from it.
	///            When using this method to create a file handle object, the file handle owns its associated file descriptor and is
	///            responsible for closing it.
	///	@param	   Path  A string of the path to the file, device, or named socket to access.  
	void InitForReadingAtPath(const CPString& Path);

	/// @brief     Sets a file handle initialized for reading the file, device, or named socket at the given path.
	/// @details   Sets a file handle initialized for reading the file, device, or named socket at the given path.	The file pointer is 
	///            set to the beginning of the file.  You cannot write data to the returned file handle object. Use the ReadDataToEndOfFile
	///            or ReadDataOfLength methods to read data from it.
	///            When using this method to create a file handle object, the file handle owns its associated file descriptor and is
	///            responsible for closing it.
	///	@param	   Path  A string of the path to the file, device, or named socket to access.  
	///	@param	   ShareOptions  Flag to specify sharing options for access to the file. Flags currently only affect Windows behavior.  
	void InitForReadingAtPathWithShareAccess(const CPString& Path, CPFileShareOptions ShareOptions);
	
	/// @brief     Sets a file handle initialized for writing to the file, device, or named socket at the given path.
	/// @details   Sets a file handle initialized for writing to the file, device, or named socket at the given path.  The file pointer
	///            is set to the beginning of the file. You cannot read data from the returned file handle object. Use the WriteData method 
	///            to write data to the file handle.
	///            When using this method to create a file handle object, the file handle owns its associated file descriptor and is
	///            responsible for closing it.
	///	@param	   Path  A string of the path to the file, device, or named socket to access. 
	void InitForWritingAtPath(const CPString& Path);
	
	/// @brief     Sets a file handle initialized for writing to the file, device, or named socket at the given path.
	/// @details   Sets a file handle initialized for writing to the file, device, or named socket at the given path.  The file pointer
	///            is set to the beginning of the file. You cannot read data from the returned file handle object. Use the WriteData method 
	///            to write data to the file handle.
	///            When using this method to create a file handle object, the file handle owns its associated file descriptor and is
	///            responsible for closing it.
	///	@param	   Path  A string of the path to the file, device, or named socket to access. 
	///	@param	   ShareOptions  Flag to specify sharing options for access to the file. Flags currently only affect Windows behavior.  
	void InitForWritingAtPathWithShareAccess(const CPString& Path, CPFileShareOptions ShareOptions);
	
	/// @brief     Sets a file handle initialized for reading and writing to the file, device, or named socket at the given path.
	/// @details   Sets a file handle initialized for reading and writing to the file, device, or named socket at the given path. The file
	///            pointer is set to the beginning of the file. Both read and write methods can be used on this object. When using this method
	///            to create a file handle object, the file handle owns its associated file descriptor and is responsible for closing it.
	///	@param	   Path  A string of the path to the file, device, or named socket to access. 
	void InitForUpdatingAtPath(const CPString& Path);

	/// @brief     Sets a file handle initialized for reading and writing to the file, device, or named socket at the given path.
	/// @details   Sets a file handle initialized for reading and writing to the file, device, or named socket at the given path. The file
	///            pointer is set to the beginning of the file. Both read and write methods can be used on this object. When using this method
	///            to create a file handle object, the file handle owns its associated file descriptor and is responsible for closing it.
	///	@param	   Path  A string of the path to the file, device, or named socket to access. 
	///	@param	   ShareOptions  Flag to specify sharing options for access to the file. Flags currently only affect Windows behavior.  
	void InitForUpdatingAtPathWithShareAccess(const CPString& Path, CPFileShareOptions ShareOptions);

	/// @brief     Returns the data currently available in the receiver.
	/// @details   Returns the data currently available in the receiver.  If the receiver is a file, this method returns the data obtained 
	///            by reading the file from the current file pointer to the end of the file.  If the receiver is a communications channel, 
	///            this method reads up to a buffer of data and returns it; if no data is available, the method blocks.  Returns an empty 
	///            data object if the end of file is reached. This method raises CPFileHandleOperationException if attempts to determine the
	///            file-handle type fail or if attempts to read from the file or channel fail.
	/// @return    The data currently available through the receiver, up to the the maximum size that can be represented by a CPData object.
	/// @throws    CPException
	CPData AvailableData(void);
	
	/// @brief     Synchronously reads data up to the specified number of bytes.
	/// @details   Synchronously reads data up to the specified number of bytes.  If the receiver is a file, this method returns the data
	///            obtained by reading length bytes starting at the current file pointer. If length bytes are not available, this method 
	///            returns the data from the current file pointer to the end of the file. If the receiver is a communications channel, the 
	///            method reads up to length bytes from the channel. Returns an empty CPData object if the file is positioned at the end of
	///            the file or if an end-of-file indicator is returned on a communications channel. This method raises 
	///            CPFileHandleOperationException if attempts to determine the file-handle type fail or if attempts to read from the file or
	///            channel fail.
	///	@param	   Length  The number of bytes to read from the receiver.
	/// @return    The data available through the receiver up to a maximum of length bytes, or the maximum size that can be represented by a
	///            CPData object, whichever is the smaller.
	/// @throws    CPException
	CPData ReadDataOfLength(CPUInteger Length);
	
	/// @brief     Synchronously reads the available data up to the end of file or maximum number of bytes.
	/// @details   Synchronously reads the available data up to the end of file or maximum number of bytes.  This method invokes the
	///            ReadDataOfLength method as part of its implementation.
	/// @return    The data available through the receiver up to maximum size that can be represented by a CPData object or, if a 
	///            communications channel, until an end-of-file indicator is returned.
	/// @throws    CPException
	CPData ReadDataToEndOfFile(void);

	/// @brief     Synchronously writes the specified data to the receiver.
	/// @details   Synchronously writes the specified data to the receiver.  If the receiver is a file, writing takes place at the file 
	///            pointer’s current position. After it writes the data, the method advances the file pointer by the number of bytes 
	///            written. This method raises an exception if the file descriptor is closed or is not valid, if the receiver represents an 
	///            unconnected pipe or socket endpoint, if no free space is left on the file system, or if any other writing error occurs.
	///	@param	   Data  The data to be written.
	/// @throws    CPException
	void WriteData(const CPData& Data);
	
	/// @brief     Returns the position of the file pointer within the file represented by the receiver.
	/// @details   Returns the position of the file pointer within the file represented by the receiver.  Raises an exception if the message
	///            is sent to a file handle representing a pipe or socket or if the file descriptor is closed.
	/// @return    The position of the file pointer within the file represented by the receiver.
	CPULongLong OffsetInFile(void) const;
	
	/// @brief     Puts the file pointer at the end of the file referenced by the receiver and returns the new file offset.
	/// @details   Puts the file pointer at the end of the file referenced by the receiver and returns the new file offset.  Raises an 
	///            exception if the message is sent to a CPFileHandle object representing a pipe or socket or if the file descriptor is 
	///            closed.	
	/// @return    The file offset with the file pointer at the end of the file. This is therefore equal to the size of the file.
	/// @throws    CPException
	CPULongLong SeekToEndOfFile(void);

	/// @brief     Moves the file pointer to the specified offset within the file represented by the receiver.
	/// @details   Moves the file pointer to the specified offset within the file represented by the receiver.  Raises an exception if the
	///            message is sent to a CPFileHandle object representing a pipe or socket, if the file descriptor is closed, or if any other
	///            error occurs in seeking.
	///	@param	   Offset  The offset to seek to.
	/// @throws    CPException
	void SeekToFileOffset(CPULongLong Offset);

	/// @brief     Truncates or extends the file represented by the receiver to a specified offset within the file and puts the file pointer
	///            at that position.
	/// @details   Truncates or extends the file represented by the receiver to a specified offset within the file and puts the file pointer
	///            at that position.  If the file is extended (if offset is beyond the current end of file), the added characters are null 
	///            bytes.
	///	@param	   Offset  The offset within the file that will mark the new end of the file.
	/// @throws    CPException
	void TruncateFileAtOffset(CPULongLong Offset);
	
	/// @brief     Causes all in-memory data and attributes of the file represented by the receiver to be written to permanent storage.
	/// @details   Causes all in-memory data and attributes of the file represented by the receiver to be written to permanent storage.  
	///            This method should be invoked by programs that require the file to always be in a known state.  An invocation of this 
	///            method does not return until memory is flushed.
	void SynchronizeFile(void);
	
	/// @brief     Closes the file for reading, writing, & updating.
	/// @details   Disallows further access to the represented file or communications channel and signals end of file on communications
	///            channels that permit writing.
	void CloseFile(void);

	/// @brief     Returns status of the current file being open.
	///	@return	   Returns true if the current file is open, otherwise returns false.
	bool IsOpen(void) const;

#ifdef __OBJC__
	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the specified file handle.
	///	@param     Object  the file handle to copy.
	CPFileHandle(NSFileHandle* Object);
	
	///	@brief     Copy Operator.
	///	@details   Copies the file handle.
	///	@param	   FileHandle  the file handle to copy.
	const CPFileHandle& operator=(NSFileHandle* FileHandle);
	
	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as a NSFileHandle*.
	operator NSFileHandle*(void) const;
#else
	
	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as a void*.
	void* FileHandle(void) const;
#endif

	virtual void Release(void);
	virtual void Nullify(void);
};
