////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPFileManager.h
//    Cross-platform file manager.
//
//  Copyright © 2009-2013 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  04/17/12  G.R.W.  Added the DirectoryContentsAtPath method.
//	01/03/10  K.L.S.  Added DisplayNameAtPath.
//  12/14/09  G.R.W.  Added the DirectoryContentsAtPath method.
//	03/11/09  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/CPULongLong.h"
#include "CrossPlatform/CPString.h"
#include "CrossPlatform/CPArray.h"
#include "CrossPlatform/CPDictionary.h"
#include "CrossPlatform/CPFileAttributes.h"

class CPError;

/// @brief   A collection of static file related utilities.
/// @details CPFileManager is a collection class that provides a number of static methods that the user can use to perform file related 
//			 operations.  
///
///			 These operations include a number of checks if: an item exists, a directory exists, the item is readable & writeable, if
///			 the item can be executed, & that the item can be deleted.
///
///			 This class contains operational methods for the following: copying & moving an item, creating & deleting an item, creating a 
///			 directory with specific attributes, & obtaining the items from a dictory listing (with or without specific attributes).
///
///          The CPFileManager class enables you to perform many generic file-system operations and insulates an application from the 
///          underlying file system. Most file operations can be performed using the shared file manager object. 
///
///          In Cocoa applications, a file manager object is usually your first interaction with the file system. You use this object to 
///          locate, create, copy, and move files and directories. You also use this object to get information about files and directories, 
///          such as its size, modification date, and BSD permissions. You can also use a file manager object to change the values of many 
///          file and directory attributes.
///
///          The CPFileManager class uses the CPString class as ways to specify the location of a file or directory. 
///
/// For more information:
/// https://developer.apple.com/library/mac/#documentation/Cocoa/Reference/Foundation/Classes/NSFileManager_Class
///
class CP_EXTERNAL_API CPFileManager
{
public:
	/// @brief Constant identifying predefined directory search locations.
	///
	/// @details The constants were given values to match the values that exist in MacOS Cocoa API.  These are used in combination with
	///          CPSearchPathDomain to get the paths. Not all combinations are valid and will represent no directory paths.
	///
	enum CPSearchPathDirectoryIdentifier
	{
		/// Supported applications (Applications)
		CPApplicationDirectory = 1,
		/// Various documentation, support, and configuration files, resources (Library)
		CPLibraryDirectory = 5,
		/// User home directories (Users)
		CPUserDirectory = CPLibraryDirectory+2,
		/// Documents (Documents)
		CPDocumentDirectory = CPUserDirectory+2,
		/// Location of user's desktop
		CPDesktopDirectory = 12,
		/// Location of discardable cache files (Library/Caches)
		CPCachesDirectory = 13,
		/// Location of application support files (Library/Application Support)
		CPApplicationSupportDirectory = 14,
		/// Location of the user's "Downloads" directory
		CPDownloadsDirectory = 15,
		/// Location of user's Movies directory (~/Movies)
		CPMoviesDirectory = 17,
		/// Location of user's Music directory (~/Music)
		CPMusicDirectory = 18,
		/// Location of user's Pictures directory (~/Pictures)
		CPPicturesDirectory = 19,
		/// Location of user's Public sharing directory (~/Public)
		CPSharedPublicDirectory = 21,
	};
	typedef CPUInteger CPSearchPathDirectory;
	
	/// @brief   Constant masks of domains for the search path directories.
	///
	/// @details Constants were given values to match the values that exist in MacOS Cocoa API.
	///
	enum CPSearchPathDomain
	{
		/// user's home directory --- place to install user's personal items (~)
		CPUserDomainMask = 1,
		/// local to the current machine --- place to install items available to everyone on this machine (/Library)
		CPLocalDomainMask = 2,      
	};
	typedef CPUInteger CPSearchPathDomainMask;

	/// @brief Constants of the possible file manager errors.
	///
	enum FileManagerErrorCode
	{
		/// Class implementation is incomplete; method has not been implemented.
		kUnimplementedErrorCode = 1,
		/// File or Directory already exist.
		kAlreadyExistsErrorCode,      
	};

	///	@brief   Return the error domain for this class.
	///
	///	@return  This returns the error domain for this class.
	///
	static CPString ErrorDomain(void);

	///	@brief     Return the key for the file size.
	///	@details   Returns the key for file size used in the item's attributes dictionary.
	///	@return    This returns the key used to determine a file's size.
	static CPString FileSizeKey(void);

	///	@brief     Checks if the given item exists.
	///	@details   This static method checks for the existance of the item whose name was passed into this method.  This could be a file 
	/// or directory.
	///	@param[in] Path  The file path of the the item to be checked.
	///	@return    true if the file/directory exists and false otherwise.
	static bool ItemExistsAtPath(const CPString& Path);

	///	@brief     Checks if the given item exists and if it is a directory.
	///	@details   This static method checks for the existance of the item whose name was passed into this method.  This could be a file 
	/// or directory.
	///	@param[in] Path  The file path of the the item to be checked.
	///	@param[out] IsDirectory  A bool flag set to true if the item passed was found and is a directory, otherwise false.
	///	@return    true if the file/directory exists and false otherwise.
	static bool ItemExistsAtPath(const CPString& Path, bool& IsDirectory);

	///	@brief     Checks if the given item is readable.
	///	@details   This static method checks for the existance of the item whose name was passed into this method.  It also checks to see if 
	/// the permissions on the file are set such that the caller can actually access this file for reading.
	///	@param[in] Path  The file path of the the item to be checked.
	///	@return    true if the file/directory is readable and false otherwise.
	static bool IsReadableItemAtPath(const CPString& Path);

	///	@brief     Checks if the given item is writeable.
	///	@details   This static method checks for the existance of the item whose name was passed into this method.  It also checks to see if 
	/// the permissions on the file are set such that the caller can actually access this file for writing.
	///	@param[in] Path  The file path of the the item to be checked.
	///	@return    true if the file/directory is writeable and false otherwise.
	static bool IsWritableItemAtPath(const CPString& Path);

	///	@brief     Checks if the given item is executable.
	///	@details   This static method checks for the existance of the item whose name was passed into this method.  It also checks to see if 
	/// the permissions on the file are set such that the caller can actually access this file for executable.
	///	@param[in] Path  The file path of the the item to be checked.
	///	@return    true if the file/directory is executable and false otherwise.
	static bool IsExecutableItemAtPath(const CPString& Path);

	///	@brief     Checks if the given item is able to be deleted.
	///	@details   This static method checks for the existance of the item whose name was passed into this method.  It also checks to see if
	/// the permissions on the file are set such that the caller can delete the file.
	///	@param[in] Path  The file path of the the item to be checked.
	///	@return    true if the file/directory is able to be deleted and false otherwise.
	static bool IsDeletableItemAtPath(const CPString& Path);

	///	@brief     Checks if the given item is a directory.
	///	@details   This static method checks for the existance of the passed item and if it was a directory.
	///	@param[in] Path  The file path of the the item to be checked and found as a directory.
	///	@return    true if the string passed was a directory and false otherwise.
	static bool IsDirectory(const CPString& Path);

	///	@brief     Copies a source file/dictory to a destination file/directory.
	///	@details   This static method will make a copy of the source file or directory to the destination file or directory.
	///	@param[in] SourcePath  The file path of the file that is being used to copy from.
	///	@param[in] DestinationPath  The file path you want the item copied to copy to.
	///	@return    true if the copy was successful and false otherwise.
	static bool CopyItemAtPathToPath(const CPString& SourcePath, const CPString& DestinationPath);

	///	@brief     Moves a source file/dictory to a destination file/directory.
	///	@details   This static method will move the source file or directory to the destination file or directory.
	///	@param[in] SourcePath  The file path of the file that is being used to move from.
	///	@param[in] DestinationPath  The file path of the file that is being used to move to.
	///	@return    true if the move was successful and false otherwise.
	static bool MoveItemAtPathToPath(const CPString& SourcePath, const CPString& DestinationPath);
	static bool MoveItemAtPathToPath(const CPString& SourcePath, const CPString& DestinationPath, CPError& Error);

	///	@brief     Deletes the file/directory.
	///	@details   This static method deleted the file or directory of the passed item.
	///	@param[in] Path  The file path of the file that is being deleted.
	///	@return    true if the string passed was deleted and false otherwise.
	static bool RemoveItemAtPath(const CPString& Path);

	///	@brief     Creates a directory at a path with no attributes.
	///	@details   This method creates a directory at the passed path.
	///	@param[in] Path  The path to where the directory should be created.
	///	@return    true if the directory was created or exists already and false otherwise.
	static bool CreateDirectoryAtPath(const CPString& Path);

	///	@brief      Creates a directory with given attributes at the specified path.
	///	@details    Creates a directory with given attributes at the specified path. If you specify an empty directory for the Attributes
	///             parameter, this method uses a default set of values for the owner, group, and permissions of any newly created directories
	///             in the path. Similarly, if you omit a specific attribute, the default value is used. The default values for newly created
	///             directories are as follows:
	///               Permissions are set according to the umask of the current process. For more information, see umask.
	///               The owner ID is set to the effective user ID of the process.
	///               The group ID is set to that of the parent directory.
	///	@param      DirectoryPath  A path string identifying the directory to create. You may specify a full path or a path that is relative to the
	///             current working directory. This parameter must not be empty.
	///	@param      CreateIntermediates  If YES, this method creates any non-existent parent directories as part of creating the directory in
	///             the path. If NO, this method fails if any of the intermediate parent directories does not exist. This method also fails if
	///             any of the intermediate path elements corresponds to a file and not a directory.
	///	@param      Attributes  The file attributes for the new directory and any newly created intermediate directories. You can set the owner
	///             and group numbers, file permissions, and modification date. If you specify nil for this parameter or omit a particular value,
	///             one or more default values are used as described in the discussion. For a list of keys you can include in this dictionary,
	///             see CPFileAttributes for a list of constants used as keys in the attributes dictionary. Some of the keys, such as
	///             CPFileHFSCreatorCode and CPFileHFSTypeCode, do not apply to directories or to Windows.
	///	@param[out] Error  If an error occurs, this is set to an actual error object containing the error information. You may exclude this
	///             parameter if you do not want the error information.
	///	@return     true if the directory was created or exists already and false otherwise.
	static bool CreateDirectoryAtPathWithIntermediateDirectoriesAndAttributes(const CPString& DirectoryPath, bool CreateIntermediates,
																			  const CPDictionary& Attributes, CPError& Error);
	static bool CreateDirectoryAtPathWithIntermediateDirectoriesAndAttributes(const CPString& DirectoryPath, bool CreateIntermediates,
																			  const CPDictionary& Attributes);

	///	@brief     Creates an array of all of the filenames found at the passed path.
	///	@details   This static method creates an array of string objects of all filenames found in the passed path.
	///	@param[in] Path  The path to the directory to be searched.
	///	@return    This returns an array with string objects of all filenames found at the path.
	static CPArray ContentsOfDirectoryAtPath(const CPString& Path);

	///	@brief     Creates an array of all of the filenames found at the passed path and matching the passed extension.
	///	@details   This static method creates an array of string objects of all filenames found in the passed path and matching the passed 
	/// extension.
	///	@param[in] Path  The path to the directory to be searched.
	///	@param[in] Extension  A strings containing the extension (without the '.') to be used to search for files.
	///	@return    This returns an array with string objects of all filenames found at the path.
	static CPArray ContentsOfDirectoryAtPathWithMatchingExtension(const CPString& Path, const CPString& Extension);

	///	@brief     Creates an array of all of the filenames found at the passed path and matching the passed extension.
	///	@details   This static method creates an array of string objects of all filenames found in the passed path and matching the passed 
	/// extension.
	///	@param[in] Path  The path to the directory to be searched.
	///	@param[in] Extensions  An array of strings containing the extensions (without the '.') to be used to search for files.
	///	@return    This returns an array with string objects of all filenames found at the path.
	static CPArray ContentsOfDirectoryAtPathWithMatchingExtensions(const CPString& Path, const CPArray& Extensions);

	///	@brief     Returns the display name of the file or directory at a specified path.
	///	@details   Display names are user-friendly names for files. They are typically used to localize standard file and directory names
	///            according to the user’s language settings. They may also reflect other modifications, such as the removal of filename
	///            extensions. Such modifications are used only when displaying the file or directory to the user and do not reflect the
	///            actual path to the item in the file system. For example, if the current user’s preferred language is French, the following
	///            code fragment logs the name Bibliothèque and not the name Library, which is the actual name of the directory.
	///	@param[in] Path  The path of a file or directory.
	///	@return    The name of the file or directory at path in a localized form appropriate for presentation to the user. If there is no file
	///            or directory at path, or if an error occurs, returns path as is.
	static CPString DisplayNameAtPath(const CPString& Path);

	///	@brief     Returns a string to a path in the search path that matches the filter passed.
	///	@details   This static method returns a string that respesents the full domain path that it uses to search for files.
	///	@param[in] Directory  The identifier of what standard directory to be searched.
	///	@param[in] DomainMask  A search mask to aact as a filter for the paths to be returned.
	///	@return    This returns a string of the search path used by the system.
	static CPString SearchPathForDirectoryInDomains(CPSearchPathDirectory Directory, CPSearchPathDomainMask DomainMask);

	///	@brief     Returns an array of strings each containing a path in the search path that matches the filter passed.
	///	@details   This static method returns an array of string that respesents the full domain path that it uses to search for files.
	///	@param[in] Directory  The identifier of what standard directory to be searched.
	///	@param[in] DomainMask  A search mask to aact as a filter for the paths to be returned.
	///	@return    This returns an array of the search path used by the system.
	static CPArray SearchPathForDirectoriesInDomains(CPSearchPathDirectory Directory, CPSearchPathDomainMask DomainMask);

	///	@brief     Return a string to the temporary directory.
	///	@details   This static method returns a string to the temporary directory used by the system.
	///	@return    This returns a string of the temporary path used by the system.
	static CPString TempDirectoryPath(void);
	
	/// @brief     Returns the platform specific location of the Documents directory.
	/// @details   This static method returns the platform specific location of the Documents directory.
	/// @return     Returns a string to the platform specific location of the Documents directory.
	static CPString UserDocumentsDirectoryPath(void);

	/// @brief     Returns the attributes of the given item.
	/// @details   This static method returns the attributes to the item at the given path.
	///	@param[in] Path  The path of a file or directory.
	/// @return    A dictionary that describes the attributes (file, directory, symlink, and so on) of the item specified by path. The keys
	///            in the dictionary are described in CPFileAttributes.
	static CPDictionary AttributesOfItemAtPath(const CPString& Path);

	/// @brief      Sets the attributes of the specified file or directory. TODO: KLS - Windows implementation!
	/// @details    The app either must own the file or directory or must be running as superuser for attribute changes to take effect.
	///             The method attempts to make all changes specified in attributes and ignores any rejection of an attempted modification.
	///             If the last component of the path is a symbolic link it is traversed.
	///             The FilePermissions value must be initialized with the code representing the POSIX file-permissions bit pattern.
	///             FileHFSCreatorCode and FileHFSTypeCode will only be heeded when path specifies a file.
	///	@param[in]  Attributes  Dictionary of attributes to set. The keys in the dictionary are described in CPFileAttributes.
	///	@param[in]  Path  The path of a file or directory.
	///	@param[out] ErrorOut  If an error occurs, this is set to an actual error object containing the error information. You may exclude this
	///             parameter if you do not want the error information.
	/// @return     true if all changes succeed. If any change fails, returns false, but it is undefined whether any changes actually occurred.
	static bool SetAttributesOfItemAtPath(const CPDictionary& Attributes, const CPString& Path, CPError& ErrorOut);
	static bool SetAttributesOfItemAtPath(const CPDictionary& Attributes, const CPString& Path);

	///	@brief     Return the size of the file.
	///	@details   This static method returns the size of the file whos path was passed in.
	///	@param[in] Path  The path to the file.
	///	@return    This returns an unsigned long long of the size of the file whos path was passed in.
	static CPULongLong SizeOfItemAtPath(const CPString& Path);
	
private:
	CPFileManager(void);
	~CPFileManager(void);
};
