////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPJSONSerialization.h
//    Convert JSON to CPObjects and convert CPObjects to JSON.
//
//  Copyright © 2010-2013 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  11/25/13  G.R.W.  Name changed from CPJsonTranslator and made it a class of all static methods
//  02/22/10  G.R.W.  Minor changes.
//  02/21/10  K.L.S.  Added this file into the CrossPlatform Library.
//  01/20/10  G.R.W.  Initial creation of file based on the test file that came with the jsoncpp library.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CrossPlatform.h"

class CPError;

//TODO: KLS - This class should probably be made to be able to return mutable objects when requested.

///
/// @brief    Converts JSON to CPObjects and converts CPObjects to JSON.
///
/// @details  This class provides methods for generating JSON encoded data in UTF8 encoding from CPObjects and generating CPObjects from
///           JSON encoded data in UTF8 encoding.
///           An object that may be converted to JSON must have the following properties:
///             - The top level object is an CPArray or CPDictionary or their mutable variation.
///             - All objects are instances of CPString, CPNumber, CPArray, CPDictionary, CPBool, or CPNull.
///             - All dictionary keys are instances of CPString.
///             - Numbers are not NaN or infinity.
///
///           For more information on JSON: http://www.json.org
///
class CP_EXTERNAL_API CPJSONSerialization
{
public:
	///
	/// @brief    Converts the JSON data to an object.
	///
	/// @details  Converts the JSON data to an object. Will return an empty object if the top level object within the JSONData was not an object.
	///
	/// @param    JSONData  The JSON data to convert.
	///
	/// @return   The converted object.
	///
	static CPObject ObjectWithJSONData(const CPData& JSONData);
	
	///
	/// @brief    Converts the JSON data to an object.
	///
	/// @details  Converts the JSON data to an object. Will return an empty object if the top level object within the JSONData was not an object.
	///
	/// @param    JSONData  The JSON data to convert.
	/// @param[out]  Error  If an error occurs, upon return it contains an error that describes the problem.
	///                     Possible errors:
	///                        code = CPObject::CPDecodeErrorCode; domain = CPObject::ErrorDomain()
	///
	/// @return   The converted object.
	///
	static CPObject ObjectWithJSONData(const CPData& JSONData, CPError& Error);
	
	///
	/// @brief    Converts the JSON data to an array.
	///
	/// @details  Converts the JSON data to an array. Will return an empty array if the top level object within the JSONData was not an array.
	///
	/// @param    JSONData  The JSON data to convert.
	///
	/// @return   The converted array.
	///
	static CPArray ArrayWithJSONData(const CPData& JSONData);
	
	///
	/// @brief       Converts the JSON data to an array.
	///
	/// @details     Converts the JSON data to an array. Will return an empty array if the top level object within the JSONData was not an array.
	///
	/// @param       JSONData  The JSON data to convert.
	/// @param[out]  Error  If an error occurs, upon return it contains an error that describes the problem.
	///                     Possible errors:
	///                        code = CPObject::CPDecodeErrorCode; domain = CPObject::ErrorDomain()
	///
	/// @return      The converted array.
	///
	static CPArray ArrayWithJSONData(const CPData& JSONData, CPError& Error);
	
	///
	/// @brief    Converts the JSON data to a dictionary.
	///
	/// @details  Converts the JSON data to a dictionary. Will return an empty dictionary if the top level object within the JSONData was not
	///           a dictionary.
	///
	/// @param    JSONData  The JSON data to convert.
	///
	/// @return   The converted dictionary.
	///
	static CPDictionary DictionaryWithJSONData(const CPData& JSONData);

	///
	/// @brief       Converts the JSON data to a dictionary.
	///
	/// @details     Converts the JSON data to a dictionary. Will return an empty dictionary if the top level object within the JSONData was
	///              not a dictionary.
	///
	/// @param       JSONData  The JSON data to convert.
	/// @param[out]  Error  If an error occurs, upon return it contains an error that describes the problem.
	///                     Possible errors:
	///                        code = CPObject::CPDecodeErrorCode; domain = CPObject::ErrorDomain()
	///
	/// @return      The converted dictionary.
	///
	static CPDictionary DictionaryWithJSONData(const CPData& JSONData, CPError& Error);

	///
	/// @brief     Converts the object to data containing JSON encoded using UTF8.
	///
	/// @details   Converts the object to a data containing JSON encoded using UTF8. Will return an empty data if the object cannot be converted.
	///
	/// @param     Object  The object to convert.
	///
	/// @return    The converted object.
	///
	static CPData JSONDataWithObject(const CPObject& Object);
	
	///
	/// @brief     Converts the object to data containing JSON encoded using UTF8.
	///
	/// @details   Converts the object to a data containing JSON encoded using UTF8. Will return an empty data if the object cannot be converted.
	///
	/// @param     Object  The object to convert.
	/// @param[out]  Error  If an error occurs, upon return it contains an error that describes the problem.
	///                     Possible errors:
	///                        code = CPObject::CPEncodeErrorCode; domain = CPObject::ErrorDomain()
	///
	/// @return    The converted object.
	///
	static CPData JSONDataWithObject(const CPObject& Object, CPError& Error);

	///
	/// @brief     Converts the array to a data containing JSON encoded using UTF8.
	///
	/// @details   Converts the array to a data containing JSON encoded using UTF8. Will return an empty data if the object cannot be converted.
	///
	/// @param     Array  The array to convert.
	///
	/// @return    The converted object.
	///
	static CPData JSONDataWithArray(const CPArray& Array);
	
	///
	/// @brief       Converts the array to a data containing JSON encoded using UTF8.
	///
	/// @details     Converts the array to a data containing JSON encoded using UTF8. Will return an empty data if the object cannot be converted.
	///
	/// @param       Array  The array to convert.
	/// @param[out]  Error  If an error occurs, upon return it contains an error that describes the problem.
	///                     Possible errors:
	///                        code = CPObject::CPEncodeErrorCode; domain = CPObject::ErrorDomain()
	///
	/// @return      The converted object.
	///
	static CPData JSONDataWithArray(const CPArray& Array, CPError& Error);
	
	///
	/// @brief     Converts the dictionary to a data containing JSON encoded using UTF8.
	///
	/// @details   Converts the dictionary to a data containing JSON encoded using UTF8. Will return an empty data if the object cannot be
	///            converted.
	///
	/// @param     Dictionary  The dictionary to encode.
	///
	/// @return    The converted object.
	///
	static CPData JSONDataWithDictionary(const CPDictionary& Dictionary);

	///
	/// @brief       Converts the dictionary to a data containing JSON encoded using UTF8.
	///
	/// @details     Converts the dictionary to a data containing JSON encoded using UTF8. Will return an empty data if the object cannot be
	///              converted.
	///
	/// @param       Dictionary  The dictionary to encode.
	/// @param[out]  Error  If an error occurs, upon return it contains an error that describes the problem.
	///                     Possible errors:
	///                        code = CPObject::CPEncodeErrorCode; domain = CPObject::ErrorDomain()
	///
	/// @return      The converted object.
	///
	static CPData JSONDataWithDictionary(const CPDictionary& Dictionary, CPError& Error);
};
