////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPJsonTranslator.h
//    Json Parser & Encoder converts from Json to a CPObject and vice versa.
//
//  Copyright © 2010-2012 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//	02/22/10  G.R.W.  Minor changes.
//	02/21/10  K.L.S.  Added this file into the CrossPlatform Library.
//	01/20/10  G.R.W.  Initial creation of file based on the test file that came with the jsoncpp library.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/CPObject.h"
#include "CrossPlatform/CPString.h"
#include "CrossPlatform/CPError.h"

//TODO: KLS - Look to split this into two classes. It performs two distinct operations that are usually not performed together
//            so maybe CPJSONEncoder and CPJSONDecoder


/// @brief     Encode/Decode JSON to/from a CPObject and subclasses of this.
/// @details   This class provides methods for generating JSON from cross platform objects.  These objects are CPObject and two of their
///            subclasses, namely CPArray, CPDictionary.  There are also static methods as an added convenience for quickly encoding and 
///            decoding JSON.  
///
///            This class is using a 3rd party open source library called JsonCpp.
///
/// For more information:
/// http://www.json.org
///

//TODO: KLS - This API MUST change; it is hard to use and doesnt even use the same verbs for the same operation and if no state data needed why have to create one.
class CP_EXTERNAL_API CPJsonTranslator
{
public:
	/// @brief     Parse JSON string and return a dictionary object.
	/// @details   Static method that parses the given JSON encoded string and returns a dictionary object.
	///	@param	   JSONString  The JSON encoded string to parse.
	/// @return    Returns a dictionary object.
	static CPDictionary DecodeJSONToDictionary(const CPString& JSONString);

	/// @brief     Parse JSON string and return an array object.
	/// @details   Static method that parses the given JSON encoded string and returns an array object.
	///	@param	   JSONString  The JSON encoded string to parse.
	/// @return    Returns an array object.
	static CPArray DecodeJSONToArray(const CPString& JSONString);
	
	/// @brief     Returns the object encoded in JSON format.
	/// @details   JSON Encodes the given object. Will return an ampty string if the object cannot be encoded.
	///	@param	   Object  The object to encode.
	/// @return    Returns the object encoded in JSON format.
	static CPString EncodeObject(const CPObject& Object);

	/// @brief     Creates a JSON translator object that is empty.
	/// @details   This object does not contain anything.
	CPJsonTranslator(void);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the same given JSON translator.
	///	@param     Translator  The JSON Translator to copy.
	CPJsonTranslator(const CPJsonTranslator& Translator);
	
	///	@brief     Destructor.
	///	@details   Removes any references it has to shared resources.
	virtual ~CPJsonTranslator(void); 
	
	///	@brief     Copy Operator.
	///	@details   Copies the contents from the given JSON translator.
	///	@param	   Translator  The JSON translator object to copy.
	const CPJsonTranslator& operator=(const CPJsonTranslator& Translator);

	/// @brief     Parse JSON string and return an object.
	/// @details   Parses the given JSON encoded string and returns an object.
	///	@param	   InputJsonString  The JSON encoded string to parse.
	/// @return    Returns an object from the parsed JSON string.
	CPObject Parse(const CPString& InputJsonString);

	//Parse the Json file and return a CPObject
	/// @brief     Parse the JSON file and return an object.
	/// @details   Parse the JSON file at the given filePath string and return an object.
	///	@param	   InputFilePath  The JSON input path and filename as a string.
	/// @return    Returns an object from the parsed JSON file.
	CPObject ParseFromFilePath(const CPString& InputFilePath);

	/// @brief     Return the JSON string from the given object.
	/// @details   Encodes the given object and returns a JSON encoded string.
	///	@param	   Object  The object to encode.
	/// @return    Returns a well formed JSON string.
	CPString Encode(const CPObject& Object) const;

	/// @brief     Return the JSON encoding into a file.
	/// @details   Encodes the given object and wrutes the JSON encoded string to the given output filePath.
	///	@param	   Object  The object to encode.
	///	@param     OutputFilePath  The file path to write out the JSON encoded object.
	/// @return    Returns an error condition.  This will be set to failure if the method failed to encode the object.
	CPError EncodeToFilePath(const CPObject& Object, const CPString& OutputFilePath);

private:
	CPString ReadInputFilePath(const CPString& FilePath);

	CPObject ParseNext(void* value);

	void* EncodeNext(const CPObject& Object) const;
};
