////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPLock.h
//      Used to coordinate the operation of multiple threads of execution within the same application.
//
//  Copyright © 2012-2013 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  01/18/13  G.R.W.  Modified to get rid of typos in the docset compile.
//  11/14/12  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/ExternalAPIMacros.h"
#include "CrossPlatform/CPObject.h"
#include "CrossPlatform/CPString.h"

/// @brief    An CPLock object is used to coordinate the operation of multiple threads of execution within the same application. A CPLock
///           object can be used to mediate access to an application’s global data or to protect a critical section of code, allowing it to
///           run atomically.
/// @details  An CPLock object is used to coordinate the operation of multiple threads of execution within the same application. A CPLock
///           object can be used to mediate access to an application’s global data or to protect a critical section of code, allowing it to
///           run atomically.
///           Warning: The CPLock class uses POSIX threads to implement its locking behavior. When sending an unlock message to an CPLock
///           object, you must be sure that message is sent from the same thread that sent the initial lock message. Unlocking a lock from a
///           different thread can result in undefined behavior.
///           You should not use this class to implement a recursive lock. Calling the lock method twice on the same thread will lock up your
///           thread permanently. Use the CPRecursiveLock class to implement recursive locks instead.
///           Unlocking a lock that is not locked is considered a programmer error and should be fixed in your code. The NSLock class reports
///           such errors by printing an error message to the console when they occur..
///
/// For more information:
/// http://developer.apple.com/library/mac/#documentation/Cocoa/Reference/Foundation/Classes/NSLock_Class
///
class CP_EXTERNAL_API CPLock : public CPObject
{
public:
	///	@brief     A class type that uniquely identifies this class.
	///	@return    Returns a class type that uniquely identifies this class.
	static CPUInteger ClassType(void);

	/// @brief    Returns the underlying shared empty object.
	/// @details  Static method that returns the underlying object that is shared between all empty objects of this class type.
	/// @returns  the underlying shared empty object.
	static const CPId& SharedEmptyObject(void);

	static CPLock LockWithName(const CPString& Name);

	CPLock(void);
	CPLock(const CPLock& Object);
	~CPLock(void);
	
	const CPLock& operator=(const CPLock& Object);

	/// @brief    Attempts to acquire a lock, blocking a thread’s execution until the lock can be acquired.
	/// @details  An application protects a critical section of code by requiring a thread to acquire a lock before executing the code. Once
	///           the critical section is completed, the thread relinquishes the lock by invoking unlock.
	void Lock(void);

	/// @brief    Relinquishes a previously acquired lock.
	void Unlock(void);

	/// @brief    Attempts to acquire a lock before a given time and returns a boolean value indicating whether the attempt was successful.
	/// @param    TimeLimit The time limit for attempting to acquire a lock.
	/// @returns  true if the lock was acquired, otherwise false.
	bool LockBeforeDate(const CPDate& TimeLimit);

	/// @brief    Attempts to acquire a lock and immediately returns a Boolean value that indicates whether the attempt was successful.
	/// @returns  true if the lock was acquired, otherwise false.
	bool TryLock(void);


	/// @brief    Returns the name associated with the receiver.
	/// @returns  the name associated with the receiver.
	CPString Name(void);

	/// @brief    Assigns a name to the receiver.
	/// @details  You can use a name string to identify a lock within your code. We also use this name as part of any error descriptions
	///           involving the receiver.
	/// @param    NewName The new name for the receiver.
	void SetName(const CPString& NewName);				
};
