////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPMutableArray.h
//    Cross platform mutable array class.
//
//  Copyright © 2009-2013 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  09/03/13  B.D.D.  Changed AddArray, AddData, AddDictionary to convert mutable input to non-mutable.
//  09/03/13  B.D.D.  Added AddMutableArray, AddMutableData, AddMutableDictionary.
//  03/15/12  G.R.W.  Added documentation to generate .docset files
//  02/23/12  G.R.W.  Created mutable file from CPArray.h and moved setter/change/remove methods to it.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/CPArray.h"

/// @brief   Cross platform mutable array class.
/// @details This derived mutable array class helps to manage mutable collections of objects.  This class will create a dynamic array.
/// 
/// For more information:
/// http://developer.apple.com/library/mac/#documentation/Cocoa/Reference/Foundation/Classes/NSMutableArray_Class
/// http://developer.apple.com/library/mac/#documentation/CoreFoundation/Reference/CFArrayRef
///
class CP_EXTERNAL_API CPMutableArray : public CPArray
{
public:
	///	@brief     A class type that uniquely identifies this class.
	///	@return    Returns a class type that uniquely identifies this class.
	static CPClassType ClassType(void);

	/// @brief    Returns the underlying shared empty object.
	/// @details  Static method that returns the underlying object that is shared between all empty objects of this class type.
	/// @returns  the underlying shared empty object.
	static const CPId& SharedEmptyObject(void);

	static CPMutableArray MutableArrayFromObject(const CPObject& Object);
	static CPMutableArray MutableArrayFromId(const CPId& Object);

	/// @brief     Default Constructor
	/// @details   Creates an empty mutable array.
	CPMutableArray(void);

	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the specified array.
	///	@param     Array  the mutable array to copy.
	CPMutableArray(const CPMutableArray& Array);

	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the specified array.
	///	@param     Array  the array to copy.
	CPMutableArray(const CPArray& Array);

	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the specified data.
	///	@param     Data  the data to copy.
	explicit CPMutableArray(const CPData& Data);

	/// @brief     Constructor.
	/// @details   This object will contain the specified object.
	///	@param     Object  the object to copy.
	explicit CPMutableArray(const CPObject& Object);

	/// @brief     Constructor.
	/// @details   This object will contain the specified object.
	///	@param     Object  the object to copy.
	explicit CPMutableArray(const CPId& Object);

	///	@brief     Destructor.
	///	@details   Removes any references it has to shared resources.
	~CPMutableArray(void);

	///	@brief     Copy Operator.
	///	@details   Copies the elements from the given mutable array.
	///	@param	   Array  the mutable array to copy.
	const CPMutableArray& operator=(const CPMutableArray& Array);

	///	@brief     Copy Operator.
	///	@details   Copies the elements from the given array.
	///	@param	   Array  the array to copy.
	const CPMutableArray& operator=(const CPArray& Array);

	///	@brief     Copy Operator.
	///	@details   Copies the specified data.
	///	@param     Data  the data to copy.
	const CPMutableArray& operator=(const CPData& Data);

	///	@brief     Add Operator.
	///	@details   Adds the elements from the given mutable array into the existing array.
	///	@param     Array  The mutable array of elements to add.
	const CPMutableArray& operator+=(const CPArray& Array);

	/// @brief     Creates a mutable array object containing the object passed in.
	///	@details   Initializes itself so that it contains the given object. Any previous contents are emptied.
	///	@param     Object  the object used to initialize the array.
	void InitByAddingObject(const CPObject& Object);

	///	@brief     Adds the object.
	///	@details   Adds the object into the existing array.
	///	@param     Object  The object to add.
	void AddObject(const CPObject& Object);

	///	@brief     Adds the array as a single object.
	///	@details   Adds the array object as a single object into the existing array.
	///	@details   If Object is a CPMutableArray it will be converted to a CPArray.
	///	@param     Object  The array to add.	
	void AddArray(const CPArray& Object);

	///	@brief     Adds the boolean.
	///	@details   Adds the boolean object into the existing array.
	///	@param     Object  The boolean to add.	
	void AddBool(const CPBool& Object);

	///	@brief     Adds the data.
	///	@details   Adds the data object into the existing array.
	///	@details   If Object is a CPMutableData it will be converted to a CPData.
	///	@param     Object  The data to add.	
	void AddData(const CPData& Object);

	///	@brief     Adds the date.
	///	@details   Adds the date object into the existing array.
	///	@param     Object  The date to add.	
	void AddDate(const CPDate& Object);

	///	@brief     Adds the dictionary.
	///	@details   Adds the dictionary object as a single object into the existing array.
	///	@details   If Object is a CPMutableDictionary it will be converted to a CPDictionary.
	///	@param     Object  The dictionary to add.	
	void AddDictionary(const CPDictionary& Object);

	///	@brief     Adds the number.
	///	@details   Adds the number object into the existing array.
	///	@param     Object  The number to add.	
	void AddNumber(const CPNumber& Object);

	///	@brief     Adds the retained object.
	///	@details   Adds the retained object into the existing array.
	///	@param     RetainedObject  The retained object to add.
	void AddRetainedObject(const CPRetainedObject* RetainedObject);

	///	@brief     Adds the string.
	///	@details   Adds the string object into the existing array.
	///	@param     Object  The string to add.	
	void AddString(const CPString& Object);

	///	@brief     Adds the value.
	///	@details   Adds the value object into the existing array.
	///	@param     Object  The value to add.
	void AddValue(const CPValue& Object);

	///	@brief     Sets the object at the specified index.
	///	@details   This sets the object at the specified index in the array.  Note that indexes start at 0.
	///	@param     Object  The object set into the array.
	///	@param     Index  An integer which is the index into the array.
	void SetObjectAtIndex(const CPObject& Object, CPUInteger Index);

	///	@brief     Sets the array at the specified index.
	///	@details   This sets the array at the specified index in the array.  Note that indexes start at 0.
	///	@param     Object  The array set into the array.
	///	@param     Index  An integer which is the index into the array.
	void SetArrayAtIndex(const CPArray& Object, CPUInteger Index);

	///	@brief     Sets the boolean at the specified index.
	///	@details   This sets the boolean at the specified index in the array.  Note that indexes start at 0.
	///	@param     Object  The boolean set into the array.
	///	@param     Index  An integer which is the index into the array.
	void SetBoolAtIndex(const CPBool& Object, CPUInteger Index);

	///	@brief     Sets the data at the specified index.
	///	@details   This sets the data at the specified index in the array.  Note that indexes start at 0.
	///	@param     Object The data set into the array.
	///	@param     Index  An integer which is the index into the array.
	void SetDataAtIndex(const CPData& Object, CPUInteger Index);

	///	@brief     Sets the date at the specified index.
	///	@details   This sets the date at the specified index in the array.  Note that indexes start at 0.
	///	@param     Object The date set into the array.
	///	@param     Index  An integer which is the index into the array.
	void SetDateAtIndex(const CPDate& Object, CPUInteger Index);

	///	@brief     Sets the dictionary at the specified index.
	///	@details   This sets the dictionary at the specified index in the array.  Note that indexes start at 0.
	///	@param     Object  The dictionary set into the array.
	///	@param     Index  An integer which is the index into the array.
	void SetDictionaryAtIndex(const CPDictionary& Object, CPUInteger Index);

	///	@brief     Sets the number at the specified index.
	///	@details   This sets the number at the specified index in the array.  Note that indexes start at 0.
	///	@param     Object  The number set into the array.
	///	@param     Index  An integer which is the index into the array.
	void SetNumberAtIndex(const CPNumber& Object, CPUInteger Index);

	///	@brief     Sets the retained object at the specified index.
	///	@details   This sets the retained object at the specified index in the array.  Note that indexes start at 0.
	///	@param     RetainedObject  The retained object set into the array.
	///	@param     Index  An integer which is the index into the array.
	void SetRetainedObjectAtIndex(const CPRetainedObject* RetainedObject, CPUInteger Index);

	///	@brief     Sets the string at the specified index.
	///	@details   This sets the string at the specified index in the array.  Note that indexes start at 0.
	///	@param     Object  The string set into the array.
	///	@param     Index  An integer which is the index into the array.
	void SetStringAtIndex(const CPString& Object, CPUInteger Index);

	///	@brief     Sets the value at the specified index.
	///	@details   This sets the value at the specified index in the array.  Note that indexes start at 0.
	///	@param     Object  The value set into the array.
	///	@param     Index  An integer which is the index into the array.
	void SetValueAtIndex(const CPValue& Object, CPUInteger Index);

	/// @brief     Sets the objects from another given array.
	/// @details   Sets the instance of array to contain all the same elements as the passed array into this method.
	///	@param     OtherArray  The array of objects to add.
	void SetArray(const CPArray& OtherArray);

	///	@brief     Adds array contents into the existing array.
	///	@details   Adds the elements of the passed array into the existing array.
	///	@param     OtherArray  The array of objects to add.
	void AddObjectsFromArray(const CPArray& OtherArray);

	///	@brief     Inserts the object at the specified index.
	///	@details   This will insert the object into the array at the specified index and move all subsequent objects to their next index.
	///	@param     Object  The object inserted into the array.
	///	@param     Index  The index position to insert into the array.
	void InsertObjectAtIndex(const CPObject& Object, CPUInteger Index);

	///	@brief     Inserts the array at the specified index.
	///	@details   This will insert the array into the array at the specified index and move all subsequent objects to their next index.
	///	@param     Object  The array inserted into the array.
	///	@param     Index  The index position to insert into the array.
	void InsertArrayAtIndex(const CPArray& Object, CPUInteger Index);
	
	///	@brief     Inserts the boolean object at the specified index.
	///	@details   This will insert the boolean object into the array at the specified index and move all subsequent objects to their next
	///            index.
	///	@param     Boolean  The boolean object to be inserted into the array.
	///	@param     Index  The index position to insert into the array.
	void InsertBoolAtIndex(const CPBool& Boolean, CPUInteger Index);
	
	///	@brief     Inserts the data at the specified index.
	///	@details   This will insert the data into the array at the specified index and move all subsequent objects to their next index.
	///	@param     Object  The data inserted into the array.
	///	@param     Index  The index position to insert into the array.
	void InsertDataAtIndex(const CPData& Object, CPUInteger Index);
	
	///	@brief     Inserts the date at the specified index.
	///	@details   This will insert the date into the array at the specified index and move all subsequent objects to their next index.
	///	@param     Object  The date inserted into the array.
	///	@param     Index  The index position to insert into the array.
	void InsertDateAtIndex(const CPDate& Object, CPUInteger Index);
	
	///	@brief     Inserts the dictionary at the specified index.
	///	@details   This will insert the dictionary into the array at the specified index and move all subsequent objects to their next
	///            index.
	///	@param     Object  The dictionary inserted into the array.
	///	@param     Index  The index position to insert into the array.
	void InsertDictionaryAtIndex(const CPDictionary& Object, CPUInteger Index);
	
	///	@brief     Inserts the number at the specified index.
	///	@details   This will insert the number into the array at the specified index and move all subsequent objects to their next index.
	///	@param     Object  The number inserted into the array.
	///	@param     Index  The index position to insert into the array.
	void InsertNumberAtIndex(const CPNumber& Object, CPUInteger Index);
	
	///	@brief     Inserts the retained object at the specified index.
	///	@details   This will insert the retained object into the array at the specified index and move all subsequent objects to their next index.
	///	@param     RetainedObject  The retained object inserted into the array.
	///	@param     Index  The index position to insert into the array.
	void InsertRetainedObjectAtIndex(const CPRetainedObject* RetainedObject, CPUInteger Index);

	///	@brief     Inserts the string at the specified index.
	///	@details   This will insert the string into the array at the specified index and move all subsequent objects to their next index.
	///	@param     Object  The string inserted into the array.
	///	@param     Index  The index position to insert into the array.
	void InsertStringAtIndex(const CPString& Object, CPUInteger Index);
	
	///	@brief     Inserts the value at the specified index.
	///	@details   This will insert the value into the array at the specified index and move all subsequent objects to their next index.
	///	@param     Object  The value inserted into the array.
	///	@param     Index  The index position to insert into the array.
	void InsertValueAtIndex(const CPValue& Object, CPUInteger Index);

	///	@brief     Removes all the objects from the array.
	void RemoveAllObjects(void);

	///	@brief     Removes the last object from the array.
	void RemoveLastObject(void);

	///	@brief     Removes the object at the index.
	///	@details   This will delete the object from the array at the specified index and move all subsequent objects to their 
	///            previous index.
	///	@param     Index  An integer which is the index into the array.
	void RemoveObjectAtIndex(CPUInteger Index);

	///	@brief     Removes the object matching the object passed in.
	///	@details   This will delete the object from the array that matches the object passed in and move all subsequent objects to their 
	///            previous index.
	///	@param     Object  The object to find in the array in order to delete it from the array.
	void RemoveObjectIdenticalTo(const CPObject& Object);

	///	@brief     Exchanges the 2 objects at the specified indexes.
	///	@details   This will swap the objects in the array at the specified indexs.
	///	@param     Object1Index  An integer which is the index into the array of the first matching object to swap.
	///	@param     Object2Index  An integer which is the index into the array of the second matching object to swap.
	void ExchangeObjectAtIndexWithObjectAtIndex(CPUInteger Object1Index, CPUInteger Object2Index);

#if __OBJC__
	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the specified mutable array.
	///	@param     Array  the mutable array to copy.
	CPMutableArray(NSMutableArray* Array);

	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the specified array.
	///	@param     Array  the array to copy.
	CPMutableArray(NSArray* Array);
	
	///	@brief     Copy Operator.
	///	@details   Copies the elements from the given mutable array.
	///	@param     Array  the mutable array to copy.
	const CPMutableArray& operator=(NSMutableArray* Array);
	
	///	@brief     Copy Operator.
	///	@details   Copies the elements from the given array.
	///	@param     Array  the array to copy.
	const CPMutableArray& operator=(NSArray* Array);
	
	///	@brief     Add Operator.
	///	@details   Adds the elements from the given array into the existing array.
	///	@param     Array  The array of elements to add.
	const CPMutableArray& operator+=(NSArray* Array);
	
	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as a NSArray*.
	operator NSArray*(void) const;
	
	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as a NSMutableArray*.
	operator NSMutableArray*(void) const;
#endif

#if __COREFOUNDATION__
	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the specified mutable array.
	///	@param     Array  the mutable array to copy.
	CPMutableArray(CFMutableArrayRef Array);

	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the specified array.
	///	@param     Array  the array to copy.
	CPMutableArray(CFArrayRef Array);

	///	@brief     Copy Operator.
	///	@details   Copies the elements from the given mutable array.
	///	@param     Array  the mutable array to copy.
	const CPMutableArray& operator=(CFMutableArrayRef Array);
	
	///	@brief     Copy Operator.
	///	@details   Copies the elements from the given array.
	///	@param     Array  the array to copy.
	const CPMutableArray& operator=(CFArrayRef Array);
	
	///	@brief     Add Operator.
	///	@details   Adds the elements from the given array into the existing array.
	///	@param     Array  The array of elements to add.
	const CPMutableArray& operator+=(CFArrayRef Array);

	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as a CFMutableArrayRef*.
	operator CFMutableArrayRef(void) const;
	
	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as a CFArrayRef*.
	operator CFArrayRef(void) const;
#endif //#if __COREFOUNDATION__

	///	@brief     Copy Method.
	///	@details   Gives a immutable copy of the array.
	CPArray Copy(void) const;

protected:
	void AssignUnderlyingArray(const CPId& Array);
	void AssignUnderlyingArrayAndTakeOwnership(const CPId& Array);

private:
	void EnsureUnderlyingArrayExistance(void);
	void AddInternalObject(CPId Object);
	void InsertInternalObjectAtIndex(CPId Object, CPUInteger Index);
	void SetInternalObjectAtIndex(CPId Object, CPUInteger Index);
};
