////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPMutableData.h
//    Cross-platform string class.
//
//  Copyright © 2011-2013 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  02/10/12  K.L.S.  Changes for CrossPlatform API changes. Reworked header includes.
//	11/04/11  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/ExternalAPIMacros.h"
#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/CPUInteger.h"
#include "CrossPlatform/CPObject.h"
#include "CrossPlatform/CPStringEncoding.h"
#include "CrossPlatform/CPData.h"

class CPRange;
class CPString;

/// @brief    Class for mutable data
/// @details  CPMutableData (and its superclass CPData) provide data objects, object-oriented wrappers for byte buffers. Data objects let 
///           simple allocated buffers (that is, data with no embedded pointers) take on the behavior of Foundation objects. They are 
///           typically used for data storage and are also useful in Distributed Objects applications, where data contained in data objects
///           can be copied or moved between applications. CPData creates static data objects, and CPMutableData creates dynamic data 
///           objects. You can easily convert one type of data object to the other with the initializer that takes a CPData object or a 
///           CPMutableData object as an argument.
///
/// For more information:
/// http://developer.apple.com/library/mac/#documentation/Cocoa/Reference/Foundation/Classes/NSMutableData_Class
/// http://developer.apple.com/library/mac/#documentation/CoreFoundation/Reference/CFMutableDataRef
///
class CP_EXTERNAL_API CPMutableData : public CPData
{
public:
	///	@brief     A class type that uniquely identifies this class.
	///	@return    Returns a class type that uniquely identifies this class.
	static CPClassType ClassType(void);

	/// @brief    Returns the underlying shared empty object.
	/// @details  Static method that returns the underlying object that is shared between all empty objects of this class type.
	/// @returns  the underlying shared empty object.
	static const CPId& SharedEmptyObject(void);

	/// @details   Static method that returns a mutable data object with a length of a given number of bytes
	static CPMutableData DataWithLength(CPUInteger NumBytes);

	static CPMutableData MutableDataFromObject(const CPObject& Object);
	static CPMutableData MutableDataFromId(const CPId& Object);

	/// @brief     Default Constructor.
	/// @details   Creates an empty mutable data object.
	CPMutableData(void);
	
	/// @brief     Copy Constructor.
	/// @details   This mutable data object will contain the object of the specified mutable data.
	///	@param     Data  the mutable data to copy.	
	CPMutableData(const CPMutableData& Data);
	
	/// @brief     Copy Constructor.
	/// @details   This mutable data object will contain the object of the specified data.
	///	@param     Data  the data to copy.	
	CPMutableData(const CPData& Data);
	
	/// @brief     Copy Constructor.
	/// @details   This mutable data object will contain the specified string.
	///	@param     String  the string to copy.	
	CPMutableData(const CPString& String);
	
	/// @brief     Copy Constructor.
	/// @details   Creates a mutable data object containing the dictionary.
	///	@param     Dictionary  A dictionary to copy.
	explicit CPMutableData(const CPDictionary& Dictionary);
	
	/// @brief     Copy Constructor.
	/// @details   This mutable data object will contain the object at the id specified.
	///	@param     Data  the underlying data object to copy.
	explicit CPMutableData(const CPId& Data);
	
	/// @brief     Copy Constructor.
	/// @details   This mutabledata object will contain the specified object.
	///	@param     Data  the data object to copy.
	explicit CPMutableData(const CPObject& Data);

	///	@brief     Destructor.
	///	@details   Removes any references it has to shared resources.
	~CPMutableData(void);

	
	///	@brief     Initializes itself so that it has a set capacity.
	///	@param     NumBytes  the number of bytes to set as a capacity
	void InitWithCapacity(CPUInteger NumBytes);
	
	///	@brief     Initializes itself so that it has a set length.
	///	@param     NumBytes  the number of bytes to set as its length
	void InitWithLength(CPUInteger NumBytes);

	///	@brief     Initializes itself so that it contains the given data.
	///	@details   Initializes itself so that it contains the given data object.  Any previous contents are emptied.
	///	@param     Data  the data used to initialize the data object.
	void InitWithData(const CPData& Data);

	///	@brief     Initializes itself so that it contains the given string.
	///	@details   Initializes itself so that it contains the given string.  Any previous contents are emptied.
	///	@param     String  the string used to initialize the data object.
	void InitWithString(const CPString& String);
	
	///	@brief     Initializes itself so that it contains the given string with the given encoding.
	///	@details   Initializes itself so that it contains the given string with the given encoding.  Any previous contents are emptied.
	///	@param     String  the string used to initialize the data object.
	///	@param     Encoding  The desired encoding to be used for the initialization.
	void InitWithStringUsingEncoding(const CPString& String, CPStringEncoding Encoding);

	///	@brief     Initializes itself so that it contains the contents of the file.
	///	@details   Initializes itself so that it contains the contents of the file.  Any previous contents are emptied.
	///	@param     FilePath  A path and filename to the data file to be read from used to initialize the data object.
	void InitWithContentsOfFile(const CPString& FilePath);

	///	@brief     Initializes itself so that it contains the given dictionary.
	///	@details   Initializes itself so that it contains the given dictionary.  Any previous contents are emptied.
	///	@param     Dictionary  An underlying dictionary object used to initialize the data object.
	void InitWithDictionary(const CPDictionary& Dictionary);
	
	///	@brief     Initializes itself so that it contains the given data buffer.
	///	@details   Initializes itself so that it contains a data object with the specified number of bytes using the given data buffer.
	///	@param     Bytes  A pointer to a byte array in memory.
	///	@param     Length  The length of the desired data object to be created.
	void InitWithBytesOfLength(const void* Bytes, CPUInteger Length);

	///	@brief     Copy Operator.
	/// @details   Assigns a mutable data object to another data object.
	///	@param     Data  A mutable data object to copy from.
	const CPMutableData& operator=(const CPMutableData& Data);

	///	@brief     Copy Operator.
	/// @details   Assigns a mutable data object to another data object.
	///	@param     Data  A data object to copy from.
	const CPMutableData& operator=(const CPData& Data);
	
	///	@brief     Copy Operator.
	/// @details   Assigns a mutable data object to the given string.
	///	@param     String  The string to copy.
	const CPMutableData& operator=(const CPString& String);
	
	///	@brief     Copy Operator.
	/// @details   Assigns a mutable data object to the given dictionary.
	///	@param     Dictionary  The dictionary to copy.
	const CPMutableData& operator=(const CPDictionary& Dictionary);

	///	@brief     Retrieves byte pointer.
	///	@details   Retrieves data as a mutable byte pointer.
	///	@return    A pointer variable to a byte array in memory which is used for output.
	void* MutableBytes(void) const;

	///	@brief     Increases the length of itself by a given length.
	///	@param     LengthToIncreaseBy  the number of bytes to increase its length by.
	void IncreaseLengthBy(CPUInteger LengthToIncreaseBy);
	
	/// @brief     Sets the new length of the data object.
	/// @details   Extends or truncates the mutable data object to a given length.
	/// @param     Length  The new length of the data object.
	void SetLength(CPUInteger Length);
	
	/// @brief     Adds a data object to the current data object.
	/// @details   Adds a data object to the current data object.
	/// @param     Append  The data object to append.
	void AppendData(const CPData& Append);
	
	/// @brief     Adds a data object of a given length to the current data object.
	/// @details   Adds a data object of a given length to the current data object.
	/// @param     Append  The data object to append.
	/// @param     NumBytes  Thge length of the given data to add.
	void AppendBytesOfLength(const void* Append, CPUInteger NumBytes);
	
	/// @brief     Replaces with a given set of bytes a given range within the current data object.
	/// @param     Range  The range within the current data object's contents to replace with bytes. The range must not exceed the bounds of
	///            the current data object.  If the location of range isn’t within the current data object’s range of bytes, a 
	///            CPRangeException is raised. The receiver is resized to accommodate the new bytes, if necessary.
	/// @param     Bytes  The data to insert into the current data object's contents.
	void ReplaceBytesInRangeWithBytes(const CPRange& Range, const void* Bytes);
	
	/// @brief     Replaces with a given set of bytes a given range within the contents of the current data object.
	/// @details   Replaces with a given set of bytes a given range within the contents of the current data object. If the length of range
	///            is not equal to Length, the current data object is resized to accommodate the new bytes.  Any bytes past range in the
	///            current data object are shifted to accommodate the new bytes. You can therefore pass NULL for Bytes and 0 for Length to 
	///            delete bytes in the receiver in the range Range.  You can also replace a range (which might be zero-length) with more 
	///            bytes than the length of the range, which has the effect of insertion (or “replace some and insert more”).
	/// @param     Range  The range within the current data object's contents to replace with bytes. The range must not exceed the bounds of
	///            the current data object.
	/// @param     Bytes  The data to insert into the current data object's contents.
	/// @param     ByteCount  The number of bytes to take from Bytes.
	void ReplaceBytesInRangeWithBytesOfLength(const CPRange& Range, const void* Bytes, CPUInteger ByteCount);
	
	/// @brief     Replaces with zeroes the contents of the current data object in a given range.
	/// @param     Range  The range within the current data object's contents to replace with bytes. The range must not exceed the bounds of
	///            the current data object.
	void ResetBytesInRange(const CPRange& Range);
	
	/// @brief    Assigns the given data object to the current data object.
	/// @param    Data  The data object which will be assigned.
	void SetData(const CPData& Data);

#if __OBJC__
	// Define methods that will be needed only when bridging to Cocoa

	/// @brief     Copy Constructor.
	/// @details   This mutable data object will contain the object of the specified mutable data.
	///	@param     Data  the mutable data to copy.	
	CPMutableData(NSMutableData* Data);
	
	/// @brief     Copy Constructor.
	/// @details   This mutable data object will contain the object of the specified mutable data.
	///	@param     Data  the mutable data to copy.	
	CPMutableData(NSData* Data);
	
	///	@brief     Copy Operator.
	/// @details   Assigns a mutable data object to another data object.
	///	@param     Data  A mutable data object to copy from.
	const CPMutableData& operator=(NSMutableData* Data);
	
	///	@brief     Copy Operator.
	/// @details   Assigns a mutable data object to another data object.
	///	@param     Data  A mutable data object to copy from.
	const CPMutableData& operator=(NSData* Data);
	
	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as a NSMutableData*.
	operator NSMutableData*() const;
#endif

#if __COREFOUNDATION__
	// Define methods that will be needed only when bridging to CoreFoundation
	
	/// @brief     Copy Constructor.
	/// @details   This mutable data object will contain the object of the specified mutable data.
	///	@param     Value  the mutable data to copy.	
	CPMutableData(CFMutableDataRef Value);
	
	///	@brief     Copy Operator.
	/// @details   Assigns a mutable data object to another data object.
	///	@param     Value  A mutable data object to copy from.
	const CPMutableData& operator=(CFMutableDataRef Value);
	
	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as a CFMutableDataRef*.
	operator CFMutableDataRef(void) const;
#endif

	///	@brief     Copy Method.
	///	@details   Gives a copy of the mutable data.
	CPData Copy(void) const;

protected:
	void AssignUnderlyingData(const CPId& Data);
	void AssignUnderlyingDataAndTakeOwnership(const CPId& Data);
};

#include "CrossPlatform/CPRange.h"
#include "CrossPlatform/CPString.h"
