////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPMutableDictionary.h
//    Cross platform mutable dictionary class.
//
//  Copyright © 2012-2013 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  09/03/13  B.D.D.  Changed SetArrayForKey, SetDataForKey, SetDictionaryForKey to convert mutable input to non-mutable.
//  09/03/13  B.D.D.  Added SetMutableArrayForKey, SetMutableDataForKey, SetMutableDictionaryForKey.
//  03/15/12  G.R.W.  Added documentation to generate .docset files
//	01/25/12  G.R.W.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/CPDictionary.h"

/// @brief   Cross platform mutable dictionary class.
/// @details The CPMutableDictionary class declares the programmatic interface to objects that manage mutable associations of keys 
/// and values. It adds modification operations to the basic operations it inherits from CPDictionary.
///
/// For more information:
/// http://developer.apple.com/library/mac/#documentation/Cocoa/Reference/Foundation/Classes/NSMutableDictionary_Class
/// http://developer.apple.com/library/mac/#documentation/CoreFoundation/Reference/CFDictionaryRef
///
class CP_EXTERNAL_API CPMutableDictionary : public CPDictionary
{
public:
	///	@brief     A class type that uniquely identifies this class.
	///	@return    Returns a class type that uniquely identifies this class.
	static CPClassType ClassType(void);

	/// @brief    Returns the underlying shared empty object.
	/// @details  Static method that returns the underlying object that is shared between all empty objects of this class type.
	/// @returns  the underlying shared empty object.
	static const CPId& SharedEmptyObject(void);

	static CPMutableDictionary MutableDictionaryFromObject(const CPObject& Object);
	static CPMutableDictionary MutableDictionaryFromId(const CPId& Object);

	/// @brief     Creates a mutable dictionary object that is empty.
	/// @details   This object does not contain any objects.
	CPMutableDictionary(void);

	/// @brief     Creates a mutable dictionary object containing the propertyList as a mutable dictionary object.
	/// @details   This object contains the same keys and objects that were passed as a mutable dictionary into the constructor.
	///	@param     Dictionary  A mutable dictionary to copy.
	CPMutableDictionary(const CPMutableDictionary& Dictionary);

	/// @brief     Creates a mutable dictionary object containing the propertyList as a dictionary object.
	/// @details   This object contains the same keys and objects that were passed as a dictionary into the constructor.
	///	@param     Dictionary  A dictionary to copy.
	CPMutableDictionary(const CPDictionary& Dictionary);

	/// @brief     Creates a mutable dictionary object containing the keys & objects from the data object.
	/// @details   This object contains the same objects that were passed as data object into the constructor.
	///	@param     Data  A data object to copy.
	explicit CPMutableDictionary(const CPData& Data);

	/// @brief     Creates a mutable dictionary object containing the keys & objects from the object.
	/// @details   This object contains the same objects that were passed as the id into the constructor.
	///	@param     Data  An underlying id object from which to extract the dictionary entries.
	explicit CPMutableDictionary(const CPId& Data);

	/// @brief     Creates a mutable dictionary object containing the keys & objects from the object.
	/// @details   This object contains the same objects that were passed as the id into the constructor.
	///	@param     Data  An underlying object from which to extract the dictionary entries.
	explicit CPMutableDictionary(const CPObject& Data);

	///	@brief     Deallocates the dictionary object. Removes its reference to its underlying object.
	///	@details   Removes its reference to its underlying object.
	~CPMutableDictionary(void);

	///	@brief     Assignment of a dictionary object to another dictionary object.
	///	@details   Assigns a mutable dictionary object to another dictionary object; a copy.
	///	@param     Dictionary  An underlying mutable dictionary object to copy.
	const CPMutableDictionary& operator=(const CPMutableDictionary& Dictionary);

	///	@brief     Assignment of a dictionary object to another dictionary object.
	///	@details   Assigns an immutable dictionary object to another dictionary object.
	///	@param     Dictionary  An underlying dictionary object to copy.
	const CPMutableDictionary& operator=(const CPDictionary& Dictionary);

	/// @brief     Assignment of a dictionary object to data object.
	/// @details   Assigns a data object containing the property list as a data object to the existing dictionary.
	///	@param     Data  A data object that contains a property list used to assign the dictionary.
	const CPMutableDictionary& operator=(const CPData& Data);

	///	@brief     Adds dictionary contents into the existing dictionary.
	///	@details   Adds the dictionary key-object entries of the passed dictionary into the existing dictionary.
	///	@param     Dictionary  The dictionary of key-object pairs to add.
	const CPMutableDictionary& operator+=(const CPDictionary& Dictionary);

	///	@brief     Adds/replaces the key-object pair in the existing dictionary.
	///	@details   Adds or replaces the object in the existing dictionary associated with the passed key.
	///	@param     Object  The object to add.	
	///	@param     Key  The string used to add the passed object based parameter.
	void SetObjectForKey(const CPObject& Object, const CPString& Key);

	///	@brief     Adds/replaces the key-array pair in the existing dictionary.
	///	@details   Adds or replaces the array in the existing dictionary associated with the passed key.
	///	@details   If Array is a CPMutableArray it will be converted to a CPArray.
	///	@param     Array  The array to add.	
	///	@param     Key  The string used to add the passed object based parameter.	
	void SetArrayForKey(const CPArray& Array, const CPString& Key);

	///	@brief     Adds/replaces the key-array pair in the existing dictionary.
	///	@details   Adds or replaces the array in the existing dictionary associated with the passed key.
	///	@param     Array  The array to add.	
	///	@param     Key  The string used to add the passed object based parameter.	
	void SetMutableArrayForKey(const CPArray& Array, const CPString& Key);

	///	@brief     Adds/replaces the key-bool pair in the existing dictionary.
	///	@details   Adds or replaces the boolean in the existing dictionary associated with the passed key.
	///	@param     Bool  The bool object to add.	
	///	@param     Key  The string used to add the passed object based parameter.	
	void SetBoolForKey(const CPBool& Bool, const CPString& Key);

	///	@brief     Adds/replaces the key-data pair in the existing dictionary.
	///	@details   Adds or replaces the data object in the existing dictionary associated with the passed key.
	///	@details   If Data is a CPMutableData it will be converted to a CPData.
	///	@param     Data  The data to add.	
	///	@param     Key  The string used to add the passed object based parameter.	
	void SetDataForKey(const CPData& Data, const CPString& Key);

	///	@brief     Adds/replaces the key-data pair in the existing dictionary.
	///	@details   Adds or replaces the data object in the existing dictionary associated with the passed key.
	///	@param     Data  The data to add.	
	///	@param     Key  The string used to add the passed object based parameter.	
	void SetMutableDataForKey(const CPData& Data, const CPString& Key);

	///	@brief     Adds/replaces the key-date pair in the existing dictionary.
	///	@details   Adds or replaces the date object in the existing dictionary associated with the passed key.
	///	@param     Date  The date to add.	
	///	@param     Key  The string used to add the passed object based parameter.	
	void SetDateForKey(const CPDate& Date, const CPString& Key);

	///	@brief     Adds/replaces the key-dictionary pair in the existing dictionary.
	///	@details   Adds or replaces the dictionary object in the existing dictionary associated with the passed key.
	///	@details   If Dictionary is a CPMutableDictionary it will be converted to a CPDictionary.
	///	@param     Dictionary  The dictionary to add.	
	///	@param     Key  The string used to add the passed object based parameter.	
	void SetDictionaryForKey(const CPDictionary& Dictionary, const CPString& Key);

	///	@brief     Adds/replaces the key-dictionary pair in the existing dictionary.
	///	@details   Adds or replaces the dictionary object in the existing dictionary associated with the passed key.
	///	@param     Dictionary  The dictionary to add.	
	///	@param     Key  The string used to add the passed object based parameter.	
	void SetMutableDictionaryForKey(const CPDictionary& Dictionary, const CPString& Key);

	///	@brief     Adds/replaces the key-number pair in the existing dictionary.
	///	@details   Adds or replaces the number object in the existing dictionary associated with the passed key.
	///	@param     Number  The number to add.	
	///	@param     Key  The string used to add the passed object based parameter.	
	void SetNumberForKey(const CPNumber& Number, const CPString& Key);

	///	@brief     Adds/replaces the key-retained object pair in the existing dictionary.
	///	@details   Adds or replaces the retained object in the existing dictionary associated with the passed key.
	///	@param     RetainedObject  The retained object to add.
	///	@param     Key  The string used to add the passed object based parameter.	
	void SetRetainedObjectForKey(const CPRetainedObject* RetainedObject, const CPString& Key);

	///	@brief     Adds/replaces the key-string pair in the existing dictionary.
	///	@details   Adds or replaces the string object in the existing dictionary associated with the passed key.
	///	@param     String  The string to add.	
	///	@param     Key  The string used to add the passed object based parameter.	
	void SetStringForKey(const CPString& String, const CPString& Key);

	///	@brief     Adds/replaces the key-string pair in the existing dictionary.
	///	@details   Adds or replaces the value object in the existing dictionary associated with the passed key.
	///	@param     Value  The value to add.	
	///	@param     Key  The string used to add the passed object based parameter.	
	void SetValueForKey(const CPValue& Value, const CPString& Key);

	///	@brief     Removes the object for the key.
	///	@details   This will delete the object from the dictionary associated with the specified key.
	///	@param     Key  The string used to add the passed object based parameter.	
	void RemoveObjectForKey(const CPString& Key);

	///	@brief     Removes the objects for the passed keys.
	///	@details   This will delete all of the objects from the dictionary associated with all of the specified key in the passed array.
	///	@param     KeyArray  The array consisting of string keys used to delete the associated object from the dictionary.
	void RemoveObjectsForKeys(const CPArray& KeyArray);

	///	@brief     Removes all the entries from the dictionary.
	///	@details   This will delete all of the key-object pair objects from the dictionary.
	void RemoveAllObjects(void);

	///	@brief     Adds dictionary contents into the existing dictionary.
	///	@details   Adds the dictionary key-object entries of the passed dictionary into the existing dictionary.
	///	@param     OtherDictionary  The dictionary of key-object pairs to add.
	void AddDictionary(const CPDictionary& OtherDictionary);

	///	@brief     Assignment of a dictionary to the existing dictionary.
	///	@details   Assigns the contents of the passed dictionary to the existing dictionary.
	///	@param     OtherDictionary  An underlying dictionary object to copy.
	void SetDictionary(const CPDictionary& OtherDictionary);

#if __OBJC__
	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the specified mutable dictionary.
	///	@param     Dictionary  the mutable dictionary to copy.
	CPMutableDictionary(NSMutableDictionary* Dictionary);
	
	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the specified dictionary.
	///	@param     Dictionary  the dictionary to copy.
	CPMutableDictionary(NSDictionary* Dictionary);
	//TODO: KLS - Probably should have one that takes an NSDictionary*. Take a good look at what is going on here!!!
	
	///	@brief     Copy Operator.
	///	@details   Copies the entries from the given mutable dictionary.
	///	@param	   Dictionary  the mutable dictionary to copy.
	const CPMutableDictionary& operator=(NSMutableDictionary* Dictionary);
	
	///	@brief     Copy Operator.
	///	@details   Copies the entries from the given dictionary.
	///	@param	   Dictionary  the dictionary to copy.
	const CPMutableDictionary& operator=(NSDictionary* Dictionary);
	
	///	@brief     Add Operator.
	///	@details   Adds the elements from the given dictionary into the existing dictionary.
	///	@param     Dictionary  The dictionary to add.
	const CPMutableDictionary& operator+=(NSDictionary* Dictionary);

	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as a NSDictionary*.
	operator NSDictionary*(void) const;

	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as a NSMutableDictionary*.
	operator NSMutableDictionary*(void) const;
#endif

#if __COREFOUNDATION__
	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the specified mutable dictionary.
	///	@param     Dictionary  the mutable dictionary to copy.
	CPMutableDictionary(CFMutableDictionaryRef Dictionary);
	
	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the specified dictionary.
	///	@param     Dictionary  the dictionary to copy.
	CPMutableDictionary(CFDictionaryRef Dictionary);

	///	@brief     Copy Operator.
	///	@details   Copies the entries from the given mutable dictionary.
	///	@param	   Dictionary  the mutable dictionary to copy.
	const CPMutableDictionary& operator=(CFMutableDictionaryRef Dictionary);
	
	///	@brief     Copy Operator.
	///	@details   Copies the entries from the given dictionary.
	///	@param	   Dictionary  the dictionary to copy.
	const CPMutableDictionary& operator=(CFDictionaryRef Dictionary);
	
	///	@brief     Add Operator.
	///	@details   Adds the elements from the given dictionary into the existing dictionary.
	///	@param     Dictionary  The dictionary to add.
	const CPMutableDictionary& operator+=(CFDictionaryRef Dictionary);

	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as a CFMutableDictionaryRef*.
	operator CFMutableDictionaryRef(void) const;
#endif

	///	@brief     Copy Method.
	///	@details   Gives a copy of the dictionary.
	CPDictionary Copy(void) const;

protected:	
	void AssignUnderlyingDictionary(const CPId& Dictionary);
	void AssignUnderlyingDictionaryAndTakeOwnership(const CPId& Dictionary);


private:
	void SetInternalObjectForKey(const CPId& Object, const CPString& Key);

};
