////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPNumber.h
//    Cross-platform number class.
//
//  Copyright © 2009-2012 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  02/10/12  K.L.S.  Changes for CrossPlatform API changes. Reworked header includes.
//  06/21/11  B.D.D.  Added unsigned long long.
//	04/02/09  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/ExternalAPIMacros.h"
#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/CPInteger.h"
#include "CrossPlatform/CPUInteger.h"
#include "CrossPlatform/CPFloat.h"
#include "CrossPlatform/CPDouble.h"
#include "CrossPlatform/CPObject.h"

class CPBool;
class CPString;

/// @brief    Class encapsulation of a number.
/// @details  CPNumber is a class that offers a value as any C scalar (numeric) type. It defines a set of methods specifically for setting 
///           and accessing the value as a signed or unsigned char, short int, int, long int, long long int, float, or double or as a BOOL. 
///           Note that number objects do not necessarily preserve the type they are created with.  It also defines a compare: method to 
///           determine the ordering of two CPNumber objects.
///
/// For more information:
/// http://developer.apple.com/library/mac/#documentation/Cocoa/Reference/Foundation/Classes/NSNumber_Class
/// http://developer.apple.com/library/mac/#documentation/CoreFoundation/Reference/CFNumberRef 
///
class CP_EXTERNAL_API CPNumber : public CPObject
{
public:
	///	@brief     A class type that uniquely identifies this class.
	///	@return    Returns a class type that uniquely identifies this class.
	static CPClassType ClassType(void);

	/// @brief    Returns the underlying shared empty object.
	/// @details  Static method that returns the underlying object that is shared between all empty objects of this class type.
	/// @returns  the underlying shared empty object.
	static const CPId& SharedEmptyObject(void);

	///	@brief   Creates a number with a bool value.
	///	@return  A number containing the bool value specified.
	static CPNumber NumberWithBool(bool Value);

	///	@brief   Creates a number with a char value.
	///	@return  A number containing the char value specified.
	static CPNumber NumberWithChar(char Value);

	///	@brief   Creates a number with a unsigned char value.
	///	@return  A number containing the unsigned char value specified.
	static CPNumber NumberWithUnsignedChar(unsigned char Value);

	///	@brief   Creates a number with a short value.
	///	@return  A number containing the short value specified.
	static CPNumber NumberWithShort(short Value);

	///	@brief   Creates a number with a unsigned short value.
	///	@return  A number containing the unsigned short value specified.
	static CPNumber NumberWithUnsignedShort(unsigned short Value);

	///	@brief   Creates a number with a int value.
	///	@return  A number containing the int value specified.
	static CPNumber NumberWithInt(int Value);

	///	@brief   Creates a number with a unsigned int value.
	///	@return  A number containing the unsigned int value specified.
	static CPNumber NumberWithUnsignedInt(unsigned int Value);

	///	@brief   Creates a number with a long value.
	///	@return  A number containing the long value specified.
	static CPNumber NumberWithLong(long Value);

	///	@brief   Creates a number with a unsigned long value.
	///	@return  A number containing the unsigned long value specified.
	static CPNumber NumberWithUnsignedLong(unsigned long Value);

	///	@brief   Creates a number with a long long value.
	///	@return  A number containing the long long value specified.
	static CPNumber NumberWithLongLong(long long Value);

	///	@brief   Creates a number with a unsigned long long value.
	///	@return  A number containing the unsigned long long value specified.
	static CPNumber NumberWithUnsignedLongLong(unsigned long long Value);

	///	@brief   Creates a number with a float value.
	///	@return  A number containing the float value specified.
	static CPNumber NumberWithFloat(float Value);
	
	///	@brief   Creates a number with a double value.
	///	@return  A number containing the double value specified.
	static CPNumber NumberWithDouble(double Value);

	static CPNumber NumberFromObject(const CPObject& Object);
	static CPNumber NumberFromId(const CPId& Object);

//	///	@brief     Gets the type of integer.  array instance based on the object passed in.
//	///	@details   This static method creates and returns an array containing the a given object.
//	///	@param     Object  An underlying object to create the array from.
//	///	@return    This returns an instance of an array.
//	int TypeOfNumber(void) const;
//
	/// @brief     Default Constructor.
	/// @details   Creates an empty number object.
	CPNumber(void);

	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the specified number.
	///	@param     Number  the number to copy.
	CPNumber(const CPNumber& Number);

	/// @brief     Copy Constructor.
	/// @details   This object contains the object that was passed into the constructor.
	///	@param     Number  the object to copy.
	explicit CPNumber(const CPObject& Number);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the number representation of the bool that was passed into the constructor.
	///	@param     Number  the bool to copy.
	explicit CPNumber(bool Number);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the number representation of the character that was passed into the constructor.
	///	@param     Number  the character to copy.
	CPNumber(char Number);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the number representation of the unsigned character that was passed into the constructor.
	///	@param     Number  the unsigned character to copy.
	CPNumber(unsigned char Number);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the number representation of the short that was passed into the constructor.
	///	@param     Number  the short to copy.
	CPNumber(short Number);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the number representation of the unsigned short that was passed into the constructor.
	///	@param     Number  the unsigned short to copy.
	CPNumber(unsigned short Number);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the number of the integer that was passed into the constructor.
	///	@param     Number  the integer to copy.
	CPNumber(int Number);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the number of the unsigned integer that was passed into the constructor.
	///	@param     Number  the unsigned integer to copy.
	CPNumber(unsigned int Number);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the number of the long that was passed into the constructor.
	///	@param     Number  the long to copy.
	CPNumber(long Number);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the number of the unsigned long that was passed into the constructor.
	///	@param     Number  the unsigned long to copy.
	CPNumber(unsigned long Number);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the number of the long long that was passed into the constructor.
	///	@param     Number  the long long to copy.
	CPNumber(long long Number);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the number of the unsigned long long that was passed into the constructor.
	///	@param     Number  the unsigned long long to copy.
	CPNumber(unsigned long long Number);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the number of the float that was passed into the constructor.
	///	@param     Number  the float to copy.
	CPNumber(float Number);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the number of the double that was passed into the constructor.
	///	@param     Number  the double to copy.
	CPNumber(double Number);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the number of the id that was passed into the constructor.
	///	@param     Number  the id to copy.
	explicit  CPNumber(const CPId& Number);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the number representation of the bool object that was passed into the constructor.
	///	@param     Number  the bool object to copy.
	CPNumber(CPBool Number);

	///	@brief     Destructor.
	///	@details   Removes any references it has to shared resources.
	~CPNumber(void);

	/// @brief     Checks if the number is a float.
	/// @details   This object returns true if the number object is a float type, otherwise it returns false.
	///	@return    Returns true if a float type otherwise false.
	bool IsFloatType(void) const;

	///	@brief     Copy Operator.
	///	@details   Copies the number from the given number.
	///	@param	   Number  the number to copy.
	const CPNumber& operator=(const CPNumber& Number);

	///	@brief     Copy Operator.
	///	@details   Copies the number representation of the given bool.
	///	@param	   Number  the bool to copy.
	const CPNumber& operator=(bool Number);
	
	///	@brief     Copy Operator.
	///	@details   Copies the number from the given character.
	///	@param	   Number  the character to copy.	
	const CPNumber& operator=(char Number);
	
	///	@brief     Copy Operator.
	///	@details   Copies the number from the given unsigned character.
	///	@param	   Number  the unsigned character to copy.	
	const CPNumber& operator=(unsigned char Number);
	
	///	@brief     Copy Operator.
	///	@details   Copies the number from the given short.
	///	@param	   Number  the short to copy.	
	const CPNumber& operator=(short Number);
	
	///	@brief     Copy Operator.
	///	@details   Copies the number from the given unsigned short.
	///	@param	   Number  the unsigned short to copy.	
	const CPNumber& operator=(unsigned short Number);
	
	///	@brief     Copy Operator.
	///	@details   Copies the number from the given integer.
	///	@param	   Number  the integer to copy.	
	const CPNumber& operator=(int Number);
	
	///	@brief     Copy Operator.
	///	@details   Copies the number from the given unsigned integer.
	///	@param	   Number  the unsigned integer to copy.	
	const CPNumber& operator=(unsigned int Number);
	
	///	@brief     Copy Operator.
	///	@details   Copies the number from the given long.
	///	@param	   Number  the long to copy.	
	const CPNumber& operator=(long Number);
	
	///	@brief     Copy Operator.
	///	@details   Copies the number from the given unsigned long.
	///	@param	   Number  the unsigned long to copy.	
	const CPNumber& operator=(unsigned long Number);
	
	///	@brief     Copy Operator.
	///	@details   Copies the number from the given long long.
	///	@param	   Number  the long long to copy.	
	const CPNumber& operator=(long long Number);
	
	///	@brief     Copy Operator.
	///	@details   Copies the number from the given unsigned long long.
	///	@param	   Number  the unsigned long long to copy.	
	const CPNumber& operator=(unsigned long long Number);
	
	///	@brief     Copy Operator.
	///	@details   Copies the number from the given float.
	///	@param	   Number  the float to copy.	
	const CPNumber& operator=(float Number);
	
	///	@brief     Copy Operator.
	///	@details   Copies the number from the given double.
	///	@param	   Number  the double to copy.	
	const CPNumber& operator=(double Number);
	
	///	@brief     Copy Operator.
	///	@details   Copies the number from the given boolean object.
	///	@param	   Number  the boolean object to copy.	
	const CPNumber& operator=(CPBool Number);

	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the number object is equal to the same number contained in the given number object.
	/// @param	   Number  A number object used in the compare.
	/// @return    A bool set to true if the number is the same, otherwise it is false.
	bool operator==(const CPNumber& Number) const;

	/// @brief     Equals Operator.
	/// @details   This checks to see if the number is equal to the same bool representation of a number.
	/// @param	   Number  A number object used in the compare.
	/// @return    A bool set to true if the number is the same, otherwise it is false.
	bool operator==(bool Number) const;
	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the number object is equal to the given character.
	/// @param	   Number  A character used in the compare.
	/// @return    A bool set to true if the number is the same, otherwise it is false.
	bool operator==(char Number) const;
	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the number object is equal to the given unsigned character.
	/// @param	   Number  The unsigned character used in the compare.
	/// @return    A bool set to true if the number is the same, otherwise it is false.
	bool operator==(unsigned char Number)const;
	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the number object is equal to the given short.
	/// @param	   Number  The short used in the compare.
	/// @return    A bool set to true if the number is the same, otherwise it is false.
	bool operator==(short Number) const;
	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the number object is equal to the given unsigned short.
	/// @param	   Number  The unsigned short used in the compare.
	/// @return    A bool set to true if the number is the same, otherwise it is false.
	bool operator==(unsigned short Number) const;
	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the number object is equal to the given integer.
	/// @param	   Number  The integer used in the compare.
	/// @return    A bool set to true if the number is the same, otherwise it is false.
	bool operator==(int Number) const;
	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the number object is equal to the given unsigned integer.
	/// @param	   Number  The unsigned integer used in the compare.
	/// @return    A bool set to true if the number is the same, otherwise it is false.
	bool operator==(unsigned int Number) const;
	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the number object is equal to the given long.
	/// @param	   Number  The long used in the compare.
	/// @return    A bool set to true if the number is the same, otherwise it is false.
	bool operator==(long Number) const;
	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the number object is equal to the given unsigned long.
	/// @param	   Number  The unsigned long used in the compare.
	/// @return    A bool set to true if the number is the same, otherwise it is false.
	bool operator==(unsigned long Number) const;
	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the number object is equal to the given long long.
	/// @param	   Number  The long longused in the compare.
	/// @return    A bool set to true if the number is the same, otherwise it is false.
	bool operator==(long long Number) const;
	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the number object is equal to the given unsigned long long.
	/// @param	   Number  The unsigned long long used in the compare.
	/// @return    A bool set to true if the number is the same, otherwise it is false.
	bool operator==(unsigned long long Number) const;
	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the number object is equal to the given float.
	/// @param	   Number  The float used in the compare.
	/// @return    A bool set to true if the number is the same, otherwise it is false.
	bool operator==(float Number) const;
	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the number object is equal to the given double.
	/// @param	   Number  The double used in the compare.
	/// @return    A bool set to true if the number is the same, otherwise it is false.
	bool operator==(double Number) const;
	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the number object is equal to the given boolean object.
	/// @param	   Number  The boolean object used in the compare.
	/// @return    A bool set to true if the number is the same, otherwise it is false.
	bool operator==(CPBool Number) const;

	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the number object is not equal to the same number contained in the compared given number object.
	/// @param	   Number  A number object used in the compare.
	/// @return    A bool set to true if the number is different, otherwise it is false.
	bool operator!=(const CPNumber& Number) const;
	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the number is not equal to the same bool representation of a number.
	/// @param	   Number  A number object used in the compare.
	/// @return    A bool set to true if the number is different, otherwise it is false.
	bool operator!=(bool Number) const;
	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the number object is not equal to the given character.
	/// @param	   Number  A character used in the compare.
	/// @return    A bool set to true if the number is different, otherwise it is false.
	bool operator!=(char Number) const;
	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the number object is not equal to the given unsigned character.
	/// @param	   Number  The unsigned character used in the compare.
	/// @return    A bool set to true if the number is different, otherwise it is false.
	bool operator!=(unsigned char Number)const;
	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the number object is not equal to the given short.
	/// @param	   Number  The short used in the compare.
	/// @return    A bool set to true if the number is different, otherwise it is false.
	bool operator!=(short Number) const;
	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the number object is not equal to the given unsigned short.
	/// @param	   Number  The unsigned short used in the compare.
	/// @return    A bool set to true if the number is different, otherwise it is false.
	bool operator!=(unsigned short Number) const;
	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the number object is not equal to the given integer.
	/// @param	   Number  The integer used in the compare.
	/// @return    A bool set to true if the number is different, otherwise it is false.
	bool operator!=(int Number) const;
	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the number object is not equal to the given unsigned integer.
	/// @param	   Number  The unsigned integer used in the compare.
	/// @return    A bool set to true if the number is different, otherwise it is false.
	bool operator!=(unsigned int Number) const;
	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the number object is not equal to the given long.
	/// @param	   Number  The long used in the compare.
	/// @return    A bool set to true if the number is different, otherwise it is false.
	bool operator!=(long Number) const;
	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the number object is not equal to the given unsigned long.
	/// @param	   Number  The unsigned long used in the compare.
	/// @return    A bool set to true if the number is different, otherwise it is false.
	bool operator!=(unsigned long Number) const;
	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the number object is not equal to the given long long.
	/// @param	   Number  The long longused in the compare.
	/// @return    A bool set to true if the number is different, otherwise it is false.
	bool operator!=(long long Number) const;
	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the number object is not equal to the given unsigned long long.
	/// @param	   Number  The unsigned long long used in the compare.
	/// @return    A bool set to true if the number is different, otherwise it is false.
	bool operator!=(unsigned long long Number) const;
	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the number object is not equal to the given float.
	/// @param	   Number  The float used in the compare.
	/// @return    A bool set to true if the number is different, otherwise it is false.
	bool operator!=(float Number) const;
	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the number object is not equal to the given double.
	/// @param	   Number  The double used in the compare.
	/// @return    A bool set to true if the number is different, otherwise it is false.
	bool operator!=(double Number) const;
	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the number object is not equal to the given boolean object.
	/// @param	   Number  The boolean object used in the compare.
	/// @return    A bool set to true if the number is different, otherwise it is false.
	bool operator!=(CPBool Number) const;

	
	///	@brief     Retrieves the number as a bool.
	///	@details   Retrieves the number object as a boolean.
	///	@return    A bool representation of the number object.
	bool boolValue(void) const;
	
	///	@brief     Retrieves the number as a character.
	///	@details   Retrieves the number object as a character.
	///	@return    A character representation of the number object.
	char CharValue(void) const;
	
	///	@brief     Retrieves the number as an unsigned character.
	///	@details   Retrieves the number object as an unsigned character.
	///	@return    An unsigned character representation of the number object.
	unsigned char UnsignedCharValue(void) const;
	
	///	@brief     Retrieves the number as a short.
	///	@details   Retrieves the number object as a short.
	///	@return    A short representation of the number object.
	short ShortValue(void) const;
	
	///	@brief     Retrieves the number as an unsigned short.
	///	@details   Retrieves the number object as an unsigned short.
	///	@return    An unsigned short representation of the number object.
	unsigned short UnsignedShortValue(void) const;
	
	///	@brief     Retrieves the number as an integer.
	///	@details   Retrieves the number object as an integer.
	///	@return    An integer representation of the number object.
	int IntValue(void) const;
	
	///	@brief     Retrieves the number as an unsigned integer.
	///	@details   Retrieves the number object as an unsigned integer.
	///	@return    An unsigned integer representation of the number object.
	unsigned UnsignedIntValue(void) const;
	
	///	@brief     Retrieves the number as a long.
	///	@details   Retrieves the number object as a long.
	///	@return    A long representation of the number object.
	long LongValue(void) const;
	
	///	@brief     Retrieves the number as an unsigned long.
	///	@details   Retrieves the number object as an unsigned long.
	///	@return    An unsigned long representation of the number object.
	unsigned long UnsignedLongValue(void) const;
	
	///	@brief     Retrieves the number as a long long.
	///	@details   Retrieves the number object as a long long.
	///	@return    A long long representation of the number object.
	long long LongLongValue(void) const;
	
	///	@brief     Retrieves the number as an unsigned long long.
	///	@details   Retrieves the number object as an unsigned long long.
	///	@return    An unsigned long long representation of the number object.
	unsigned long long UnsignedLongLongValue(void) const;
	
	///	@brief     Retrieves the number as a float.
	///	@details   Retrieves the number object as a float.
	///	@return    A float representation of the number object.
	float FloatValue(void) const;
	
	///	@brief     Retrieves the number as a double.
	///	@details   Retrieves the number object as a double.
	///	@return    A double representation of the number object.
	double DoubleValue(void) const;

	///	@brief     Retrieves the number as a boolean object.
	///	@details   Retrieves the number object as a boolean object.
	///	@return    A boolean object representation of the number object.
	CPBool BoolValue(void) const;
	
	///	@brief     Retrieves the number as an integer object.
	///	@details   Retrieves the number object as an integer object.
	///	@return    An integer object representation of the number object.
	CPInteger IntegerValue(void) const;
	
	///	@brief     Retrieves the number as an unsigned integer object.
	///	@details   Retrieves the number object as an unsigned integer object.
	///	@return    An unsigned integer object representation of the number object.
	CPUInteger UIntegerValue(void) const;
	
#ifdef __OBJC__
	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the specified number.
	///	@param     Number  the number to copy.
	CPNumber(NSNumber* Number);
	
	///	@brief     Copy Operator.
	///	@details   Copies the elements from the given number.
	///	@param	   Number  the number to copy.
	const CPNumber& operator=(NSNumber* Number);
	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the number object is equal to the same number contained in the given number object.
	/// @param	   Number  A number object used in the compare.
	/// @return    A bool set to true if the number is the same, otherwise it is false.
	bool operator==(NSNumber* Number) const;

	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the number object is not equal to the same number contained in the given number object.
	/// @param	   Number  A number object used in the compare.
	/// @return    A bool set to true if the number is different, otherwise it is false.
	bool operator!=(NSNumber* Number) const;

	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as a NSNumber*.
	operator NSNumber*(void) const;
	
	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as a BOOLValue.
	BOOL BOOLValue(void) const;
#endif  // #ifdef __OBJC__
#if __COREFOUNDATION__
	// Define methods that will be needed only when bridging to CoreFoundation

	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the specified number.
	///	@param     Number  the number to copy.
	CPNumber(CFNumberRef Number);
	
	///	@brief     Copy Operator.
	///	@details   Copies the number from the given number object.
	///	@param	   Number  the number to copy.
	const CPNumber& operator=(CFNumberRef Number);
	
	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as a CFNumberRef*.
	operator CFNumberRef(void) const;
#endif

protected:
	
private:
	operator bool(void) const;
};

#include "CrossPlatform/CPBool.h"
#include "CrossPlatform/CPString.h"
