////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPOSInfo.h
//    Cross platform operating system information class.
//
//  Copyright © 2015 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//	01/13/15  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/ExternalAPIMacros.h"
#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/CPInteger.h"
#include "CrossPlatform/CPVersion.h"

/// @brief   Cross platform operating system information class.
///
/// @details Class provides various information about the current operating system.
///
class CP_EXTERNAL_API CPOSInfo
{
public:
	/// @brief Constant identifying operating systems.
	///
	/// @details The constants consist of operating systems that we currently run on.
	///
	enum CPOSIdentifier
	{
		/// Apple Macintosh Operating System
		CPMacOS,
		/// Microsoft Windows Operating System
		CPWinOS
	};

	///	@brief   Returns the identifier for the currently running operating system.
	///
	static CPOSIdentifier OSIdentifier(void);

	///	@brief   Returns the identifier for Windows Vista operating system.
	///
	/// @details This is created because Windows does not have version numbers that logically match the OS. (6.0.0)
	///
	static CPVersion OSVersionWindowsVista(void);

	///	@brief   Returns the identifier for Windows 7 operating system.
	///
	/// @details This is created because Windows does not have version numbers that logically match the OS. (6.1.0)
	///
	static CPVersion OSVersionWindows7(void);

	///	@brief   Returns the identifier for Windows 8 operating system.
	///
	/// @details This is created because Windows does not have version numbers that logically match the OS. (6.2.0)
	///
	static CPVersion OSVersionWindows8(void);

	///	@brief   Returns the identifier for Windows 8.1 operating system.
	///
	/// @details This is created because Windows does not have version numbers that logically match the OS. (6.3.0)
	///
	static CPVersion OSVersionWindows8_1(void);

	///	@brief   Returns the identifier for Windows 10 operating system.
	///
	/// @details This is created because Windows does not have version numbers that logically match the OS. (6.4.0)
	///
	static CPVersion OSVersionWindows10(void);

	///	@brief     Returns if the operating system identifier passed in is the currently running operating system.
	///
	///	@param     OperatingSystemIdentifier - Operating system identifier to compare with.
	///
	///	@return    Returns true if the operating system running is equal to the operating system passed in, otherwise false.
	///
	static bool IsOS(CPOSIdentifier OperatingSystemIdentifier);

	///	@brief     Returns the version for the currently running operating system.
	///
	///	@return    Returns the version for the currently running operating system.
	///
	static CPVersion OSVersion(void);

	///	@brief   Returns if the current operating system version is at least the operating system version passed in.
	///
	///	@param   Version  Operating system version to compare with.
	///
	///	@return  Returns true if the operating system version running is greater than or equal to the operating system version passed in,
	///          otherwise false.
	///
	static bool IsOSAtLeastVersion(const CPVersion& Version);

private:
	CPOSInfo(void);
	~CPOSInfo(void);
};
