////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPObject.h
//    Cross-platform base class.
//
//  Copyright © 2009-2013 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  02/10/12  K.L.S.  Changes for CrossPlatform API changes. Reworked header includes.
//	10/20/09  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/// @header CPObject
/// @abstract  Base class for most "CP" classes. Provides some very basic functionality.
/// @discussion CPObject is the base class for most CrossPlatform classes. It provides some basic functionality.

#pragma once

#include "CrossPlatform/ExternalApiMacros.h"
#include "CrossPlatform/PlatformSpecificMacros.h"
#include "CrossPlatform/CPClassTypes.h"
#include "CrossPlatform/CPId.h"
#include "CrossPlatform/CPUInteger.h"

class CPString;

class CP_EXTERNAL_API CPObject
{
public:
	///	@brief    Error code constants.
	///
	///	@details  Constants of the possible errors that originate within this classes domain.
	///
	enum
	{
		/// Object is not the the type that was requested.
		CPObjectClassTypeMismatchErrorCode = 1  
	};

	///	@brief   Return the error domain for this class.
	///
	///	@return  This returns the error domain for this class.
	///
	static CPString ErrorDomain(void);

	///	@brief     The name associated with a class type.
	///	@return    Returns a class type name for the requested class type.
	static CPString ClassNameOfClassType(CPClassType ClassType);

	///	@brief     A class type that uniquely identifies this class.
	///	@return    Returns a class type that uniquely identifies this class.
	static CPClassType ClassType(void);

	/// @brief    Returns type of class needed.
	/// @details  Static method that returns the class type required to hold the given object passed in.
	/// @param    InternalObject  An id of the internal object to be checked.
	/// @returns  A string representing the class type of the given object.
	static CPClassType ClassTypeNeededForObject(const CPId& InternalObject);

	/// @brief    Returns the object of the given underlying object's class type.
	/// @details  Static method that returns the crossplatform object of the class type of the given underlying object.
	/// @param    Object  An id of the object.
	/// @returns  An object of the class type of the given object.
	static CPObject Create(const CPId& Object);

	/// @brief    Returns the underlying shared empty object.
	/// @details  Static method that returns the underlying object that is shared between all empty objects of this class type.
	/// @returns  the underlying shared empty object.
	static const CPId& SharedEmptyObject(void);

	/// @brief     Default Constructor.
	/// @details   Creates an empty object.
	CPObject(void);
	
	/// @brief     Copy Constructor.
	/// @details   This object will contain a copy of the given object.
	///	@param     OtherObject  the object to copy.
	CPObject(const CPObject& OtherObject);
	
	///	@brief     Destructor.
	///	@details   Removes any references it has to shared resources.
	virtual ~CPObject(void);

	/// @brief     Gets object's class type.
	/// @details   Returns the class type of the object.
	/// @returns   A string representing the class type of the object.
	CPClassType ObjectsClassType(void) const;
	
	/// @brief     Gets object's actual class type.
	/// @details   Returns the actual class type of the object.
	/// @returns   A string representing the class type of the object.
	CPClassType ObjectsActualClassType(void) const;

	/// @brief     Returns a bool depending on whether the child class is of the given class type.
	/// @details   Returns a bool depending on whether the child class of the existing object is of the given class type.
	/// @param	   ClassType  The class to check against.
	/// @returns   true if the existing child class type matched the given class type, otherwise false.
	bool IsTypeOfClass(CPClassType ClassType) const;
	
	/// @brief     Returns a bool depending on whether the child class is of the given class type.
	/// @details   Returns a bool depending on whether the child class of the existing object is of the given class type.
	/// @param	   Object  The class to check against.
	/// @returns   true if the existing child class type matched the given class type, otherwise false.
	bool IsSameClassTypeAs(const CPObject& Object) const;

	/// @brief     Copy operator
	/// @details   Sets this object equal to the provided object.
	/// @param	   OtherObject  An object to copy from.
	const CPObject& operator=(const CPObject& OtherObject);
	
	/// @brief     Equals Operator.
	/// @details   This checks to see if the object is equal to the same given object.
	/// @param	   OtherObject  An underlying object used in the compare.
	/// @return    true if the object is the same, otherwise false.
	bool operator==(const CPObject& OtherObject) const;
	
	/// @brief     Not Equals Operator.
	/// @details   This checks to see if the object is not equal to the given object.
	/// @param	   OtherObject  An underlying object used in the compare.
	/// @return    true if the object is different, otherwise false.
	bool operator!=(const CPObject& OtherObject) const;
	
	/// @brief     Checks to see if the object is empty.
	/// @details   This checks to see if the object is empty and returns its status.
	/// @return    true if the object was empty, otherwise false.
	virtual bool IsEmpty(void) const;
	
	/// @brief     Deleted the object contents.
	/// @details   This will delete all data in the object.
	virtual void Empty(void);
	
	/// @brief     Checks to see if the object is null.
	/// @details   This checks to see if the object is null and returns a status.
	/// @return    true if the object is null, otherwise false.
	virtual bool IsNull(void) const;
	
	/// @brief     Deleted the object contents.
	/// @details   This will delete all data in the object.
	virtual void Nullify(void);
	
	/// @brief     Checks if the object is mutable.
	/// @details   This checks to see if the object is mutable.
	/// @return    true if the object is mutable, otherwise false.
	bool IsMutable(void) const;

	virtual CPString Description(void) const;

	virtual void Retain(void);
	virtual void Release(void);
	CPUInteger RetainCount(void) const;

	CPUInteger Hash(void) const;

//TODO: KLS - look to remove these
	const CPObject& Assign(const CPId& OtherObject);
	const CPObject& AssignAndSetType(const CPId& OtherObject);
	const CPObject& AssignAndTakeOwnership(const CPId& OtherObject);
	const CPObject& AssignTakeOwnershipAndSetType(const CPId& OtherObject);
	const CPObject& AssignUsingClassType(const CPId& OtherObject, CPClassType ClassType);
	const CPObject& AssignAndTakeOwnershipUsingClassType(const CPId& OtherObject, CPClassType ClassType);

	CPId Ref(void) const;
	CPId RefCopy(void) const;
	CPId RefAutorelease(void) const;

//	virtual CPObject Copy(void) const;

protected:
	id m_Ref;
	CPClassType m_ClassType;
	CPClassType m_Type;

	CPObject(const CPId& OtherObject);
	const CPObject& operator=(const CPId& OtherObject);
	bool operator==(const CPId& OtherObject) const;
	bool operator!=(const CPId& OtherObject) const;
	
	CPObject(const CPId& OtherObject, CPClassType ClassType);
	CPObject(const CPObject& OtherObject, CPClassType ClassType);

	CPId Retain(const CPId& Object) const;
	CPId Release(const CPId& Object) const;
};

#include "CrossPlatform/CPString.h"
#include "CrossPlatform/CPArray.h"
