////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPRetainedObject.h
//    This is the base class of any C++ class that you want to be have a retain count and be able to natively go into CrossPlatform
//    collections.
//
//  Copyright © 2013 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  4/25/13  K.L.S  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/ExternalAPIMacros.h"
#include "CrossPlatform/CPObject.h"
#include "CrossPlatform/CPString.h"

/// @brief    Class encapsulation of a C++ pointer to a class.
/// @details  CPRetainedObject is the base class of any C++ class that you want to be have a retain count and be able to natively go into
///           CrossPlatform collections. This requires the use of Retain and Release just as a you would do with a Core Foundation object or
///           an Objective-C object. There is no autorelease pool implemented, just like Core Foundation, but one may be created in the future
///           if it seems like that would make coding easier. This design of this class is such that inorder for it to work correctly you
///           cant create an instance on the stack.
///           The following is an example of a subclass and what methods are necessary to get it to work correctly.
///              class Foo : public CPRetainedObject
///              {
///              public:
///                 // @ brief     Returns a new RetainedObject instance.
///                 // @ details   Returns a new RetainedObject instance. The instance has a retain count of 1.
///                 //            Call Release when you are done with it.
///                 //            Use this method as this is the only way to create an instance of your class.
///                 // @ return    a new RetainedObject instance.
///              	static Foo* Create();
///
///                 // @ brief     Returns the Foo instance by typecasting the CPRetainedObject passed in.
///                 // @ return    The Foo instance by typecasting the CPRetainedObject passed in.
///              	static Foo* DownCast(CPRetainedObject* RetainedObject);
///
///	                // @ brief     Returns a new RetainedObject instance that is a copy of the object passed in.
///                 // @ details   Returns a new RetainedObject instance that is a copy of the object passed in. The instance has a retain count of 1.
///	                //            Call Release when you are done with it.
///                 //            Use this method as this is the only way to create an instance of your class.
///                 // @ return    a new RetainedObject instance with the properties copied from this object.
///                 virtual Foo* CreateCopy(void) const;
///
///                 CPString Property(void) const; // example of a property
///                 CPString SetProperty(const CPString& Property);
///
///              private:
///                 // Methods made private to make it not possible to create a stack based instance.
///              	Foo(void);
///              	Foo(const Foo&);
///              	virtual ~Foo(void);
///
///                 CPString m_Property;
///              };
///
///              Foo* Foo::Create(void)
///              {
///              	return new Foo();
///              }
///
///              Foo* Foo::CreateCopy(void)
///              {
///              	Foo* foo = new Foo();
///                 foo->m_Property = m_Property;
///                 return foo;
///              }
///
///              CPString Foo::Property(void) const
///              {
///                 return m_Property;
///              }
///
///              CPString SetProperty(const CPString& Property)
///              {
///                 m_Property = Property;
///              }
///
///              Foo* Foo::DownCast(CPRetainedObject* RetainedObject)
///              {
///              	return dynamic_cast<Foo*>(RetainedObject);
///              }
///
///              Foo::Foo(void) :
///              CPRetainedObject()
///              {
///              }
///
///              Foo::~Foo(void)
///              {
///              }
///
class CP_EXTERNAL_API CPRetainedObject : protected CPObject
{
public:
	///	@brief     Returns the RetainedObject instance by typecasting the CPId passed in.
	///	@return    The RetainedObject instance by typecasting the CPId passed in.
	static CPRetainedObject* RetainedObject(CPId Object);

	///	@brief     Safe way to retain the object if you are not sure if it is NULL.
	static CPRetainedObject* Retain(CPRetainedObject* RetainedObject);

	///	@brief     Safe way to release the object if you are not sure if it is NULL.
	static CPRetainedObject* Release(CPRetainedObject* RetainedObject);

	///	@brief     A class type that uniquely identifies this class.
	///	@return    Returns a class type that uniquely identifies this class.
	static CPClassType ClassType(void);

	/// @brief     Returns a new RetainedObject instance that is a copy of the object passed in.
	/// @details   Returns a new RetainedObject instance that is a copy of the object passed in. The instance has a retain count of 1.
	///            Call Release when you are done with it.
	///            Use this method as this is the only way to create an instance of your class.
	///	@return    a new RetainedObject instance with the properties copied from this object.
	virtual CPRetainedObject* CreateCopy(void) const = 0;

	/// @brief    Compares the object passed in for equality. (Required for Core Foundation and Objective-C to work)
	/// @details  Compares the object passed in for equality. Base implementation will return true if the retained object pointers are equal.
	///           Overloading is highly recommended!
	/// @param    Object  the rerained object instance to compare with.
	/// @returns  true if the they objects are equal otherwise false;
	virtual bool IsEqual(const CPRetainedObject* Object) const;

	/// @brief    Returns a hash value of the object. (Required for Core Foundation and Objective-C to work)
	/// @details  Returns a hash value of the object. Base implementation will return the object pointer as the hash value.
	///           Overloading is recommended!
	/// @returns  true if the they objects are equal otherwise false;
	virtual CPUInteger Hash(void) const;

	/// @brief    Returns a string representation of the object. (Required for Core Foundation and Objective-C to work)
	/// @details  Returns a string representation of the object. Base implementation will return "<CPRetainedObject %p>" using the this pointer.
	///           Overloading is highly recommended!
	/// @returns  a string representation of the object.
	virtual CPString Description(void) const;

	///	@brief     Destructor.
	virtual ~CPRetainedObject(void);

	/// @brief    Reduces the retain count of the object by one. Will deallocate the object when the retain count goes to zero.
	/// @details  Reduces the retain count of the object by one. Will deallocate the object when the retain count goes to zero. The user
	///           needs to make sure they dont try to use the pointer after they call release.
	virtual void Release(void);

	virtual void Retain(void);
	CPUInteger RetainCount(void) const;
	virtual CPId Ref(void) const;
	const CPObject& AssignAndTakeOwnership(const CPId& OtherObject);


protected:
	/// @brief    Returns a new RetainedObject instance. The instance has a retain count of 1. Call Release when you are done with it.
	/// @details  Use this method as this is the only way to create an instance of your class. Protected to force subclassing.
	/// @returns  Returns a new RetainedObject instance.
	static CPId CreateContainer(const CPRetainedObject* RetainedObject);

	CPRetainedObject(void);
	CPRetainedObject(const CPId& OtherObject, CPClassType ClassType);
	CPRetainedObject(const CPObject& OtherObject, CPClassType ClassType);

private:
	CPRetainedObject(const CPRetainedObject& RetainedObject);
	CPRetainedObject(const CPObject& RetainedObject);
	CPRetainedObject(const CPId& RetainedObject);
	const CPRetainedObject& operator=(const CPRetainedObject& RetainedObject);

	// These methods are here to not allow calling as they  will break things
	virtual void Empty(void);
	virtual void Nullify(void);
};
