////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPScanner.h
//    Cross-platform scanner class.
//
//  Copyright © 2011-2015 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  07/12/12  G.R.W.  Started the rest of documentation.
//  06/12/12  G.R.W.  Started to add some documentation.
//	10/11/11  B.D.D.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/CPObject.h"
#include "CrossPlatform/CPString.h"

/// @brief   Represent a scanner object of a CPString object.
///
/// @details A CPScanner object interprets and converts the characters of an CPString object into number and string values.  You assign the
///          scanner’s string on creating it, and the scanner progresses through the characters of that string from beginning to end as you
///          request items. You can set an CPScanner object to ignore a set of characters as it scans the string using the
///          SetCharactersToBeSkipped method. The default set of characters to skip is the whitespace and newline character set.
///
/// For more information:
///   http://developer.apple.com/library/mac/#documentation/Cocoa/Reference/Foundation/Classes/NSScanner_Class
///
class CP_EXTERNAL_API CPScanner : public CPObject
{
public:
	///	@brief     A class type that uniquely identifies this class.
	///
	///	@return    Returns a class type that uniquely identifies this class.
	///
	static CPClassType ClassType(void);

	/// @brief    Returns the underlying shared empty object.
	///
	/// @details  Static method that returns the underlying object that is shared between all empty objects of this class type.
	///
	/// @returns  the underlying shared empty object.
	///
	static const CPId& SharedEmptyObject(void);

	/// @brief     Returns a scanner object that scans a given string.
	///
	/// @details   Returns a scanner object that scans a given string.  Sets the string to scan by invoking initWithString: with aString.
	///
	/// @param     String  A string to scan.
	///
	/// @return    This returns a static scanner object that scans the passed string.
	///
	static CPScanner ScannerWithString(const CPString& String);

	/// @brief     Creates a scanner object that is empty.
	///
	/// @details   A scanner object that is empty, it does not represent a scanner value.  This object does not have an underlying scanner
	///            object and as such is limited in functionality. Think of this like having an object pointer that is null.
	///
	CPScanner(void);

	///	@brief     Creates a scanner object that is the same as the scanner passed in.
	///
	///	@details   This copy is accomplished by using the same underlying scanner object as the scanner passed in.
	///
	///	@param     Scanner  A scanner to copy.
	///
	CPScanner(const CPScanner& Scanner);
	
	/// @brief     Assignment of a scanner object to another scanner object.
	///
	/// @details   Assigns a scanner object to another scanner object.
	///
	/// @param	   Scanner  An underlying scanner object to copy.
	///
	const CPScanner& operator=(const CPScanner& Scanner);

	/// @brief     Deallocates the scanner object. Removes its reference to its underlying object.
	///
	/// @details   Removes its reference to its underlying object.
	///
	~CPScanner(void);
	
	///	@brief     Initializes a scanner object to scan a given string.
	///
	///	@details   Initializes a scanner object to scan a given string from the beginning.  The returned object might be different than the
	///            original receiver.
	///
	/// @param     String  A string to scan.
	///
	void InitWithString(const CPString& String);
	
	/// @brief     Scans the string as long as characters from a given character set are encountered, accumulating characters into a string
	///            that’s returned by reference.
	///
	/// @details   Scans the string as long as characters from a given character set are encountered, accumulating characters into a string
	///            that’s returned by reference.
	///
	/// @param     CharacterSet  The set of characters to scan.
	/// @param     OutputString  Upon return, contains the characters scanned.
	///
	/// @return    true if the receiver scanned any characters, otherwise false.
	///
	bool ScanCharactersFromSet(const CPCharacterSet& CharacterSet, CPString& OutputString);

	/// @brief     Scans the string as long as characters from a given character set are encountered, accumulating characters into a string
	///            that’s returned by reference.
	///
	/// @details   Scans the string as long as characters from a given character set are encountered, accumulating characters into a string
	///            that’s returned by reference.  Invoke this method with NULL as OutputString to simply scan past a given set of characters.
	///
	/// @param     CharacterSet  The set of characters to scan.
	/// @param     OutputString  Upon return, contains the characters scanned.
	///
	/// @return    true if the receiver scanned any characters, otherwise false.
	///
	bool ScanCharactersFromSetIntoString(const CPCharacterSet& CharacterSet, CPString* OutputString);

	/// @brief     Scans the string as long as characters from a given character set are not encountered, accumulating characters into a
	///            string that’s returned by pointer.
	///
	/// @details   Scans the string as long as characters from a given character set are not encountered, accumulating characters into a
	///            string that’s returned by pointer.  Invoke this method with NULL as OutputString to simply scan past a given set of
	///            characters.
	///
	/// @param     CharacterSet  The set of characters to scan.
	/// @param     OutputString  Upon return, contains the characters scanned.
	///
	/// @return    true if the receiver scanned any characters, otherwise false.
	////
	bool ScanUpToCharactersFromSetIntoString(const CPCharacterSet& CharacterSet, CPString* OutputString);

	/// @brief     Returns a boolean value that indicates whether the receiver has exhausted all significant characters.
	///
	/// @return    true if the receiver has exhausted all significant characters in its string, otherwise false.
	///
	bool IsAtEnd(void);
	
	/// @brief     Sets the location at which the next scan operation will begin to a given index.
	///
	/// @details   Sets the location at which the next scan operation will begin to a given index.  This method is useful for backing up to
	///            rescan after an error.  Rather than setting the scan location directly to skip known sequences of characters, use 
	///            scanString:intoString: or scanCharactersFromSet:intoString:, which allow you to verify that the expected substring (or 
	///            set of characters) is in fact present.
	///
	/// @param	   scannerIndex  The location at which the next scan operation will begin. Raises an CPRangeException if index is beyond the
	///            end of the string being scanned.
	///
	void SetScanLocation(CPUInteger scannerIndex);
	
#if __OBJC__
	///	@brief     Creates a scanner object that is the same as the scanner passed in.
	///
	///	@details   This copy is accomplished by using the same underlying scanner object as the scanner passed in.
	///
	///	@param     Scanner  A scanner to copy.
	///
	CPScanner(NSScanner* Scanner);
	
	/// @brief     Assignment of a scanner object to another scanner object.
	///
	/// @details   Assigns a scanner object to another scanner object.
	///
	/// @param	   Scanner  An underlying scanner object to copy.
	///
	const CPScanner& operator=(NSScanner* Scanner);

	///	@brief     Conversion Operator.
	///
	///	@details   Returns the underlying object as a NSScanner*.
	///
	operator NSScanner*(void) const;
#endif

private:
#if ! __OBJC__
	CPUInteger m_StringIndex;
#endif
};
