////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPStopwatch.h
//    Class used to collect durations of events.
//
//  Copyright © 2009-2013 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  03/20/13  K.L.S.  Renamed and reworked class; it used to be called CPTimer. CPTimer will now have a different purpose.
//  02/10/12  K.L.S.  Reworked header includes.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Copyright © 2008 MYOB Technology Pty Ltd., all rights reserved.
//
//  Change History (most recent first):
//
//  03/08/08  K.L.S.  Added file.
//
//  Notes:
//    Class has a object duration tracker and a shared duration tracker. The global one is useful if you want to accumulate
//    time from multiple files. You can create several objects in different places and have them all have there durations
//    collected in one place. The local one is the default and it will allow multiple object to track durations and not
//    affect each other.
//    CPStopwatch stopwatch(true);  -- will create a stopwatch using the shared duration
//    CPStopwatch stopwatch; or CPStopwatch stopwatch(false) -- will create a stopwatch that uses its internal duration.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPDate.h"

/// @brief    CPStopwatch objects represent a class for handling elapsed time.
/// @details  CPStopwatch is used to collect durations.
///
///   You use the CPStopwatch class to create stopwatch objects or, more simply, stopwatches. These stopwatches are used mostly for debugging
///   to aid the developer in calculating time intervals between differnt pieces of code, effectively knowing how long certain routines take
///   to execute.
///
///   Once a stopwatch object is instantiated, the StartTiming method must be executed in order to begin the timing.  The StopTiming method
///   would then be executed in order to generate a time duration, which can be read using the Duration method.
///
/// @note   Class has an 'internal' duration tracker and a 'global' or shared duration tracker. The 'global' is useful if you want to
///         accumulate time from multiple files. You can create several objects in different places and have them all have their durations
///         collected in one place. The 'internal' duration tracker is local and will allow multiple objects to track durations and not
///         affect each other.	The internal duration tracker is the default. Care needs to be taken to not have two unrelated timings use
///         the global duration at the same time as one will affect the other.

class CP_EXTERNAL_API CPStopwatch
{
public:
	/// @brief    default constructor.
	/// @details  Creates an instance of the CPStopwatch class.
	CPStopwatch(void);

	/// @brief    Constructor - will it use the shared duration.
	/// @param    UseGlobalDuration  If `true`, then a shared duration is used.  If `false`, then an internal duration is used.
	CPStopwatch(bool UseGlobalDuration);

	/// @brief    destructor.
	/// @details  Removes its reference to its underlying object.
	virtual ~CPStopwatch();

	/// @brief    Starts the stopwatch.
	/// @details  Start keeping track of time passed.
	/// @see      StopTiming()
	/// @see      ResetTiming()
	void StartTiming(void);

	/// @brief    Stops the stopwatch.
	/// @details  Stops tracking time passed.  StartTiming must have been called to start tracking time.
	/// @see      StartTiming()
	/// @see      ResetTiming()
	void StopTiming(void);
	
	/// @brief    Reset the stopwatch.
	/// @details  Resets the duration and splitCount of the currently active timer.
	/// @see      StopTiming()
	/// @see      StartTiming()
	void ResetTiming(void);

	/// @brief    Gets the total duration of time passed.
	/// @details  Gets the duration of time passed.  Can be called anytime after the timer has been called.
	/// @return   Returns the total duration of time passed.
	/// @see      StartTiming()
	/// @see      StopTiming()
	CPDouble Duration(void) const;

	/// @brief    Gets the split duration of time passed.
	/// @details  Gets the duration of time passed since the last StartTiming was called.
	/// @return   Returns the split duration of time passed.
	/// @see      StartTiming()
	/// @see      StopTiming()
	/// @see      ResetTiming()
	CPDouble SplitDuration(void) const;

	/// @brief    Number of events timed since the timing was reset.
	/// @details  Each time you call StartTiming a count is incremented, it is reset to zero when ResetTiming is called.
	/// @return   A long number respresenting the number of milliseconds of time passed.
	/// @see      StartTiming()
	/// @see      StopTiming()
	/// @see      ResetTiming()
	CPUInteger SplitCount(void) const;

	/// @brief    Sends a Log message string to the duration log.
	/// @param    LogMessage  An underlying CPString instance containing the log description.
	/// @see      CPString()
	void LogDuration(CPString LogMessage) const;

	/// @brief    Sends a Log message string to the duration log.
	/// @param    LogMessage  An underlying CPString instance containing the log description.
	/// @see      CPString()
	void LogSplitDuration(CPString LogMessage) const;

	/// @brief    Sends a Log message string to the LogDuration with the current count.
	/// @param    LogMessage  An underlying CPString instance containing the log description.
	/// @see      CPString()
	void LogDurationWithSplitCount(CPString LogMessage) const;

private:
	static CPDouble m_GlobalDuration;
	static CPUInteger m_GlobalSplitCount;

	CPDouble& m_ActiveDuration;
	CPUInteger& m_ActiveSplitCount;
	CPDouble m_Duration;
	CPDouble m_SplitDuration;
	CPUInteger m_SplitCount;
	CPDouble m_SplitStartTime;

	const CPStopwatch& operator=(const CPStopwatch& OtherStopwatch);
};
