////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPString.h
//    Cross-platform string class.
//
//  Copyright © 2009-2013 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  11/14/13  B.D.D.  Added WindowsLocaleID.
//  11/08/13  B.D.D.  Added LowercaseStringWithLocale, UppercaseStringWithLocale.
//  07/01/13  B.D.D.  Added RangeOfCharacterFromSet, RangeOfCharacterFromSetWithOptions,
//                    RangeOfCharacterFromSetWithOptionsInRange and RangeOfCharacterFromSetInRange.
//  05/25/12  G.R.W.  CPString::Range() removed
//  10/20/11  B.D.D.  Added ComponentsSeparatedByCharactersInSet.
//  10/19/11  B.D.D.  Added LowercaseString, UppercaseString, CapitalizedString and StringByTrimmingCharactersInSet.
//	04/15/10  A.L.L.  Added CStringUsingEncodingWithLossyConversion, to provide lossy conversion between encodings.
//                    Added CPASCIIStringEncoding to support ASCIIString().
//	04/14/10  A.L.L.  Added StringWithCStringUsingEncoding.
//	04/13/10  A.L.L.  Added MacOSRomanString, WindowsLatin1String functions.
//	01/25/10  K.L.S.  Added DoubleToCPString function.
//	01/03/10  K.L.S.  Added constrtuctor and operator= for CPObject.
//	10/20/09  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/ExternalApiMacros.h"
#include "CrossPlatform/CPInteger.h"
#include "CrossPlatform/CPUInteger.h"
#include "CrossPlatform/CPLongLong.h"
#include "CrossPlatform/CPULongLong.h"
#include "CrossPlatform/CPUniChar.h"
#include "CrossPlatform/CPStringEncoding.h"
#include "CrossPlatform/CPObject.h"
#include "CrossPlatform/CPComparisonResult.h"
#include "CrossPlatform/CPRange.h"
#include "CrossPlatform/CPArray.h"
#include "CrossPlatform/CPData.h"
#include "CrossPlatform/CPCharacterSet.h"

class CPLocaleIdentifier;
class CPError;

/// @brief   Represent a string class.
///
/// @details CPString is a class that provides the programmatic interface for an object that manages immutable strings. An immutable string
///          is a text string that is defined when it is created and subsequently cannot be changed. CPString is implemented to represent
///          an array of Unicode characters, in other words, a text string.
///          CPString declares methods for finding and comparing strings. It also declares methods for reading numeric values from strings,
///          for combining strings in various ways, and for converting a string to different forms (such as encoding and case changes).
///          CPString is “toll-free bridged” with its Core Foundation counterpart, CFString (see CFStringRef). This means that the Core
///          Foundation type is interchangeable in function or method calls with the bridged Foundation object.
///
///          String Objects: CPString objects represent character strings in frameworks. Representing strings as objects allows you to use
///          strings wherever you use other objects. It also provides the benefits of encapsulation, so that string objects can use whatever
///          encoding and storage are needed for efficiency while simply appearing as arrays of characters.
///
///          Note: An immutable string is a text string that is defined when it is created and subsequently cannot be changed. An immutable
///          string is implemented as an array of Unicode characters (in other words, a text string).
///
///          The objects you create using CPString are referred to as string objects (or, when no confusion will result, merely as strings).
///          The term C string refers to the standard char * type. Because of the nature of class clusters, string objects aren’t actual
///          instances of the CPString classes but of one of their private subclasses. Although a string object’s class is private, its
///          interface is public, as declared by the abstract superclass CPString.
///
/// For more information:
///http://developer.apple.com/library/mac/#documentation/Cocoa/Reference/Foundation/Classes/NSString_Class
///http://developer.apple.com/library/mac/#documentation/CoreFoundation/Reference/CFStringRef
///
class CP_EXTERNAL_API CPString : public CPObject
{
public:
	enum ErrorId_Tags {
		/// Exception id for out of range
		CPRangeExceptionErrorId = 1,
	};

	enum Enum_CPStringCompareOptions
	{
		/// A case-insensitive search
		CPCaseInsensitiveSearch = 1,
		/// Exact character-by-character equivalence.
		CPLiteralSearch = 2,
		/// Search from end of source string.
		CPBackwardsSearch = 4,
		/// Search is limited to start (or end, if NSBackwardsSearch) of source string.
		CPAnchoredSearch = 8,
		/// Numbers within strings are compared using numeric value, that is, Name2.txt < Name7.txt < Name25.txt. Numeric comparison only
		/// applies to the numerals in the string, not other characters that would have meaning in a true number such as a negative sign or
		/// a decimal point. This option only applies to compare methods, not find.
		CPNumericSearch = 64
	};
	typedef CPUInteger CPStringCompareOptions;

	///	@brief     A cla3ss type that uniquely identifies this class.
	///
	///	@return    Returns a class type that uniquely identifies this class.
	///
	static CPClassType ClassType(void);

	/// @brief    Returns the underlying shared empty object.
	///
	/// @details  Static method that returns the underlying object that is shared between all empty objects of this class type. This is the
	///           same as having a string and calling the Empty() method. This is to be used as an optimization to avoid create multiple
	///           objects that will always be the same.
	///
	/// @returns  the underlying shared empty object.
	///
	static const CPId& SharedEmptyObject(void);

	/// @brief    Returns a string object that contains a nil string.
	///
	/// @details  Static method that returns a string object that contains a nil string. This is the same as having a string and calling
	///           the Nullify() method. This is to be used as an optimization to avoid create multiple objects that will always be the same.
	///
	/// @returns  a null string object.
	///
	static CPString SharedNullString(void);

	/// @brief    Returns a string object that contains a empty string.
	///
	/// @details  Static method that returns a string object that contains a empty string.
	///
	/// @returns  a empty string object.
	///
	static CPString SharedEmptyString(void);

	/// @brief    Returns a string created by using a given format string and argument values.
	///
	/// @details  Static method that returns a string created by using a given format string as a template into which the remaining argument
	///           values are substituted.
	///
	/// @param    Format  The format string to be applied using the rest of the arguments given.  For formatting info
	///           see: http://developer.apple.com/mac/library/documentation/Cocoa/Conceptual/Strings/Articles/formatSpecifiers.html
	///           Notes: Raises an CPInvalidArgumentException if Format is nil.
	/// @param    ...    This can be a variable amount of parameters that can be used as arguments in the format string.
	///
	/// @return   A string with all the given arguments formatted using the given format string.
	///
	static CPString StringWithFormat(CPString Format,...);

	/// @brief    Returns a string object initialized by reading data from the file at a given path using a given encoding.
	///
	/// @details  Static method that returns a string object initialized by reading data from the file at a given path using a given
	///           encoding.
	///
	///	@param    Path  A string of the path to the file.
	/// @param    Encoding  The string encoding to be used for making the returned string.
	///
	/// @return   A string object initialized by reading data from the file named by path using the given encoding. The returned object may
	///           be different from the original existing string. If the file can’t be opened or there is an encoding error, returns nil.
	///
	static CPString StringWithContentsOfFileUsingEncoding(const CPString& Path, CPStringEncoding Encoding);

	/// @brief    Returns a string object initialized by using a given encoding to encode the given character string.
	///
	/// @details  Static method that returns a string object initialized by using a given encoding to encode the given character string.
	///
	///	@param    String  A character string to encode.
	/// @param    Encoding  The string encoding to be used for making the returned string.
	///
	/// @return   A string object from the character string using the encoding.
	///
	static CPString StringWithCStringUsingEncoding(const char* String, CPStringEncoding Encoding);

	/// @brief    Returns a string object initialized by using a given encoding to encode the given unsigned character string.
	///
	/// @details  Static method that returns a string object initialized by using a given encoding to encode the given unsigned character
	///           string.
	///
	///	@param    String  An unsigned character string to encode.
	/// @param    Encoding  The string encoding to be used for making the returned string.
	///
	/// @return   A string object from the unsigned character string using the encoding.
	///
	static CPString StringWithCStringUsingEncoding(const unsigned char* String, CPStringEncoding Encoding);

	/// @brief    Returns a string object initialized by using a given encoding to encode the given wide character string.
	///
	/// @details  Returns a string object initialized by using a given encoding to encode the given wide character string.
	///
	///	@param    String  An wide character string to encode.
	/// @param    Encoding  The string encoding to be used for making the returned string.
	///
	/// @return   A string object from the wide character string using the encoding.
	///
	static CPString StringWithCStringUsingEncoding(const CPUniChar* String, CPStringEncoding Encoding);
#if _MAC
	static CPString StringWithCStringUsingEncoding(const wchar_t* String, CPStringEncoding Encoding);
#endif

	/// @brief    Returns a string object initialized by using a given encoding to encode the given data object.
	///
	/// @details  Static method that returns a string object initialized by using a given encoding to encode the given data object.
	///
	///	@param    Data  Data to encode.
	/// @param    Encoding  The string encoding to be used for making the returned string.
	///
	/// @return   A string object from the data using the encoding.
	///
	static CPString StringWithDataUsingEncoding(const CPData& Data, CPStringEncoding Encoding);

	/// @brief    Returns a string object initialized by using a given encoding to encode the given data object.
	///
	/// @details  Static method that returns a string object initialized by using a given encoding to encode the given data object.
	///
	///	@param    Bytes  the byte array to encode.
	/// @param    Length  the length of the bytes to be encoded.
	/// @param    Encoding  The string encoding to be used for making the returned string.
	///
	/// @return   A string object from the data using the encoding.
	///
	static CPString StringWithBytesOfLengthUsingEncoding(const void* Bytes, CPUInteger Length, CPStringEncoding Encoding);

	/// @brief    Returns a string object initialized by unicode string.
	///
	/// @details  Returns a string object initialized by unicode string. The parameter needs to be a constant that will not become unallocated
	///           during life of string.
	///
	///	@param    UnicodeEscapeSequence  the unicode string.
	///
	/// @return   A string object containing the unicode string.
	///
	static CPString StringWithUnicodeEscapeSequence(const CPUniChar* UnicodeEscapeSequence);
	static CPString StringWithUnicodeEscapeSequence(const char* UnicodeEscapeSequence);
	static CPString StringWithUnicodeEscapeSequence(const CPString& UnicodeEscapeSequence);
#if _MAC
	static CPString StringWithUnicodeEscapeSequence(const wchar_t* UnicodeEscapeSequence);
#endif

	/// @brief    Returns a string object initialized with the unicode character.
	///
	/// @details  Returns a string object initialized with the unicode character.
	///
	///	@param    UnicodeCharacter the unicode character.
	///
	/// @return   A string object containing the unicode character.
	///
	static CPString StringWithUnicodeCharacter(CPUniChar UnicodeCharacter);
#if _MAC
	static CPString StringWithUnicodeCharacter(wchar_t UnicodeCharacter);
#endif

	/// @brief    Converts the unicode escape sequence to its unicode character.
	///
	///	@param    UnicodeEscapeSequence the unicode escape sequence.
	///
	/// @return   A the unicode character.
	///
	static CPUniChar UnicodeEscapeSequenceToUniChar(const CPString& UnicodeEscapeSequence);

	///	@brief     Return an string instance from the object passed in.
	///
	///	@details   Creates and returns an string that is the object passed in. Will be empty if the object is not an string.
	///
	///	@param     Object  An string object.
	///
	///	@return    a string with the underlying string object equal to the underlying object of the object passed in.
	///
	static CPString StringFromObject(const CPObject& Object);

	///	@brief     Return an string instance from the object passed in.
	///
	///	@details   Creates and returns an string with the underlying object set to the object passed in.
	///
	///	@param     Object  An underlying string object.
	///
	///	@return    a string with the underlying string object equal to the object passed in.
	///
	static CPString StringFromId(const CPId& Object);

	/// @brief     Default Constructor.
	///
	/// @details   Creates an empty string.
	///
	CPString(void);

	/// @brief     Copy Constructor.
	///
	/// @details   This will contain the object of the given string object.
	///
	///	@param     String  the string object to copy.
	///
	CPString(const CPString& String);

	/// @brief     Copy Constructor.
	///
	/// @details   This will contain the object of the character string.
	///
	///	@param     String  the character string to copy.
	///
	CPString(const char* String);

	/// @brief     Copy Constructor.
	///
	/// @details   This will contain the object of the unsigned character string.
	///
	///	@param     String  the unsigned character string to copy.
	///
	CPString(const unsigned char* String);

	/// @brief     Copy Constructor.
	///
	/// @details   This will contain the object of the wide character string.
	///
	///	@param     String  the wide character string to copy.
	///
	CPString(const CPUniChar* String);
#if _MAC
	CPString(const wchar_t* String);
#endif

	/// @brief     Copy Constructor.
	///
	///	@details   Creates a string object from the given object.
	///
	///	@param     String  An object to copy.
	///
	explicit CPString(const CPObject& String);

	/// @brief     Copy Constructor.
	///
	///	@details   Creates a string object from the given object.
	///
	///	@param     StringData  An object to copy.
	///
	CPString(const CPData& StringData);

	/// @brief     Copy Constructor.
	///
	///	@details   Creates a string object from the object at the given id.
	///
	///	@param     String  An id of a string object to copy.
	///
	explicit CPString(const CPId& String);

	///	@brief     Destructor.
	///
	///	@details   Removes any references it has to shared resources.
	///
	~CPString(void);


	/// @brief    Creates a string initialized by using a given format string and argument values.
	///
	/// @details  Static method that creates a string by using a given format string as a template into which the remaining argument
	///           values are substituted.
	///
	/// @param    Format  The format string to be applied using the rest of the arguments given.  For formatting info
	///           see: http://developer.apple.com/mac/library/documentation/Cocoa/Conceptual/Strings/Articles/formatSpecifiers.html
	///           Notes: Raises an CPInvalidArgumentException if Format is nil.
	/// @param    ...    This can be a variable amount of parameters that can be used as arguments in the format string.
	///
	void InitWithFormat(CPString Format,...);

	/// @brief    Initializes a string by copying the given string object.
	///
	/// @details  A string object initialized by copying the given string object.
	///
	///	@param    String  The string from which to copy characters. This value must not be nil.  This raises a CPInvalidArgumentException if
	///           String is nil.
	///
	void InitWithString(const CPString& String);

	/// @brief    Initializes a string from the data file using encoding.
	///
	/// @details  Creates a string initialized by reading data from the file at a given path using a given encoding.  If the file can’t be
	///           opened or there is an encoding error, returns nil.
	///
	///	@param    Path  A string of the path to the file.
	/// @param    Encoding  The string encoding to be used for making the string.
	///
	void InitWithContentsOfFileUsingEncoding(const CPString& Path, CPStringEncoding Encoding);

	/// @brief    Initializes a string by using a given encoding to encode the given character string.
	///
	/// @details  A string object initialized by using a given encoding to encode the given character string.
	///
	///	@param    String  A character string to encode.
	/// @param    Encoding  The string encoding to be used for making the string.
	///
	void InitWithCStringUsingEncoding(const char* String, CPStringEncoding Encoding);

	/// @brief    Initializes a string by using a given encoding to encode the given unsigned character string.
	/// @details  A string object initialized by using a given encoding to encode the given unsigned character string.
	///	@param    String  An unsigned character string to encode.
	/// @param    Encoding  The string encoding to be used for making the string.
	void InitWithCStringUsingEncoding(const unsigned char* String, CPStringEncoding Encoding);

	/// @brief    Initializes a string by using a given encoding to encode the given wide character string.
	///
	/// @details  A string object initialized by using a given encoding to encode the given wide character string.
	///
	///	@param    String  A wide character string to encode.
	/// @param    Encoding  The string encoding to be used for making the string.
	///
	void InitWithCStringUsingEncoding(const CPUniChar* String, CPStringEncoding Encoding);
#if _MAC
	void InitWithCStringUsingEncoding(const wchar_t* String, CPStringEncoding Encoding);
#endif

	/// @brief    Initializes a string by using a given encoding to encode the given data object.
	///
	/// @details  A string object initialized by using a given encoding to encode the given data object.
	///
	///	@param    Data  Data to encode.
	/// @param    Encoding  The string encoding to be used for making the string.
	///
	void InitWithDataUsingEncoding(const CPData& Data, CPStringEncoding Encoding);

	/// @brief    Initializes a string by using a given encoding to encode the given data object.
	///
	/// @details  A string object initialized by using a given encoding to encode the given data object.
	///
	///	@param    Bytes  the byte array to encode.
	/// @param    Length  the length of the bytes to be encoded.
	/// @param    Encoding  The string encoding to be used for making the string.
	///
	void InitWithBytesOfLengthUsingEncoding(const void* Bytes, CPUInteger Length, CPStringEncoding Encoding);

	/// @brief    Initializes the string object with the unicode string.
	///
	/// @details  Initializes the string object with the unicode string. The parameter needs to be a constant that will not become unallocated
	///           during life of string.
	///
	///	@param    UnicodeEscapeSequence  the unicode string.
	///
	void InitWithUnicodeEscapeSequence(const CPUniChar* UnicodeEscapeSequence);
	void InitWithUnicodeEscapeSequence(const CPString& UnicodeEscapeSequence);
	void InitWithUnicodeEscapeSequence(const char* UnicodeEscapeSequence);
#if _MAC
	void InitWithUnicodeEscapeSequence(const wchar_t* UnicodeEscapeSequence);
#endif

	/// @brief    Initializes the string object with the unicode character.
	///
	/// @details  Initializes the string object with the unicode character.
	///
	///	@param    UnicodeCharacter the unicode character.
	///
	void InitWithUnicodeCharacter(CPUniChar UnicodeCharacter);
#if _MAC
	void InitWithUnicodeCharacter(wchar_t UnicodeCharacter);
#endif

	/// @brief    Initializes the string object with the default single byte encoded string.
	///
	/// @details  Initializes the string object with the default single byte encoded string. The parameter needs to be a constant that will
	///           not become unallocated during life of string.
	///
	///	@param    DefaultSingleByteConstantString  the sefault single byte string.
	///
	void InitWithDefaultSingleByteConstantString(const char* DefaultSingleByteConstantString);

	///	@brief     Copy Operator.
	///
	///	@details   Copies the string from the given string object.
	///
	///	@param	   String  the string object to copy.
	///
	const CPString& operator=(const CPString& String);

	///	@brief     Copy Operator.
	///
	///	@details   Copies the string from the given character string.
	///
	///	@param	   String  the character string to copy.
	///
	const CPString& operator=(const char* String);

	///	@brief     Copy Operator.
	///
	///	@details   Copies the string from the given unsigned character string.
	///
	///	@param	   String  the unsigned character string to copy.
	///
	const CPString& operator=(const unsigned char* String);

	///	@brief     Copy Operator.
	///
	///	@details   Copies the string from the given wide character string.
	///
	///	@param	   String  the wide character string to copy.
	///
	const CPString& operator=(const CPUniChar* String);
#if _MAC
	const CPString& operator=(const wchar_t* String);
#endif

	///	@brief     Concatenation Operator.
	///
	///	@details   Concatenates the given string object to the end of the existing string.
	///
	///	@param     String  The string object to add.
	///
	const CPString& operator+=(const CPString& String);

	///	@brief     Concatenation Operator.
	///
	///	@details   Concatenates the given character string to the end of the existing string.
	///
	///	@param     String  The character string to add.
	///
	const CPString& operator+=(const char* String);

	///	@brief     Concatenation Operator.
	///
	///	@details   Concatenates the given unsigned character string to the end of the existing string.
	///
	///	@param     String  The unsigned character string to add.
	///
	const CPString& operator+=(const unsigned char* String);

	///	@brief     Concatenation Operator.
	///
	///	@details   Concatenates the given wide character string to the end of the existing string.
	///
	///	@param     String  The wide character string to add.
	///
	const CPString& operator+=(const CPUniChar* String);
#if _MAC
	const CPString& operator+=(const wchar_t* String);
#endif

	///	@brief     Concatenation Operator.
	///
	///	@details   Concatenates the given string object to the end of the given string object.
	///
	///	@param     String1  The string object to concatenate to.
	///	@param     String2  The string object to add to String1.
	///
	friend CPString CP_EXTERNAL_API operator+(const CPString& String1, const CPString& String2);

	///	@brief     Concatenation Operator.
	///
	///	@details   Concatenates the given character string to the end of the given string object.
	///
	///	@param     String1  The string object to concatenate to.
	///	@param     String2  The character string to add to String1.
	///
	friend CPString CP_EXTERNAL_API operator+(const CPString& String1, const char* String2);

	///	@brief     Concatenation Operator.
	///
	///	@details   Concatenates the given string object to the end of the given character string.
	///
	///	@param     String1  The character string to concatenate to.
	///	@param     String2  The string object to to add to String1.
	///
	friend CPString CP_EXTERNAL_API operator+(const char* String1, const CPString& String2);

	///	@brief     Concatenation Operator.
	///
	///	@details   Concatenates the given unsigned character string to the end of the given string object.
	///
	///	@param     String1  The string object to concatenate to.
	///	@param     String2  The unsigned character string to add to String1.
	///
	friend CPString CP_EXTERNAL_API operator+(const CPString& String1, const unsigned char* String2);

	///	@brief     Concatenation Operator.
	///
	///	@details   Concatenates the given string object to the end of the given unsigned character string.
	///
	///	@param     String1  The unsigned character string to concatenate to.
	///	@param     String2  The string object to add to String1.
	///
	friend CPString CP_EXTERNAL_API operator+(const unsigned char* String1, const CPString& String2);

	///	@brief     Concatenation Operator.
	///
	///	@details   Concatenates the given wide character string to the end of the given string object.
	///
	///	@param     String1  The string object to concatenate to.
	///	@param     String2  The wide character string to add to String1.
	///
	friend CPString CP_EXTERNAL_API operator+(const CPString& String1, const CPUniChar* String2);
#if _MAC
	friend CPString CP_EXTERNAL_API operator+(const CPString& String1, const wchar_t* String2);
#endif

	///	@brief     Concatenation Operator.
	///
	///	@details   Concatenates the given wide character string to the end of the given string object.
	///
	///	@param     String1  The wide character string to concatenate to.
	///	@param     String2  The string object to add to String1.
	///
	friend CPString CP_EXTERNAL_API operator+(const CPUniChar* String1, const CPString& String2);
#if _MAC
	friend CPString CP_EXTERNAL_API operator+(const wchar_t* String1, const CPString& String2);
#endif

	/// @brief     Equals Operator.
	///
	/// @details   This checks to see if the string object is equal to the given string object.
	///
	/// @param	   String  A string object used in the compare.
	/// @return    A bool set to true if the string object is the same, otherwise it is false.
	///
	bool operator==(const CPString& String) const;

	/// @brief     Equals Operator.
	///
	/// @details   This checks to see if the string object is equal to the given character string.
	///
	/// @param	   String  A character string used in the compare.
	///
	/// @return    A bool set to true if the character string is the same, otherwise it is false.
	///
	bool operator==(const char* String) const;

	/// @brief     Equals Operator.
	///
	/// @details   This checks to see if the string object is equal to the given unsigned character string.
	///
	/// @param	   String  A unsigned character string used in the compare.
	///
	/// @return    A bool set to true if the unsigned character string is the same, otherwise it is false.
	///
	bool operator==(const unsigned char* String) const;

	/// @brief     Equals Operator.
	///
	/// @details   This checks to see if the string object is equal to the given wide character string.
	///
	/// @param	   String  A wide character string used in the compare.
	///
	/// @return    A bool set to true if the wide character string is the same, otherwise it is false.
	///
	bool operator==(const CPUniChar* String) const;
#if _MAC
	bool operator==(const wchar_t* String) const;
#endif

	/// @brief     Not Equals Operator.
	///
	/// @details   This checks to see if the string object is not equal to the given string object.
	///
	/// @param	   String  A string object used in the compare.
	///
	/// @return    A bool set to true if the string object is different, otherwise it is false.
	///
	bool operator!=(const CPString& String) const;

	/// @brief     Not Equals Operator.
	///
	/// @details   This checks to see if the string object is not equal to the given character string.
	///
	/// @param	   String  A character string used in the compare.
	///
	/// @return    A bool set to true if the character string is different, otherwise it is false.
	///
	bool operator!=(const char* String) const;

	/// @brief     Not Equals Operator.
	///
	/// @details   This checks to see if the string object is not equal to the given unsigned character string.
	///
	/// @param	   String  A unsigned character string used in the compare.
	///
	/// @return    A bool set to true if the unsigned character string is different, otherwise it is false.
	///
	bool operator!=(const unsigned char* String) const;

	/// @brief     Not Equals Operator.
	///
	/// @details   This checks to see if the string object is not equal to the given wide character string.
	///
	/// @param	   String  A wide character string used in the compare.
	///
	/// @return    A bool set to true if the wide character string is different, otherwise it is false.
	///
	bool operator!=(const CPUniChar* String) const;
#if _MAC
	bool operator!=(const wchar_t* String) const;
#endif

	/// @brief     Less than Operator.
	///
	/// @details   Is the objects string less than the given string object.
	///
	/// @param	   String  The string to compare that the objects string is less than.
	///
	/// @return    true if the objects string is less than String, otherwise false.
	///
	bool operator<(const CPString& String) const;

	/// @brief     Greater than Operator.
	///
	/// @details   Is the objects string greater than the given string object.
	///
	/// @param	   String  The string to compare that the objects string is greater than.
	///
	/// @return    true if the objects string is greater than String, otherwise false.
	///
	bool operator>(const CPString& String) const;

	/// @brief     Returns the specified character from the string.
	///
	/// @details   This operator method retrieves the unicode character specified by the given integer from the existing string.
	///
	/// @param	   Index  The index into the string which will be used to retireve the specified character.
	///
	/// @return    Returns the unicode character from the string.
	///
	/// @exception CPException of type RangeException.
	///
	CPUniChar operator[](CPUInteger Index) const;
	CPUniChar operator[](int Index) const;

	/// @brief     Returns the specified character from the string.
	///
	/// @details   This operator method retrieves the unicode character specified by the given integer from the existing string.
	///
	/// @param	   Index  The index into the string which will be used to retireve the specified character.
	///
	/// @return    Returns the unicode character from the string.
	///
	/// @exception CPException of type RangeException.
	///
	CPUniChar CharacterAtIndex(CPUInteger Index) const;

	/// @brief     Returns a null-terminated UTF8 representation of the object.
	///
	/// @details   The returned C string is automatically freed just as a returned object would be released; you should copy the C string if
	///            it needs to be stored it outside of the memory context in which the C string is created.**** Add Windows specifics
	///
	/// @return    A null-terminated UTF8 representation of the object.
	///
	const char* UTF8CString(void) const;

	/// @brief     Returns a null-terminated single byte encoding, MacOSRoman (Mac) or WindowLatin1 (Windows) representation of the object.
	///
	/// @details   The returned C string is automatically freed just as a returned object would be released; you should copy the C string if
	///            it needs to be stored it outside of the memory context in which the C string is created.**** Add Windows specifics
	///
	/// @return    A null-terminated MacOSRoman (Mac) or WindowLatin1 (Windows) representation of the object.
	///
	const char* DefaultOSSingleByteCString(void) const;

	/// @brief     Returns a representation of the object as a C string using a given encoding.
	///
	/// @details   The returned C string is guaranteed to be valid only until either the receiver is freed, or until the current memory is
	///            emptied, whichever occurs first. You should copy the C string or use DataContainingCStringUsingEncoding if it needs to
	///            store the C string beyond this time.**** Add Windows specifics
	///            This method can return NULL if the string cant be represented in the given encoding. If lossy conversion wanted use
	///            DataContainingCStringUsingEncoding.
	///
	/// @param	   StringEncoding  The encoding for the returned C string.
	///
	/// @return    A representation of the object as a C string using a given encoding.
	///
	const char* CStringUsingEncoding(CPStringEncoding StringEncoding) const;

	/// @brief     Returns a null-terminated Unicode representation of the object.
	///
	/// @details   The returned C string is automatically freed just as a returned object would be released; you should copy the C string if
	///            it needs to be stored it outside of the memory context in which the C string is created.**** Add Windows specifics
	///
	/// @return    A null-terminated Unicode representation of the object.
	///
	const CPUniChar* UnicodeCString(void) const;

	/// @brief     Is Equals To.
	///
	/// @details   This checks to see if the string object is equal to the given string object.
	///
	/// @param	   String  A string object used in the compare.
	///
	/// @return    A bool set to true if the string object is the same, otherwise it is false.
	///
	bool IsEqualToString(const CPString& String) const;

	/// @brief     Is Equals To.
	///
	/// @details   This checks to see if the string object is equal to the given character string.
	///
	/// @param	   String  A character string used in the compare.
	///
	/// @return    A bool set to true if the character string is the same, otherwise it is false.
	///
	bool IsEqualToString(const char* String) const;

	/// @brief     Is Equals To.
	///
	/// @details   This checks to see if the string object is equal to the given unsigned character string.
	///
	/// @param	   String  A unsigned character string used in the compare.
	///
	/// @return    A bool set to true if the unsigned character string is the same, otherwise it is false.
	///
	bool IsEqualToString(const unsigned char* String) const;

	/// @brief     Is Equals To.
	///
	/// @details   This checks to see if the string object is equal to the given wide character string.
	///
	/// @param	   String  A wide character string used in the compare.
	///
	/// @return    A bool set to true if the wide character string is the same, otherwise it is false.
	///
	bool IsEqualToString(const CPUniChar* String) const;
#if _MAC
	bool IsEqualToString(const wchar_t* String) const;
#endif

	/// @brief     Is Equals To without regard to case.
	///
	/// @details   This checks to see if the string object is equal to the given string object without regard to case.
	///
	/// @param	   String  A string object used in the compare.
	///
	/// @return    A bool set to true if the string object is the same without regard to case., otherwise it is false.
	///
	bool IsEqualToStringIgnoringCase(const CPString& String) const;


	/// @brief     Compares a given string to the existing string.
	///
	/// @details   Returns a CPComparisonResult value that indicates the lexical ordering within the existing string and the given string.
	///
	/// @param	   String  The string with which to compare the existing string.  This value must not be nil. If this value is nil, the
	///            behavior is undefined and may change in future versions of Mac OS X.
	///
	/// @return    Returns a CPComparisonResult value that indicates the lexical ordering within the existing string and the given string.
	///
	CPComparisonResult Compare(const CPString& String) const;

	/// @brief     Compares a given string to the existing string using the given search options.
	///
	/// @details   Returns a CPComparisonResult value that indicates the lexical ordering within the existing string and the given string.
	///
	/// @param	   String  The string with which to compare the existing string.  This value must not be nil. If this value is nil, the
	///            behavior is undefined and may change in future versions of Mac OS X.
	/// @param	   CompareOptions  The specified option used to sort by.  These search options can be: Case Insensitive, Literal, Backwards,
	///            Anchored, or Numeric.
	///
	/// @return    Returns a CPComparisonResult value that indicates the lexical ordering within the existing string and the given string.
	///
	CPComparisonResult CompareWithOptions(const CPString& String, CPStringCompareOptions CompareOptions) const;

	/// @brief     Compares a given string to the existing string using the given case insensitive search option.
	///
	/// @details   Returns a CPComparisonResult value that indicates the lexical ordering within the existing string and the given string.
	///
	/// @param	   String  The string with which to compare the existing string.  This value must not be nil. If this value is nil, the
	///            behavior is undefined and may change in future versions of Mac OS X.
	///
	/// @return    Returns a CPComparisonResult value that indicates the lexical ordering within the existing string and the given string.
	///
	CPComparisonResult CaseInsensitiveCompare(const CPString& String) const;

	/// @brief     Compares a given string to the existing string using a localized comparison.
	///
	/// @details   Returns a CPComparisonResult value that indicates the lexical ordering within the existing string and the given string.
	///
	/// @param	   String  The string with which to compare the existing string.  This value must not be nil. If this value is nil, the
	///            behavior is undefined and may change in future versions of Mac OS X.
	///
	/// @return    Returns a CPComparisonResult value that indicates the lexical ordering within the existing string and the given string.
	///
	CPComparisonResult LocalizedCompare(const CPString& String) const;

	/// @brief     Compares a given string to the existing string using the given case insensitive search option and a localized comparison.
	///
	/// @details   Returns a CPComparisonResult value that indicates the lexical ordering within the existing string and the given string.
	///
	/// @param	   String  The string with which to compare the existing string.  This value must not be nil. If this value is nil, the
	///            behavior is undefined and may change in future versions of Mac OS X.
	///
	/// @return    Returns a CPComparisonResult value that indicates the lexical ordering within the existing string and the given string.
	///
	CPComparisonResult LocalizedCaseInsensitiveCompare(const CPString& String) const;

	///	@brief     Returns whether or not the existing string is empty.
	///
	///	@return    true if the existing string is empty, otherwise false.
	///
	bool IsEmpty(void) const;

	/// @brief    The length of the string.
	///
	/// @details  The length method returns the total number of Unicode characters in the string object.  The number returned includes the
	///           individual characters of composed character sequences, so you cannot use this method to determine if a string will be
	///           visible when printed or how long it will appear.
	///
	/// @return   An integer representing the total number of unicode characters in the string object.
	///
	CPUInteger Length(void) const;

	/// @brief    Finds and returns the range of the first occurrence of a given character set within the existing string.
	///
	/// @details  Finds and returns the range of the first occurrence of a given character set within the existing string.  This method actually
	///           invokes the RangeOfCharacterFromSetWithOptions method with no options.  This method detects all invalid ranges (including those with
	///           negative lengths).  For applications linked against Mac OS X v10.6 and later, this error causes an exception; for
	///           applications linked against earlier releases, this error causes a warning, which is displayed just once per application
	///           execution.
	///
	/// @param    FindCharacterSet  The character set to search for. This value must not be nil.
	///
	/// @return   A CPRange object giving the location and length in the existing string of the first occurrence of FindCharacterSet. Returns
	///           {CPNotFound, 0} if FindCharacterSet is not found or is empty (@"").
	///
	CPRange RangeOfCharacterFromSet(const CPCharacterSet& FindCharacterSet) const;

	/// @brief    Finds and returns the range of the first occurrence of a given character set within the existing string, subject to given options.
	///
	/// @details  Finds and returns the range of the first occurrence of a given character set within the existing string, subject to given options.
	///           This method actually invokes the RangeOfCharacterFromSetWithOptionsInRange method with the options specified by Options and the
	///           entire extent of the existing string as the range.  This method detects all invalid ranges (including those with negative
	///           lengths). For applications linked against Mac OS X v10.6 and later, this error causes an exception; for applications
	///           linked against earlier releases, this error causes a warning, which is displayed just once per application execution.
	///
	/// @param    FindCharacterSet  The string to search for. This value must not be nil.
	/// @param    Options  A mask specifying search options. The following options may be specified by combining them with the C bitwise OR
	///           operator: CPCaseInsensitiveSearch, CPLiteralSearch, CPBackwardsSearch, CPAnchoredSearch.
	///
	/// @return   A CPRange object giving the location and length in the existing string of the first occurrence of FindCharacterSet. Returns
	///           {CPNotFound, 0} if FindCharacterSet is not found or is empty (@"").
	///
	CPRange RangeOfCharacterFromSetWithOptions(const CPCharacterSet& FindCharacterSet, CPStringCompareOptions Options) const;

	/// @brief    Finds and returns the range of the first occurrence of a given character set, within the given range of the existing string,
	///           subject to given options.
	///
	/// @details  Finds and returns the range of the first occurrence of a given character set, within the given range of the existing string,
	///           subject to given options.	 The length of the returned range and that of FindCharacterSet may differ if equivalent composed
	///           character sequences are matched.  This method detects all invalid ranges (including those with negative lengths). For
	///           applications linked against Mac OS X v10.6 and later, this error causes an exception; for applications linked against
	///           earlier releases, this error causes a warning, which is displayed just once per application execution.
	///
	/// @param    FindCharacterSet  The character set to search for. This value must not be nil.
	/// @param    Options  A mask specifying search options. The following options may be specified by combining them with the C bitwise OR
	///           operator: CPCaseInsensitiveSearch, CPLiteralSearch, CPBackwardsSearch, CPAnchoredSearch.
	/// @param    SearchRange  The range within the existing string for which to search for FindCharacterSet.  Important: Raises a
	///           CPRangeException if any part of SearchRange lies beyond the end of the string.
	///
	/// @return   A CPRange object giving the location and length in the existing string of FindCharacterSet within SearchRange in the existing
	///           string, modulo the options in mask.  The range returned is relative to the start of the string, not to the passed-in
	///           range.  Returns {CPNotFound, 0} if FindCharacterSet is not found or is empty (@"").
	///
	CPRange RangeOfCharacterFromSetWithOptionsInRange(const CPCharacterSet& FindCharacterSet, CPStringCompareOptions Options, const CPRange& SearchRange) const;

	/// @brief    Finds and returns the range of the first occurrence of a given character set, within the given range of the existing string.
	///
	/// @details  Finds and returns the range of the first occurrence of a given character set, within the given range of the existing string.
	///           This method actually invokes the RangeOfCharacterFromSetWithOptionsInRange method with no options specified.  The length of the
	///           returned range and that of FindCharacterSet may differ if equivalent composed character sequences are matched.  This method
	///           detects all invalid ranges (including those with negative lengths). For applications linked against Mac OS X v10.6 and
	///           later, this error causes an exception; for applications linked against earlier releases, this error causes a warning,
	///           which is displayed just once per application execution.
	///
	/// @param    FindCharacterSet  The character set to search for. This value must not be nil.
	/// @param    SearchRange  The range within the existing string for which to search for FindCharacterSet.  Important: Raises a
	///           CPRangeException if any part of SearchRange lies beyond the end of the string.
	///
	/// @return   A CPRange object giving the location and length in the existing string of FindCharacterSet within SearchRange in the existing
	///           string.  The range returned is relative to the start of the string, not to the passed-in range.  Returns {CPNotFound, 0}
	///           if FindCharacterSet is not found or is empty (@"").
	///
	CPRange RangeOfCharacterFromSetInRange(const CPCharacterSet& FindCharacterSet, const CPRange& SearchRange) const;

	/// @brief    Finds and returns the range of the first occurrence of a given string within the existing string.
	///
	/// @details  Finds and returns the range of the first occurrence of a given string within the existing string.  This method actually
	///           invokes the RangeOfStringWithOptions method with no options.  This method detects all invalid ranges (including those with
	///           negative lengths).  For applications linked against Mac OS X v10.6 and later, this error causes an exception; for
	///           applications linked against earlier releases, this error causes a warning, which is displayed just once per application
	///           execution.
	///
	/// @param    FindString  The string to search for. This value must not be nil.
	///
	/// @return   A CPRange object giving the location and length in the existing string of the first occurrence of FindString. Returns
	///           {CPNotFound, 0} if FindString is not found or is empty (@"").
	///
	CPRange RangeOfString(const CPString& FindString) const;

	/// @brief    Finds and returns the range of the first occurrence of a given string within the existing string, subject to given options.
	///
	/// @details  Finds and returns the range of the first occurrence of a given string within the existing string, subject to given options.
	///           This method actually invokes the RangeOfStringWithOptionsInRange method with the options specified by Options and the
	///           entire extent of the existing string as the range.  This method detects all invalid ranges (including those with negative
	///           lengths). For applications linked against Mac OS X v10.6 and later, this error causes an exception; for applications
	///           linked against earlier releases, this error causes a warning, which is displayed just once per application execution.
	///
	/// @param    FindString  The string to search for. This value must not be nil.
	/// @param    Options  A mask specifying search options. The following options may be specified by combining them with the C bitwise OR
	///           operator: CPCaseInsensitiveSearch, CPLiteralSearch, CPBackwardsSearch, CPAnchoredSearch.
	///
	/// @return   A CPRange object giving the location and length in the existing string of the first occurrence of FindString. Returns
	///           {CPNotFound, 0} if FindString is not found or is empty (@"").
	///
	CPRange RangeOfStringWithOptions(const CPString& FindString, CPStringCompareOptions Options) const;

	/// @brief    Finds and returns the range of the first occurrence of a given string, within the given range of the existing string,
	///           subject to given options.
	///
	/// @details  Finds and returns the range of the first occurrence of a given string, within the given range of the existing string,
	///           subject to given options.	 The length of the returned range and that of FindString may differ if equivalent composed
	///           character sequences are matched.  This method detects all invalid ranges (including those with negative lengths). For
	///           applications linked against Mac OS X v10.6 and later, this error causes an exception; for applications linked against
	///           earlier releases, this error causes a warning, which is displayed just once per application execution.
	///
	/// @param    FindString  The string to search for. This value must not be nil.
	/// @param    Options  A mask specifying search options. The following options may be specified by combining them with the C bitwise OR
	///           operator: CPCaseInsensitiveSearch, CPLiteralSearch, CPBackwardsSearch, CPAnchoredSearch.
	/// @param    SearchRange  The range within the existing string for which to search for FindString.  Important: Raises a
	///           CPRangeException if any part of SearchRange lies beyond the end of the string.
	///
	/// @return   A CPRange object giving the location and length in the existing string of FindString within SearchRange in the existing
	///           string, modulo the options in mask.  The range returned is relative to the start of the string, not to the passed-in
	///           range.  Returns {CPNotFound, 0} if FindString is not found or is empty (@"").
	///
	CPRange RangeOfStringWithOptionsInRange(const CPString& FindString, CPStringCompareOptions Options, const CPRange& SearchRange) const;

	/// @brief    Finds and returns the range of the first occurrence of a given string, within the given range of the existing string.
	///
	/// @details  Finds and returns the range of the first occurrence of a given string, within the given range of the existing string.
	///           This method actually invokes the RangeOfStringWithOptionsInRange method with no options specified.  The length of the
	///           returned range and that of FindString may differ if equivalent composed character sequences are matched.  This method
	///           detects all invalid ranges (including those with negative lengths). For applications linked against Mac OS X v10.6 and
	///           later, this error causes an exception; for applications linked against earlier releases, this error causes a warning,
	///           which is displayed just once per application execution.
	///
	/// @param    FindString  The string to search for. This value must not be nil.
	/// @param    SearchRange  The range within the existing string for which to search for FindString.  Important: Raises a
	///           CPRangeException if any part of SearchRange lies beyond the end of the string.
	///
	/// @return   A CPRange object giving the location and length in the existing string of FindString within SearchRange in the existing
	///           string.  The range returned is relative to the start of the string, not to the passed-in range.  Returns {CPNotFound, 0}
	///           if FindString is not found or is empty (@"").
	///
	CPRange RangeOfStringInRange(const CPString& FindString, const CPRange& SearchRange) const;


	/// @brief     Retrieves a string from beginning up to, but not including, character at index.
	///
	/// @details   Returns a new string containing the characters of the existing string up to, but not including, the one at a given index.
	///
	/// @param	   ToIndex  An index. The value must lie within the bounds of the existing string, or be equal to the length of the existing
	///            string.  This raises a CPRangeException if (ToIndex - 1) lies beyond the end of the existing string.
	///
   	/// @return    A new string containing the characters of the existing string up to, but not including, the one at ToIndex. If ToIndex is
	///            equal to the length of the string, returns a copy of the existing string.
	///
	CPString SubstringToIndex(CPUInteger ToIndex) const;

	/// @brief     Returns a string from character at index to the end.
	///
	/// @details   Returns a new string containing the characters of the existing string from the one at a given index to the end.  This
	///            raises a CPRangeException if From lies beyond the end of the existing string.
	///
	/// @param	   FromIndex  An index. The value must lie within the bounds of the existing string, or be equal to the length of the
	///            existing string.
	///
	/// @return    A new string containing the characters of the existing string from the one at FromIndex to the end. If FromIndex is equal
	///            to the length of the string, returns an empty string.
	///
	CPString SubstringFromIndex(CPUInteger FromIndex) const;

	/// @brief     Returns a string object containing the characters of the existing string that lie within a given range.
	///
	/// @details   Returns a string object containing the characters of the existing string that lie within a given range.  This method
	///            treats the length of the string as a valid range value that returns an empty string.  This method detects all invalid
	///            ranges (including those with negative lengths).  For applications linked against Mac OS X v10.6 and later, this error
	///            causes an exception; for applications linked against earlier releases, this error causes a warning, which is displayed
	///            just once per application execution.
	///
	/// @param	   Range  A range. The range must not exceed the bounds of the existing string.
	///
	/// @return    A string object containing the characters of the  existing string that lie within Range.
	///
	CPString SubstringWithRange(CPRange Range) const;

	/// @brief     Checks to see if whether a given string matches the beginning characters of the existing string.
	///
	/// @details   Returns a bool that indicates whether a given string matches the beginning characters of the existing string.
	///
	/// @param	   Prefix  A string used in the compare.
	///
	/// @return    A bool set to true if Prefix matches the beginning characters of the existing string, otherwise false. Returns false if
	///            Prefix is empty.
	///
	bool HasPrefix(const CPString& Prefix) const;

	/// @brief     Checks to see if whether a given string matches the ending characters of the existing string.
	///
	/// @details   Returns a bool that indicates whether a given string matches the ending characters of the existing string.
	///
	/// @param	   Suffix  A string used in the compare.
	///
	/// @return    A bool set to true if Suffix matches the ending characters of the existing string, otherwise false. Returns false if
	///            Suffix is empty.
	///
	bool HasSuffix(const CPString& Suffix) const;

	/// @brief     Returns an array containing substrings from the existing string that have been divided by a given separator.
	///
	/// @details   Returns an array containing substrings from the existing string that have been divided by a given separator.  The
	///            substrings in the array appear in the order they did in the existing string.  Adjacent occurrences of the separator
	///            string produce empty strings in the result. Similarly, if the string begins or ends with the separator, the first or last
	///            substring, respectively, is empty. For example, this code fragment:
	///
	///            CPString list = "Norman, Stanley, Fletcher";
	///            CPArray listItems = list.ComponentsSeparatedByString(", ");
	///
	///            produces an array { "Norman", "Stanley", "Fletcher" }.
	///
	///            If list begins with a comma and space—for example, ", Norman, Stanley, Fletcher"—the array has these contents:
	///            { "", "Norman", "Stanley", "Fletcher" }
	///
	///            If list has no separators—for example, "Norman"—the array contains the string itself, in this case { "Norman" }.
	///
	/// @param	   Separator  The separator string.
	///
   	/// @return    An array object containing substrings from the existing string that have been divided by Separator.
	///
	CPArray ComponentsSeparatedByString(const CPString& Separator) const;

	/// @brief     Returns an array containing substrings from the existing string that have been divided by characters in a given set.
	///
	/// @details   Returns an array containing substrings from the existing string that have been divided by characters in a given set.  The
	///            substrings in the array appear in the order they did in the existing string.  Adjacent occurrences of the separator
	///            characters produce empty strings in the result.  Similarly, if the string begins or ends with separator characters, the
	///            first or last substring, respectively, is empty.
	///
	/// @param	   CharacterSet  A character set containing the characters to to use to split the existing string. Must not be nil.
	///
   	/// @return    An array object containing substrings from the existing string that have been divided by characters in CharacterSet.
	///
	CPArray ComponentsSeparatedByCharactersInSet(const CPCharacterSet& CharacterSet) const;


	/// @brief     Returns a new string in which all occurrences of a given target string are replaced by another given string.
	///
	/// @details   Returns a new string in which all occurrences of a given target string in the existing string are replaced by another
	///            given string.
	///
	/// @param	   FindString  The string to replace.
	/// @param	   ReplacementString  The string with which to replace FindString.
	///
   	/// @return    A new string in which all occurrences of FindString in the existing string are replaced by ReplacementString.
	///
	CPString StringByReplacingOccurrencesOfStringWithString(const CPString& FindString, const CPString& ReplacementString) const;

	/// @brief     Returns a data containg the string in the encoding specified.
	///
	/// @details   Returns a data containg the string in the encoding specified. Some encodings can produce a buffer that is not equal to
	///            the string, for example, encoding is a single byte encoding and the string contains characters that dont exist within it.
	///
	/// @param	   Encoding  The encoding that we want the string to be in the data.
	///
	/// @return    A data with the contents of the string in the encoding specified.
	///
	CPData DataContainingStringUsingEncoding(CPStringEncoding Encoding) const;

	/// @brief     Returns a data containg the string in the encoding UTF8.
	///
	/// @details   Returns a data containg the string in the encoding UTF8.
	///
	/// @return    A data with the contents of the string in the encoding UTF8.
	///
	CPData DataContainingUTF8String(void) const;

	/// @brief     Returns a data containg the string in the encoding ASCII.
	///
	/// @details   Returns a data containg the string in the encoding ASCII.
	///
	/// @return    A data with the contents of the string in the encoding ASCII.
	///
	CPData DataContainingASCIIString(void) const;

	/// @brief     Returns a data containg the string in the encoding MacOSRoman.
	///
	/// @details   Returns a data containg the string in the encoding MacOSRoman.
	///
	/// @return    A data with the contents of the string in the encoding MacOSRoman.
	///
	CPData DataContainingMacOSRomanString(void) const;

	/// @brief     Returns a data containg the string in the encoding Windows Latin 1.
	///
	/// @details   Returns a data containg the string in the encoding Windows Latin 1.
	///
	/// @return    A data with the contents of the string in the encoding Windows Latin 1.
	///
	CPData DataContainingWindowsLatin1String(void) const;

	/// @brief     Returns a data containg the string in the encoding UNICODE 16.
	///
	/// @details   Returns a data containg the string in the encoding UNICODE 16.
	///
	/// @return    A data with the contents of the string in the encoding UNICODE 16.
	///
	CPData DataContainingUnicodeString(void) const;

	/// @brief     Returns a data containg the string in the encoding specified. Data will have a C string terminator at the end.
	///
	/// @details   Returns a data containg the string in the encoding specified. Some encodings can produce a buffer that is not equal to
	///            the string, for example, encoding is a single byte encoding and the string contains characters that dont exist within it.
	///
	/// @param	   Encoding  The encoding that we want the string to be in the data.
	///
	/// @return    A data with the contents of the string in the encoding specified.
	///
	CPData DataContainingCStringUsingEncoding(CPStringEncoding Encoding) const;

	/// @brief     Returns a data containg the string in the encoding Windows Latin 1. Data will have a C string terminator at the end.
	///
	/// @details   Returns a data containg the string in the encoding Windows Latin 1.
	///
	/// @return    A data with the contents of the string in the encoding Windows Latin 1.
	///
	CPData DataContainingWindowsLatin1CString(void) const;

	/// @brief     Returns a data containg the string in the encoding UNICODE 16. Data will have a C string terminator at the end.
	///
	/// @details   Returns a data containg the string in the encoding UNICODE 16.
	///
	/// @return    A data with the contents of the string in the encoding UNICODE 16.
	///
	CPData DataContainingUnicodeCString(void) const;

	/// @brief    Gives the double value of the string.
	///
	/// @return   Returns the double of the string object.
	///
	double DoubleValue(void) const;

	/// @brief    Gives the float value of the string.
	///
	/// @return   Returns the float of the string object.
	///
	float FloatValue(void) const;

	/// @brief    Gives the int value of the string.
	///
	/// @return   Returns the int of the string object.
	///
	int IntValue(void) const;

	/// @brief    Gives the unsigned int value of the string.
	///
	/// @return   Returns the unsigned int of the string object.
	///
	unsigned int UnsignedIntValue(void) const;

	/// @brief    Gives the bool value of the string.
	///
	/// @return   Returns the bool of the string object.
	///
	bool BoolValue(void) const;

	/// @brief    Gives the CPInteger of the string.
	///
	/// @return   Returns the CPInteger object.
	///
	CPInteger IntegerValue(void) const;

	/// @brief    Gives the CPUInteger of the string.
	///
	/// @return   Returns the CPUInteger object.
	///
	CPUInteger UIntegerValue(void) const;

	/// @brief    Gives the CPLongLong of the string.
	///
	/// @return   Returns the CPLongLong object.
	///
	CPLongLong LongLongValue(void) const;

	/// @brief    Gives the CPULongLong of the string.
	///
	/// @return   Returns the CPULongLong object.
	///
	CPULongLong ULongLongValue(void) const;

	/// @brief    Gives the lowercase of the string.
	///
	/// @return   Returns the lowercase string object.
	///
	CPString LowercaseString(void) const;

	/// @brief    Gives the uppercase of the string.
	///
	/// @return   Returns the uppercase string object.
	///
	CPString UppercaseString(void) const;

	/// @brief    Gives the lowercase of the string using the given locale.
	///
	/// @param	  LocaleIdentifier  The locale to use when converting to lower case.
	///
	/// @return   Returns the lowercase string object.
	///
	CPString LowercaseStringWithLocale(const CPLocaleIdentifier& LocaleIdentifier) const;

	/// @brief    Gives the uppercase of the string using the given locale.
	///
	/// @param	  LocaleIdentifier  The locale to use when converting to upper case.
	///
	/// @return   Returns the uppercase string object.
	///
	CPString UppercaseStringWithLocale(const CPLocaleIdentifier& LocaleIdentifier) const;

	/// @brief    Gives the capitalized of the string.
	///
	/// @return   Returns the capitalized string object.
	///
	CPString CapitalizedString(void) const;

	/// @brief    Gives the trimmed string.
	///
	/// @details  Gives the trimmed string after deleting the characters in the current string that are included in the given character set.
	///
	/// @return   Returns the trimmed string object.
	///
	CPString StringByTrimmingCharactersInSet(const CPCharacterSet& CharacterSet) const;

	/// @brief    Returns a new string by appending a path component.
	///
	/// @details  Returns a new string by appending a given path component string to the existing path string object.  The following
	///           table illustrates the effect of this method on a variety of different paths, assuming that "PathComponent" is
	///           supplied as “scratch.tiff”:
	///
	///           Existing String Value         Resulting String
	///           ------------------------      -------------------------
	///               “/tmp”                    “/tmp/scratch.tiff”
	///               “/tmp/”                   “/tmp/scratch.tiff”
	///               “/”                       “/scratch.tiff”
	///               "” (an empty string)      “scratch.tiff”
	///
	///	@param    PathComponent  A string of the path component to add.
	///
	/// @return   A new string made by appending PathComponent to the existing string object, preceded, if necessary, by a path separator.
	///
	CPString StringByAppendingPathComponent(const CPString& PathComponent) const;

	/// @brief    Returns a new string made by appending an extension separator followed by a given extension to the existing string.
	///
	/// @details  Returns a new string by appending a given path component string to the existing path string object.  The following
	///           table illustrates the effect of this method on a variety of different paths, assuming that ext is supplied as @"tiff":
	///
	///           Existing String Value         Resulting String
	///           ------------------------      -------------------------
	///               “/tmp/scratch.old”        “/tmp/scratch.old.tiff”
	///               “/tmp/scratch.”           “/tmp/scratch..tiff”
	///               “/tmp/”                   “/tmp.tiff”
	///               "scratch”                 “scratch.tiff”
	///
	///           Note that adding an extension to @"/tmp/" causes the result to be @"/tmp.tiff" instead of @"/tmp/.tiff". This difference
	///           is because a file named @".tiff" is not considered to have an extension, so the string is appended to the last nonempty
	///           path component.  This method does not allow you to append file extensions to filenames starting with the tilde
	///           character (~).  Note that this method only works with file paths (not, for example, string representations of URLs).
	///
	///	@param    PathExtensionIn  A string of the path extension to append to the existing string path.
	///
	/// @return   Returns a new string made by appending an extension separator followed by a given extension to the existing string.
	///
	CPString StringByAppendingPathExtension(const CPString& PathExtensionIn) const;

	/// @brief    Returns a new string made by deleting the last path component from the existing string, along with any final path
	///           separator.
	///
	/// @details  Returns a new string made by deleting the last path component from the existing string, along with any final path
	///           separator.  The following table illustrates the effect of this method on a variety of different paths:
	///
	///           Existing String Value         Resulting String
	///           ------------------------      -------------------------
	///               “/tmp/scratch.tiff”        “/tmp”
	///               “/tmp/lock/”               “/tmp”
	///               “/tmp/”                    “/”
	///               "/tmp”                     “/”
	///               "/”                        “/”
	///               "scratch.tiff”             "” (an empty string)
	///
	///           Note that this method only works with file paths (not, for example, string representations of URLs).
	///
	/// @return   A new string made by deleting the last path component from the existing string, along with any final path separator. If
	///           the existing string represents the root path it is returned unaltered.
	///
	CPString StringByDeletingLastPathComponent(void) const;

	/// @brief    Returns a new string made by deleting the extension (if any, and only the last) from the existing string.
	///
	/// @details  Returns a new string made by deleting the extension (if any, and only the last) from the existing string.  The following
	///           table illustrates the effect of this method on a variety of different paths:
	///
	///           Existing String Value         Resulting String
	///           ------------------------      -------------------------
	///               “/tmp/scratch.tiff”        “/tmp/scratch”
	///               “/tmp/”                    “/tmp”
	///               “scratch.bundle/”          “scratch”
	///               “scratch..tiff”            “scratch.”
	///               “.tiff”                    “.tiff”
	///               "/"                        "/"
	///
	///           Note that attempting to delete an extension from @".tiff" causes the result to be @".tiff" instead of an empty string.
	///           This difference is because a file named @".tiff" is not considered to have an extension, so nothing is deleted.  Note also
	///           that this method only works with file paths (not, for example, string representations of URLs).
	///
	/// @return   A new string made by deleting the extension (if any, and only the last) from the existing string. Strips any trailing path
	///           separator before checking for an extension. If the existing string represents the root path, it is returned unaltered.
	///
	CPString StringByDeletingPathExtension(void) const;

	/// @brief    Returns the last path component of the existing string.
	///
	/// @details  Returns the last path component of the existing string.  The following table illustrates the effect of this method on a
	///           variety of different paths:
	///
	///           Existing String Value         Resulting String
	///           ------------------------      -------------------------
	///               “/tmp/scratch.tiff”        “scratch.tiff”
	///               “/tmp/scratch”             “scratch”
	///               “/tmp/”                    “tmp”
	///               “scratch”                  “scratch”
	///               "/"                        "/"
	///
	///           Note that this method only works with file paths (not, for example, string representations of URLs).
	///
	/// @return   Returns the last path component of the existing string.
	///
	CPString LastPathComponent(void) const;

	/// @brief    Interprets the existing string as a path and returns the existing string’s extension, if any.
	///
	/// @details  Returns the existing string's extension, if any.  The following table illustrates the effect of this method on a variety
	///           of different paths:
	///
	///           Existing String Value         Resulting String
	///           ------------------------      -------------------------
	///               “/tmp/scratch.tiff”        “tiff”
	///               “/tmp/scratch”             "” (an empty string)
	///               “/tmp/”                    "” (an empty string)
	///               “/tmp/scratch..tiff”       "tiff”
	///
	///           Note that this method only works with file paths (not, for example, string representations of URLs).
	///
	/// @return   The existing string's extension, if any (not including the extension divider).
	///
	CPString PathExtension(void) const;

	/// @brief    Returns a new string made by expanding the initial component of the existing string to its full path value.
	///
	/// @details  Returns a new string made by expanding the initial component of the existing string to its full path value.  Note that
	///           this method only works with file paths (not, for example, string representations of URLs).
	///
	/// @return   A new string made by expanding the initial component of the existing string, if it begins with “~” or “~user”, to its full
	///           path value.  Returns a new string matching the existing string if the existing string’s initial component can’t be
	///           expanded.
	///
	CPString StringByExpandingTildeInPath(void) const;

	/// @brief    Returns a representation of the existing string path.
	///
	/// @details  Returns a representation of the existing string path using the encoding "NSUTF8StringEncoding" to determine the percent
	///           escapes necessary to convert the existing string into a legal URL string.
	///
	/// @return   Returns a representation of the existing string path using the encoding "NSUTF8StringEncoding" to determine the percent
	///           escapes necessary to convert the existing string into a legal URL string.  Returns nil if encoding cannot encode a
	///           particular character.
	///
	CPString StringByAddingPercentEscapes(void) const;

	/// @brief    Returns a new string made by replacing certain characters with the equivalent percent escape sequence using the
	///           given string of escape characters and based on the encoding kCFStringEncodingUTF8.
	///
	/// @details  Returns a new string made by replacing certain characters with the equivalent percent escape sequence using the
	///           given string of escape characters.  The characters escaped are all characters that are not legal URL characters (based on
	///           RFC 3986).  This will use kCFStringEncodingUTF8 as the encoding parameter.
	///
	/// @return   Returns a new string made by replacing certain characters.  If it does not need to be modified (no percent
	///           escape sequences are missing), this function may merely return the existing string with its reference count incremented.
	///
	CPString StringByAddingPercentEscapesSpecifyingEscapeCharacters(const CPString& CharactersToBeEscaped) const;

	/// @brief    Returns a new string made by replacing all percent escapes with the matching characters using UTF8 encoding.
	///
	/// @details  Returns a new string made by replacing all percent escapes with the matching characters using UTF8 encoding. Returns an
	///           empty string if the transformation is not possible, for example, the percent escapes give a byte
	///           sequence not legal in UTF8 encoding.
	///
	/// @return   A new string made by replacing all percent escapes with the matching characters using UTF8 encoding.
	///           Returns an empty string if the transformation is not possible, for example, the percent escapes give a byte sequence
	///           not legal in UTF8 encoding.
	///
	CPString StringByReplacingPercentEscapes(void) const;

	/// @brief    Returns a new string made by replacing all percent escapes with the matching characters as determined by a given encoding.
	///
	/// @details  Returns a new string made by replacing all percent escapes with the matching characters as determined by the given encoding.
	///           Returns an empty string if the transformation is not possible, for example, the percent escapes give a byte sequence not
	///           legal in encoding.
	///
	/// @return   A new string made by replacing all percent escapes with the matching characters as determined by the given encoding.
	///           Returns an empty string if the transformation is not possible, for example, the percent escapes give a byte sequence
	///           not legal in encoding.
	///
	CPString StringByReplacingPercentEscapesUsingEncoding(CPStringEncoding Encoding) const;

	/// @brief    Writes the contents to a file.
	///
	/// @details  Writes the contents of the existing string to a file at a given path using a given encoding.  This method overwrites any
	///           existing file at path.
	///
	///	@param    Path  The file to which to write the existing string.  If path contains a tilde (~) character, you must expand it with the
	///           method StringByExpandingTildeInPath before invoking this method.
	///	@param    Encoding  The encoding to use for the output.
	///	@param    Error  returns the error if one occured. [out, optional]
	///
	/// @return   A bool set to true if the file is written successfully, otherwise it is false.
	///
	bool WriteToFileUsingEncoding(const CPString& Path, CPStringEncoding Encoding, CPError& Error) const;
	bool WriteToFileUsingEncoding(const CPString& Path, CPStringEncoding Encoding) const;

	/// @brief    Writes the contents to a temp file then renames to given file.
	///
	/// @details  The existing string is written to an auxiliary file, and then the auxiliary file is renamed to the given path.  This
	///           guarantees that the given path, if it exists at all, won’t be corrupted even if the system should crash during writing.
	///
	///	@param    Path  The file to which to write the existing string.  If path contains a tilde (~) character, you must expand it with the
	///           method StringByExpandingTildeInPath before invoking this method.
	///	@param    Encoding  The encoding to use for the output.
	///	@param    Error  returns the error if one occured. [out, optional]
	///
	/// @return   A bool set to true if the file is written successfully, otherwise it is false.
	///
	bool WriteToFileAtomicallyUsingEncoding(const CPString& Path, CPStringEncoding Encoding, CPError& Error) const;
	bool WriteToFileAtomicallyUsingEncoding(const CPString& Path, CPStringEncoding Encoding) const;

#ifdef __OBJC__
	///	@brief     Creates a string object that is the same as the string passed in.
	///
	///	@details   This copy is accomplished by using the same underlying string object as the string passed in.
	///
	///	@param     String  A string to copy.
	///
	CPString(NSString* String);

	/// @brief     Assignment of a string object to another string object.
	///
	/// @details   Assigns a string object to another string object.
	///
	/// @param	   String  An underlying string object to copy.
	///
	const CPString& operator=(NSString* String);

	///	@brief     Conversion Operator.
	///
	///	@details   Returns the underlying object as a NSString*.
	///
	operator NSString*(void) const;

	/// @brief     Equals Operator.
	///
	/// @details   This checks to see if the string object is equal to the given string object.
	///
	/// @param	   String  A string object used in the compare.
	///
	/// @return    true if the string object is the same, otherwise it is false.
	///
	bool operator==(NSString* String) const;

	/// @brief     Not Equals Operator.
	///
	/// @details   This checks to see if the string object is not equal to the given string object.
	///
	/// @param	   String  A string object used in the compare.
	///
	/// @return    true if the string object is different, otherwise it is false.
	///
	bool operator!=(NSString* String) const;

	///	@brief     Conversion & Concatenation Operator.
	///
	///	@details   Concatenates the given string object to the end of the given string object.
	///
	///	@param     String1  The string object to concatenate to.
	///	@param     String2  The string object to add to String1.
	///
	friend CPString CP_EXTERNAL_API operator+(const CPString& String1, NSString* String2);

	///	@brief     Conversion & Concatenation Operator.
	///
	///	@details   Concatenates the given string object to the end of the given string object.
	///
	///	@param     String1  The string object to concatenate to.
	///	@param     String2  The string object to add to String1.
	///
	friend CPString CP_EXTERNAL_API operator+(NSString* String1, const CPString& String2);
#endif
#ifdef __COREFOUNDATION__
	/// @brief     Copy Constructor.
	///
	/// @details   This object will contain the object of the specified string.
	///
	///	@param     String  the string to copy.
	///
	CPString(CFStringRef String);

	/// @brief     Assignment of a string object to another string object.
	///
	/// @details   Assigns a string object to another string object.
	///
	/// @param	   String  An underlying string object to copy.
	///
	const CPString& operator=(CFStringRef String);

	///	@brief     Conversion Operator.
	///
	///	@details   Returns the underlying object as a CFStringRef*.
	///
	operator CFStringRef(void) const;

	/// @brief     Equals Operator.
	///
	/// @details   This checks to see if the string object is equal to the given string object.
	///
	/// @param	   String  A string object used in the compare.
	///
	/// @return    true if the string object is the same, otherwise it is false.
	///
	bool operator==(CFStringRef String) const;

	/// @brief     Not Equals Operator.
	///
	/// @details   This checks to see if the string object is not equal to the given string object.
	///
	/// @param	   String  A string object used in the compare.
	///
	/// @return    true if the string object is different, otherwise it is false.
	///
	bool operator!=(CFStringRef String) const;

	///	@brief     Conversion & Concatenation Operator.
	///
	///	@details   Concatenates the given string object to the end of the given string object.
	///
	///	@param     String1  The string object to concatenate to.
	///	@param     String2  The string object to add to String1.
	///
	friend CPString CP_EXTERNAL_API operator+(const CPString& String1, CFStringRef String2);

	///	@brief     Conversion & Concatenation Operator.
	///
	///	@details   Concatenates the given string object to the end of the given string object.
	///
	///	@param     String1  The string object to concatenate to.
	///	@param     String2  The string object to add to String1.
	///
	friend CPString CP_EXTERNAL_API operator+(CFStringRef String1, const CPString& String2);
#endif

	friend class CPObject;

protected:
	CPString(const CPId& OtherString, CPClassType ClassType);
	CPString(const CPObject& OtherObject, CPClassType ClassType);

private:
	void AssignUnderlyingString(const CPId& String);
	void AssignUnderlyingStringAndTakeOwnership(const CPId& String);

#ifdef _WINDOWS
	static unsigned long WindowsLocaleID(CPLocaleIdentifier LocaleIdentifier);
#endif

	CPData* m_UTF8Buffer;
	CPData* m_DefSingleByteBuffer;
	CPData* m_UnicodeBuffer;
};

#ifdef __OBJC__
#define STR(s) @s
#define CPSTR(s) ((CPString)@s)
#else
#if __COREFOUNDATION__
#define STR(s) CFSTR(s)
//#define STR(s) L##s
#else
#define STR(s) s
#endif
#ifdef _UNICODE
//#define STR(s) s
#define CPSTR(s) ((CPString)s)
//CPString::StringWithCStringUsingEncoding(STR(s), CPUnicodeStringEncoding)
//#define CPSTR(s) CPString(STR(s))
#else
//#define STR(s) s
//#ifdef _MBCS
#define CPSTR(s) CPString(STR(s))
//#define CPSTR(s) CPString::StringWithUnicodeConstantString(L##s)
//#else
//#define CPSTR(s) CPString::StringWithCStringUsingEncoding(s, CPWindowsLatin1StringEncoding)
//#endif
#endif
#endif
