////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPTimeZone.h
//      Time zones for date manipulation.
//
//  Copyright © 2015 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  04/11/15  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/ExternalAPIMacros.h"
#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/CPObject.h"

/// @brief   Time zone objects represent geopolitical regions. Consequently, these objects have names for these regions. Time zone objects
///          also represent a temporal offset, either plus or minus, from Greenwich Mean Time (GMT).
///
/// @details CPTimeZone provides two class methods to get time zone objects: GMTTimeZone and CurrentTimeZone.
///
///          Notes: CPTimeZone uses NSTimeZone or its Core Foundation counterpart, CFTimeZone Reference to provide most of the functionality.
///          NSTimeZone and CFTimeZone are bridged Foundation objects. Therefore, in a method where you see an NSTimeZone* parameter, you can
///          pass a CFTimeZoneRef, and in a function where you see a CFTimeZoneRef parameter, you can pass an NSTimeZone instance.
///
/// For more information:
///http://developer.apple.com/library/mac/#documentation/Cocoa/Reference/Foundation/Classes/NSTimeZone_Class
///http://developer.apple.com/library/mac/documentation/CoreFoundation/Reference/CFTimeZoneRef
///
class CP_EXTERNAL_API CPTimeZone : public CPObject
{
public:
	///	@brief     A class type that uniquely identifies this class.
	///
	///	@return    Returns a class type that uniquely identifies this class.
	///
	static CPClassType ClassType(void);

	/// @brief    Returns the underlying shared empty object.
	///
	/// @details  Static method that returns the underlying object that is shared between all empty objects of this class type.
	///
	/// @returns  the underlying shared empty object.
	///
	static const CPId& SharedEmptyObject(void);

	/// @brief    Returns a time zone for the GMT time zone.
	///
	/// @details  Static method that returns a shared GMT time zone.
	///
	/// @returns  the GMT time zone.
	///
	static const CPTimeZone& GMTTimeZone(void);

	/// @brief    Returns a time zone for the current time zone.
	///
	/// @details  Static method that returns a shared current time zone.
	///
	/// @returns  the current time zone.
	///
	static const CPTimeZone& CurrentTimeZone(void);

	/// @brief     Default Constructor.
	///
	/// @details   Creates an empty time zone.
	///
	CPTimeZone(void);

	/// @brief     Copy Constructor.
	///
	/// @details   This object will contain the object of the specified time zone.
	///
	///	@param     TimeZone  the time zone to copy.
	///
	CPTimeZone(const CPTimeZone& TimeZone);

	///	@brief     Destructor.
	///
	///	@details   deallocates any unused resources.
	///
	~CPTimeZone(void);

	/// @brief     Assignment of a time zone object to another time zone object.
	///
	/// @details   Assigns a time zone object to another time zone object.
	///
	/// @param	   TimeZone  An underlying time zone object to copy.
	///
	const CPTimeZone& operator=(const CPTimeZone& TimeZone);

	/// @brief     Is this time zone a GMT time zone.
	///
	/// @details   True if this time zone is a GMT time zone.
	///
	/// @return	   True if the time zone is a GMT time zone.
	///
	bool IsGMT(void) const;
	
	/// @brief     The number of seconds difference between time in this time zone and GMT at this point in time.
	///
	/// @details   The number of seconds difference between time in this time zone and GMT at this point in time. This value will not be the
	///            same on a daylight savings time day as a daylight standard time day.
	///
	/// @return	   The number of seconds difference between time in this time zone and GMT.
	///
	CPDouble SecondsFromGMT(void) const;
	
	/// @brief     The number of seconds difference between time in this time zone and GMT at the point in time specified.
	///
	/// @details   The number of seconds difference between time in this time zone and GMT at the point in time specified. This value will not
	///            be the same on a daylight savings time date as a daylight standard time date.
	///
	/// @param     Date  The date on which to get the time difference between this time zone and GMT.
	///
	/// @return	   The number of seconds difference between time in this time zone and GMT.
	///
	CPDouble SecondsFromGMTOnDate(const CPDate& Date) const;

#if __OBJC__
	///	@brief     Creates a time zone object that is the same as the time zone passed in.
	///
	///	@details   This copy is accomplished by using the same underlying time zone object as the time zone passed in.
	///
	///	@param     TimeZone  A time zone to copy.
	///
	CPTimeZone(NSTimeZone* TimeZone);

	/// @brief     Assignment of a time zone object to another time zone object.
	///
	/// @details   Assigns a time zone object to another time zone object.
	///
	/// @param	   TimeZone  An underlying time zone object to copy.
	///
	const CPTimeZone& operator=(NSTimeZone* TimeZone);

	///	@brief     Conversion Operator.
	///
	///	@details   Returns the underlying object as a NSTimeZone*.
	///
	operator NSTimeZone*(void) const;
#endif
#if __COREFOUNDATION__
	/// @brief     Copy Constructor.
	///
	/// @details   This object will contain the object of the specified time zone.
	///
	///	@param     TimeZone  the time zone to copy.
	///
	CPTimeZone(CFTimeZoneRef TimeZone);

	///	@brief     Copy Operator.
	///
	///	@details   Copies the entries from the given time zone.
	///
	///	@param	   TimeZone  the time zone to copy.
	///
	const CPTimeZone& operator=(CFTimeZoneRef TimeZone);

	///	@brief     Conversion Operator.
	///
	///	@details   Returns the underlying object as a CFTimeZoneRef.
	///
	operator CFTimeZoneRef(void) const;
#endif

	friend class CPObject;

protected:
	explicit CPTimeZone(const CPId& Id);

private:
	static CPTimeZone m_GMTTimeZone;
	static CPTimeZone m_CurrentTimeZone;
};
