////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPURL.h
//    Cross-platform URL class.
//
//  Copyright © 2009 MYOB US, Inc., all rights reserved.
//
//  Change History (most recent first):
//
//	10/20/09  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/CPObject.h"
#include "CrossPlatform/CPString.h"

/// @brief   Cross platform URL class.
///
/// @details A CPURL object represents a URL that can potentially contain the location of a resource on a remote server, the path of a local
///          file on disk, or even an arbitrary piece of encoded data. You can use URL objects to construct URLs and access their parts. For
///          URLs that represent local files, you can also manipulate properties of those files directly, such as changing the file’s last
///          modification date. Finally, you can pass URL objects to other APIs to retrieve the contents of those URLs. URL objects are the
///          preferred way to refer to local files. Most objects that read data from or write data to a file have methods that accept an CPURL
///          object instead of a pathname as the file reference. For example, you can get the contents of a local file URL as an CPString
///          object by calling the StringWithContentsOfURL method, or as an CPData object by calling the DataWithContentsOfURL method.
///          CPURL methods were not added to CrossPlatform originally but should and will be added in the future.
///
/// For more information:
/// http://developer.apple.com/library/mac/#documentation/Cocoa/Reference/Foundation/Classes/NSURL_Class
/// http://developer.apple.com/library/mac/#documentation/CoreFoundation/Reference/CFURLRef
///
class CP_EXTERNAL_API CPURL : public CPObject
{
public:
	///	@brief     A class type that uniquely identifies this class.
	///
	///	@return    Returns a class type that uniquely identifies this class.
	///
	static CPClassType ClassType(void);

	/// @brief    Returns the underlying shared empty object.
	///
	/// @details  Static method that returns the underlying object that is shared between all empty objects of this class type.
	///
	/// @returns  the underlying shared empty object.
	///
	static const CPId& SharedEmptyObject(void);

	/// @brief    Returns a URL with the specified file path.
	///
	/// @details  Returns a URL with the specified file path. This file path is a path in the form that the OS would specify it.
	///
	/// @returns  Returns a URL with the specified file path.
	///
	static CPURL URLFromFilePath(const CPString& FilePath);

	/// @brief    Returns a file URL with the specified path.
	///
	/// @returns  Returns a file URL with the specified path.
	///
	static CPURL FileURLWithPath(const CPString& FilePath);

	/// @brief    Returns a URL with the specified absolute path.
	///
	/// @details  Returns a URL with the specified absolute path. A file path string is not a valid absolute url to a file. A valid absolute
	///           path to a file needs the file scheme "file//" as part of the path.
	///
	/// @returns  Returns a URL with the specified absolute path.
	///
	static CPURL URLWithAbsolutePath(const CPString& AbsolutePath);

	CPURL(void);
	CPURL(const CPURL& URL);
	//TODO: KLS put explicit on all string constuctors below
	CPURL(const CPString& AbsoluteURL);
	CPURL(const char* AbsoluteURL);
	CPURL(const unsigned char* AbsoluteURL);
	CPURL(const CPUniChar* AbsoluteURL);
	CPURL(const CPURL& BaseURL, const CPString& RelativeURL);
	explicit CPURL(const CPId& URL);

	~CPURL(void);

	const CPURL& operator=(const CPURL& URL);
	const CPURL& operator=(const CPString& URL);

	const bool operator==(const CPURL& URL);
	const bool operator!=(const CPURL& URL);

	// Path routines
	CPURL URLByAppendingPathComponent(const CPString& PathComponent) const;
	CPURL URLByAppendingPathExtension(const CPString& PathComponent) const;
	CPURL URLByDeletingLastPathComponent(void) const;
	CPURL URLByDeletingPathExtension(void) const;

	CPURL AbsoluteURL(void) const;
	CPURL BaseURL(void) const;
	CPString AbsoluteString(void) const;
	CPString RelativeString(void) const;
	CPString RelativePath(void) const;
	CPString Path(void) const;
	CPString LastPathComponent(void) const;
	CPString PathExtension(void) const;

	///	@brief     Returns a new file path URL that points to the same resource as the original URL.
	///
	///	@details   Returns a new file path URL that points to the same resource as the original URL. If the original URL is a file reference
	///            URL, this method converts it to a URL that contains a file system path. If the original URL is a file path URL, the returned
	///            URL is identical. If the original URL is not a file URL, or if the resource is not reachable or no longer exists, this
	///            method returns an object that contains null.
	///
	///	@return    Returns a new file path URL that points to the same resource as the original URL.
	///
	CPURL FilePathURL(void) const;

	///	@brief     Returns a new file reference URL that points to the same resource as the original URL.
	///
	///	@details   Returns a new file reference URL that points to the same resource as the original URL. This form of file URL remains valid
	///            when the file system path of the URL’s underlying resource changes.
	///            If the original URL is a file path URL, this method converts it to a file reference URL. If the original URL is a file
	///            reference URL, the returned URL is identical. If the original URL is not a file URL, this method returns object that contains
	///            null. File reference URLs cannot be created to file system objects which do not exist or are not reachable. If you try, this
	///            method returns null object. In some areas of the file system hierarchy, file reference URLs cannot be generated to the leaf
	///            node of the URL path.
	///            Important: A file reference URL's path should never be persistently stored, because it is not valid across system restarts
	///            or remounts of volumes.
	///
	///	@return    Returns a new file reference URL that points to the same resource as the original URL.
	///
	CPURL FileReferenceURL(void) const;

	///	@brief     Returns whether the receiver uses the file scheme.
	///
	///	@details   Returns whether the receiver uses the file scheme. Both file path and file reference URLs are considered to be file URLs.
	///            if myURL.IsFileURL() is true, then myURL.Path() is suitable for input into CPFileManager.
	///
	///	@return    Returns true if the receiver uses the file scheme, false otherwise.
	///
	bool IsFileURL(void) const;

	///	@brief     Returns whether the receiver uses the file reference scheme.
	///
	///	@details   Returns whether the receiver uses the file reference scheme. if myURL.IsFileRefernceURL() is true, then myURL.Path() is
	///            suitable for input into CPFileManager.
	///
	///	@return    Returns true if the receiver uses the file reference scheme, false otherwise.
	///
	bool IsFileReferenceURL(void) const;

#ifdef __OBJC__
	// Define methods that will be needed only when bridging to Cocoa
	CPURL(NSURL* Url);

	const CPURL& operator=(NSURL* Url);

	operator NSURL*() const;

#endif  // #ifdef __OBJC__
#if __COREFOUNDATION__
	// Define methods that will be needed only when bridging to CoreFoundation
	CPURL(CFURLRef Value);
	const CPURL& operator=(CFURLRef Value);

	operator CFURLRef(void) const;
#endif
};
