////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPValue.h
//    Cross-platform Value class.
//
//  Copyright © 2009-2012 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//  02/10/12  K.L.S.  Changes for CrossPlatform API changes. Reworked header includes.
//	06/02/09  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/ExternalAPIMacros.h"
#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/CPObject.h"

class CPString;

#ifdef __OBJC__
#define CP_ENCODE(type) @encode(type)
#else
#define CP_ENCODE(type) CPSTR(#type)
#endif


/// @brief    Class encapsulation of a value.
/// @details  A CPValue object is a simple container for a single C or Objective-C data item. It can hold any of the scalar types such 
///           as int, float, and char, as well as pointers, structures, and object ids. The purpose of this class is to allow items of such
///           data types to be added to collections such as an instance of CPArray, which requires its elements to be objects. CPValue 
///           objects are always immutable.
///
/// For more information:
/// http://developer.apple.com/library/mac/#documentation/Cocoa/Reference/Foundation/Classes/NSValue_Class
///
class CP_EXTERNAL_API CPValue : public CPObject
{
public:
	///	@brief     A class type that uniquely identifies this class.
	///	@return    Returns a class type that uniquely identifies this class.
	static CPClassType ClassType(void);

	/// @brief    Returns the underlying shared empty object.
	/// @details  Static method that returns the underlying object that is shared between all empty objects of this class type.
	/// @returns  the underlying shared empty object.
	static const CPId& SharedEmptyObject(void);

	/// @brief     Creates a value from a given set of bytes of a given type.
	/// details    Creates and returns a value object that contains a given value, which is interpreted as being of a given Obj-C type.
	/// @param[out] Bytes  The pointer to the value for the new value object.
	/// @param     CType  Types include int, unsigned int, const void*, void*, long, unsigned long, long long, unsigned long long, short, 
	///            unsigned short, char, unsigned char, double, float, & bool.
	/// @return    A new value object that contains value, which is interpreted as being of the Obj-C type.
	static CPValue ValueWithBytesOfCType(const void* Bytes, const CPString& CType);
	
	/// @brief     Creates and returns a value object that contains a given pointer.
	/// @details   This method does not copy the contents of Pointer, so you must not deallocate the memory at the pointer destination
	///            while the value object exists. Data objects may be more suited for arbitrary pointers than value objects.
	/// @param     Pointer  A pointer to a value of the new object.
	/// @return    A new value object that contains pointer.
	static CPValue ValueWithPointer(const void* Pointer);

	static CPValue ValueFromObject(const CPObject& Object);
	static CPValue ValueFromId(const CPId& Object);

	/// @brief     Default Constructor.
	/// @details   Creates an empty value.
	CPValue(void);
	
	/// @brief     Creates a value object containing the value passed in.
	/// @details   This object contains the value that was passed into the constructor.
	///	@param     Value  the value to copy.
	CPValue(const CPValue& Value);
	
	/// @brief     Creates a value object containing the object passed in.
	/// @details   This object contains the object that was passed into the constructor.
	///	@param     Value  the object to copy.
	explicit CPValue(const CPObject& Value);
	
	/// @brief     Copy Constructor.
	/// @details   This object contains the value of the id that was passed into the constructor.
	///	@param     Value  the id to copy.
	explicit CPValue(const CPId& Value);
	
	///	@brief     Destructor.
	///	@details   Removes any references it has to shared resources.
	~CPValue(void);
	
	///	@brief     Copy Operator.
	///	@details   Copies the value from the given value object.
	///	@param	   Value  the value to copy.	
	const CPValue& operator=(const CPValue& Value);

	/// @brief     Initializes a value from a given set of bytes of a given type.
	/// details    Initializes and returns a value object that contains a given value, which is interpreted as being of a given Obj-C type. 
	/// @param[out] Bytes  The pointer to the value for the new value object.
	/// @param     CType  Types include int, unsigned int, const void*, void*, long, unsigned long, long long, unsigned long long, short, 
	///            unsigned short, char, unsigned char, double, float, & bool.
	void InitWithBytesOfCType(const void* Bytes, const CPString& CType);

	/// @brief     Initializes and returns a value object that contains a given pointer.
	/// @details   This method does not copy the contents of Pointer, so you must not deallocate the memory at the pointer destination
	///            while the value object exists. Data objects may be more suited for arbitrary pointers than value objects.
	/// @param     Pointer  A pointer to a value of the new object.
	void InitWithPointer(const void* Pointer);
	
	///	@brief     Retrieves the value as a pointer to a void.
	///	@return    A pointer to a void of the value object.
	void* PointerValue(void) const;
	
	///	@brief     Retrieves the value as a pointer to a void.
	///	param[out] A pointer to a void of the value object.
	void GetValue(void* ReturnedValue) const;
	
	/// @brief     Get the type of the value.
	/// @details   Since the value object can hold many different types, this will give the type of value contained in the object.
	/// @return    Types of returns are int, unsigned int, const void*, void*, long, unsigned long, long long, unsigned long long, short, 
	///            unsigned short, char, unsigned char, double, float, & bool.
	CPString CType(void) const;

#ifdef __OBJC__
	/// @brief     Copy Constructor.
	/// @details   This object will contain the object of the specified value.
	///	@param     Value  the value to copy.
	CPValue(NSValue* Value);
	
	///	@brief     Copy Operator.
	///	@details   Copies the value from the given value object.
	///	@param	   Value  the value to copy.	
	const CPValue& operator=(NSValue* Value);
	
	///	@brief     Conversion Operator.
	///	@details   Returns the underlying object as a NSValue*.
	operator NSValue*(void) const;
#endif

protected:
};

#include "CrossPlatform/CPString.h"
