////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPVersion.h
//    Cross platform Operating system version class.
//
//  Copyright © 2015 Acclivity Group LLC, all rights reserved.
//
//  Change History (most recent first):
//
//	01/13/15  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/ExternalAPIMacros.h"
#include "CrossPlatform/CPOS_API.h"
#include "CrossPlatform/CPInteger.h"
#include "CrossPlatform/CPString.h"

/// @brief   A version class is a 3-tuple(triplet) identifier.
///
/// @details Class holds major, minor, and patch version numbers and allows comparing them. A version is a 3-tuple(triplet) identifier in the
///          form of (majorVersion, minorVersion, patchVersion). So an example could be OSX 10.9.1, wich would be major version 10, minor
///          version 9, and patch version 1. (10.9.1)
///
class CP_EXTERNAL_API CPVersion
{
public:
	/// @brief   Creates an version object with the major and minor version passed in.
	///
	///	@param	 MajorVersion  the major version.
	///	@param	 MinorVersion  the minor version.
	///
	/// @return  A verion with the major and minor version set. (majorVersion, minorVersion, 0)
	///
	static CPVersion VersionWithMajorAndMinorVersion(CPInteger MajorVersion, CPInteger MinorVersion);

	/// @brief   Creates an version object with the major, minor, and patch version passed in.
	///
	///	@param	 MajorVersion  the major version.
	///	@param	 MinorVersion  the minor version.
	///	@param	 PatchVersion  the patch version.
	///
	/// @return  A verion with the major minor and patch version set. (MajorVersion, MinorVersion, PatchVersion)
	///
	static CPVersion VersionWithMajorMinorAndPatchVersion(CPInteger MajorVersion, CPInteger MinorVersion, CPInteger PatchVersion);

	/// @brief   Creates an version object with the version string passed in.
	///
	///	@details Creates an version object with the major, minor, and patch version pulled from the version string passed in. It assumes the
	///          values are separated by a period character ".". It will take up to 3 values from the string and assign them to the version.
	///
	///	@param	 VersionString  the version string to pull the version numbers from.
	///
	/// @return  A verion with the major minor and patch version set from values within the version string.
	///
	static CPVersion VersionWithVersionString(const CPString& VersionString);

	/// @brief   Default Constructor.
	///
	/// @details Creates an empty version object.
	///
	CPVersion(void);

	///	@brief   Destructor.
	///
	///	@details Deallocates the object.
	///
	~CPVersion(void);

	/// @brief   Constructor.
	///
	/// @details Creates an version object with the major and minor version passed in.
	///
	///	@param	 MajorVersion  the major version.
	///	@param	 MinorVersion  the minor version.
	///
	CPVersion(CPInteger MajorVersion, CPInteger MinorVersion);

	/// @brief   Constructor.
	///
	/// @details Creates an version object with the major, minor, and patch version passed in.
	///
	///	@param	 MajorVersion  the major version.
	///	@param	 MinorVersion  the minor version.
	///	@param	 PatchVersion  the patch version.
	///
	CPVersion(CPInteger MajorVersion, CPInteger MinorVersion, CPInteger PatchVersion);

	/// @brief   Copy Constructor.
	///
	/// @details Creates an version object that is equal to the one passed in.
	///
	///	@param	 Version  the version to copy.
	///
	CPVersion(const CPVersion& Version);

	///	@brief   Copy Operator.
	///
	///	@details Copies the version information.
	///
	///	@param	 Version  the version to copy.
	///
	const CPVersion& operator=(const CPVersion& Version);

	/// @brief   Equals Operator.
	///
	/// @details This checks to see if the object is equal to the given object.
	///
	/// @param	 Version  the version object used in the compare.
	///
	/// @return  Returns true if the object is the same, otherwise it is false.
	///
	bool operator==(const CPVersion& Version) const;

	/// @brief   Not Equals Operator.
	///
	/// @details This checks to see if the object is not equal to the given object.
	///
	/// @param	 Version  the version object used in the compare.
	///
	/// @return  Returns true if the object is not the same, otherwise it is false.
	///
	bool operator!=(const CPVersion& Version) const;

	/// @brief   greater than Operator.
	///
	/// @details This checks to see if the object greater than the given object.
	///
	/// @param	 Version  the version object used in the compare.
	///
	/// @return  Returns true if the object is greater than, otherwise it is false.
	///
	bool operator>(const CPVersion& Version) const;

	/// @brief   Less than Operator.
	///
	/// @details This checks to see if the object less than the given object.
	///
	/// @param	 Version  the version object used in the compare.
	///
	/// @return  Returns true if the object is less than, otherwise it is false.
	///
	bool operator<(const CPVersion& Version) const;

	/// @brief   Greater than equal to Operator.
	///
	/// @details This checks to see if the object greater than or equal to the given object.
	///
	/// @param	 Version  the version object used in the compare.
	///
	/// @return  Returns true if the object is greater than or equal to, otherwise it is false.
	///
	bool operator>=(const CPVersion& Version) const;

	/// @brief   Less than equal to Operator.
	///
	/// @details This checks to see if the object less than or equal to the given object.
	///
	/// @param	 Version  the version object used in the compare.
	///
	/// @return  Returns true if the object is less than or equal to, otherwise it is false.
	///
	bool operator<=(const CPVersion& Version) const;

	/// @brief   The major version number of the version.
	///
	/// @details The major version number of the is the most significant number of the version. If the version is 10.9.1, then the 10 is the
	///          major version value.
	///
	/// @return  Returns the major version number.
	///
	CPInteger MajorVersion(void) const;

	/// @brief   The minor version number of the version.
	///
	/// @details The minor version number of the is the second most significant number of the version. If the version is 10.9.1, then the 9
	///          is the minor version value.
	///
	/// @return  Returns the minor version number.
	///
	CPInteger MinorVersion(void) const;

	/// @brief   The patch version number of the version.
	///
	/// @details The patch version number of the is the least significant number of the version. If the version is 10.9.1, then the 1 is the
	///          patch version value.
	///
	/// @return  Returns the patch version number.
	///
	CPInteger PatchVersion(void) const;

	/// @brief   The version in a string.
	///
	/// @details The version in a string with the format "majorVersion.minorVersion.patchVersion". An example if the version is 10.9.1, then
	///          the string would be "10.9.1"
	///
	/// @return  Returns the version string.
	///
	CPString VersionString(void) const;

private:
	CPInteger m_MajorVersion;
	CPInteger m_MinorVersion;
	CPInteger m_PatchVersion;
};
